@@ -21,8 +21,6 @@
 /* GpEPSEngine declared in eps.h */
 #include "eps.h"
 
-extern long strtol(const char *, char **, int);
-
 extern int cgmScaleToFit;
 extern int gtDoEscapes;
 
@@ -68,7 +66,6 @@ static int FindDevice(void);
 static int SaveName(int device, char *name);
 static void DoSpecial(int nPrefix, int cSuffix);
 
-static int HandleInput(FILE *file, void *context);
 static int xPrefix= 0;
 static void HandleExpose(Engine *engine, Drauing *drauing, int xy[]);
 static void HandleOther(Engine *engine, int k, int md);
@@ -113,6 +110,7 @@ on_launch(int argc, char *argv[])
   p_quitter(&on_quit);
   p_idler(&on_idle);
   p_stdinit(&on_stdin);
+  g_stdout = p_stdout;
 
   for (i=1 ; i<argc ; i++) {
     arg= argv[i];
@@ -121,43 +119,57 @@ on_launch(int argc, char *argv[])
       int fileType= -1;
       arg++;
       if (strcmp(arg, "cgm")==0) {
-	fileType= 0;
-	i++;
-	if (i>=argc) return MessageAndExit("Missing file or display name");
-	arg= argv[i];
+        fileType= 0;
+        i++;
+        if (i>=argc) return MessageAndExit("Missing file or display name");
+        arg= argv[i];
       } else if (strcmp(arg, "ps")==0) {
-	fileType= 1;
-	i++;
-	if (i>=argc) return MessageAndExit("Missing file or display name");
-	arg= argv[i];
+        fileType= 1;
+        i++;
+        if (i>=argc) return MessageAndExit("Missing file or display name");
+        arg= argv[i];
       } else if (strcmp(arg, "in")==0) {
-	i++;
-	if (i>=argc) return MessageAndExit("Missing file or display name");
-	if (inName) return HelpAndExit();
-	else inName= argv[i];
+        i++;
+        if (i>=argc) return MessageAndExit("Missing file or display name");
+        if (inName) return HelpAndExit();
+        else inName= argv[i];
       } else if (strcmp(arg, "display")==0 || strcmp(arg, "d")==0) {
-	fileType= 2;
-	i++;
-	if (i>=argc) return MessageAndExit("Missing file or display name");
-	arg= argv[i];
+        fileType= 2;
+        i++;
+        if (i>=argc) return MessageAndExit("Missing file or display name");
+        arg= argv[i];
       } else if (strcmp(arg, "f")==0) {
-	amBatch= 1;
-	fileType= 1;
-	arg= "*stdout*";
+        amBatch= 1;
+        fileType= 1;
+        arg= "*stdout*";
       } else if (strcmp(arg, "nd")==0) noDisplay= 1;
       else if (strcmp(arg, "b")==0) amBatch= 1;
       else if (strcmp(arg, "nowarn")==0) no_warnings= 1;
-      else if (strcmp(arg, "75")==0) defaultDPI= 75;
+      else if (strcmp(arg, "geometry")==0) {
+        char *suffix;
+        int w=0,h=0;
+        i++;
+        if (i>=argc) MessageAndExit("Missing geometry");
+        arg = argv[i];
+        w = (int)strtol(arg, &suffix, 10);
+        if (suffix!=arg && *suffix=='x') {
+          arg = suffix+1;
+          h = (int)strtol(arg, &suffix, 10);
+        }
+        if (w < 10 || h < 10) MessageAndExit("Invalid geometry");
+        gx75width = gx100width = w;
+        gx75height = gx100height = h;
+      } else if (strcmp(arg, "75")==0) defaultDPI= 75;
       else if (strcmp(arg, "100")==0) defaultDPI= 100;
       else if (strcmp(arg, "gks")==0) {
-	gx75width= gx75height= 600;     /* 8x8 X window size */
-	gx100width= gx100height= 800;   /* 8x8 X window size */
-	cgmScaleToFit= 1;               /* 8x8 PostScript plotting area */
-	gtDoEscapes= 0;
+        gx75width= gx75height= 600;     /* 8x8 X window size */
+        gx100width= gx100height= 800;   /* 8x8 X window size */
+        cgmScaleToFit= 1;               /* 8x8 PostScript plotting area */
+        gtDoEscapes= 0;
       } else if (strcmp(arg, "x")==0) x_only= 1;
       else if (strcmp(arg, "gks")==0) {
-	cgmScaleToFit= 1;
-	gtDoEscapes= 0;
+        cgmScaleToFit= 1;
+        gtDoEscapes= 0;
       }
       else if (strcmp(arg, "fmbug")==0) epsFMbug= 1;
       else if (strcmp(arg, "bg0fg1")==0) bg0fg1= 1;
@@ -166,13 +178,13 @@ on_launch(int argc, char *argv[])
       else return HelpAndExit();
 
       if (fileType>=0) {
-	if (nOut>=8)
-	  return MessageAndExit("At most 8 output files/displays allowed");
-	for (j=0 ; j<nOut ; j++) if (strcmp(outNames[j], arg)==0)
-	  return MessageAndExit("Duplicate output filenames not allowed");
-	outNames[nOut]= arg;
-	outTypes[nOut]= fileType;
-	nOut++;
+        if (nOut>=8)
+          return MessageAndExit("At most 8 output files/displays allowed");
+        for (j=0 ; j<nOut ; j++) if (strcmp(outNames[j], arg)==0)
+          return MessageAndExit("Duplicate output filenames not allowed");
+        outNames[nOut]= arg;
+        outTypes[nOut]= fileType;
+        nOut++;
       }
 
     } else if (arg[0]<='9' && arg[0]>='0') {
@@ -216,7 +228,7 @@ on_launch(int argc, char *argv[])
     }
   }
 
-  if (!noDisplay) g_initializer(&argc, argv);
+  g_initializer(&argc, argv);
 
   return 0;
 }
@@ -234,8 +246,8 @@ do_startup(void)
     }
     if (noX && nOut<8) {
       if (!CreateX(nOut, 0)) {
-	nOut++;
-	noX= 0;
+        nOut++;
+        noX= 0;
       }
     }
     noDisplay= noX;
@@ -255,10 +267,13 @@ static int need_prompt = 1;
 int
 on_idle(void)
 {
+  int flag;
   if (!did_startup && do_startup()) return 0;
-  if (need_prompt && !x_only) p_stdout("gist> ");
+  if (need_prompt && !x_only && !amBatch) p_stdout("gist> ");
   need_prompt = 0;
-  return CatalogCGM();
+  flag = CatalogCGM();
+  if (!flag && amBatch) p_quit();
+  return flag;
 }
 
 int
@@ -284,12 +299,13 @@ static int HelpAndExit(void)
   p_stderr("             -cgm cgmout   (open cgmout for output)\n");
   p_stderr("             -ps psout     (open psout for output)\n");
   p_stderr("             -display host:n.m  (connect to X server)\n");
-  p_stderr("             -75  (X windows default to 75 dpi)\n");
-  p_stderr("             -100 (X windows default to 100 dpi)\n");
-  p_stderr("             -gks (X windows default to 8x8 inches)\n");
+  p_stderr("             -75  (default to 75 dpi)\n");
+  p_stderr("             -100 (default to 100 dpi)\n");
+  p_stderr("             -gks (default to 8x8 inches)\n");
+  p_stderr("             -geometry WxH (initial window size in pixels)\n");
   p_stderr("             -nd  (not even default X server)\n");
   p_stderr("             -b   (batch mode, implies -nd)\n");
-  p_stderr("             -x   (X window only, no keyboard)\n");
+  p_stderr("             -x   (graphics window only, no keyboard)\n");
   p_stderr("             -nowarn (only one warning message printed)\n");
   p_stderr("             -f   (PostScript to stdout, implies -b)\n");
   p_stderr("             -fmbug (if EPS files are for FrameMaker)\n");
@@ -358,7 +374,7 @@ static int GetCommand(char *token)
     nPrefix= (int)strtol(now, &suffix, 10);
     cSuffix= *suffix;
     if ((cSuffix=='f' || cSuffix=='b' || cSuffix=='g') &&
-	*(suffix+1)=='\0') {
+        *(suffix+1)=='\0') {
       while (*command) command++;
       return command-commandList;
     }
@@ -385,10 +401,10 @@ static int GetCommand(char *token)
     if (cmd && *now++==*cmd++) {
       while (*cmd && *now==*cmd) { now++;  cmd++; }
       if (!*now) {
-	p_stderr("gist: Ambiguous command: ");
-	p_stderr(token);
-	p_stderr("\n      Type help for help.\n");
-	return -1;
+        p_stderr("gist: Ambiguous command: ");
+        p_stderr(token);
+        p_stderr("\n      Type help for help.\n");
+        return -1;
       }
     }
   }
@@ -469,20 +485,20 @@ static int Help(int help)
     len= 27;
     for (;;) {
       for (;;) {
-	if (len+strlen(*command) > 72) {
-	  strcpy(line+len, "\n");
-	  p_stderr(line);
-	  break;
-	}
-	strcpy(line+len, *command);
-	len+= strlen(*command++);
-	if (!*command) {
-	  strcpy(line+len, "\n");
-	  p_stderr(line);
-	  break;
-	}
-	strcpy(line+len, ", ");
-	len+= 2;
+        if (len+strlen(*command) > 72) {
+          strcpy(line+len, "\n");
+          p_stderr(line);
+          break;
+        }
+        strcpy(line+len, *command);
+        len+= strlen(*command++);
+        if (!*command) {
+          strcpy(line+len, "\n");
+          p_stderr(line);
+          break;
+        }
+        strcpy(line+len, ", ");
+        len+= 2;
       }
       if (!*command) break;
       strcpy(line, "     ");
@@ -890,7 +906,7 @@ static int Info(int help)
     if (outEngines[i]) {
       char msg[80];
       sprintf(msg, "%3d    %s %s %s  ", i, yorn[outDraw[i]], yorn[outSend[i]],
-	     tname[outTypes[i]]);
+             tname[outTypes[i]]);
       p_stdout(msg);
       p_stdout(outNames[i]? outNames[i] : "");
       p_stdout("\n");
@@ -998,23 +1014,23 @@ static int GetDeviceList(int ds, char *command)
     do {
       device= (int)strtol(token, &suffix, 10);
       if (*suffix || device<0 || device>7) {
-	p_stderr("gist: (SYNTAX) ");
-	p_stderr(token);
-	p_stderr(" not a legal device# in ");
-	p_stderr(command);
-	p_stderr(" command\n");
-	return 1;
+        p_stderr("gist: (SYNTAX) ");
+        p_stderr(token);
+        p_stderr(" not a legal device# in ");
+        p_stderr(command);
+        p_stderr(" command\n");
+        return 1;
       }
       if (!outEngines[device])
-	Warning("ignoring non-existent device# ", token);
+        Warning("ignoring non-existent device# ", token);
       else
-	outMark[device]= 1;
+        outMark[device]= 1;
     } while ((token= strtok(0, " \t\n")));
     if (ds==0) {
       for (device=0 ; device<8 ; device++) if (outMark[device]) break;
       if (device>=8) {
-	Warning(command, " command with no legal devices, no action taken");
-	return 1;
+        Warning(command, " command with no legal devices, no action taken");
+        return 1;
       }
     }
 
@@ -1061,7 +1077,7 @@ static int Special(int help)
   if (help) {
     char msg[80];
     sprintf(msg, "gist: n%c command syntax:\n     n%c\n",
-	    cSuffix, cSuffix);
+            cSuffix, cSuffix);
     p_stderr(msg);
     if (cSuffix=='f')
       p_stderr("  Forward n (default 1) pages, then draw\n");
@@ -1128,7 +1144,6 @@ static int cSuffices[]= { 'f', 'b', 'g' };
 static void HandleOther(Engine *engine, int k, int md)
 {
   XEngine *xEngine= GisXEngine(engine);
-  char buffer[8];
   int go, bad;
 
   if (!xEngine) return;
@@ -1171,13 +1186,6 @@ static void HandleOther(Engine *engine, int k, int md)
       if (n) ReadCGM(mPage, nPage, sPage, nPageGroups);
       else Warning("no devices active for send command", "");
     } else if (go==5) {
-      int i;
-      for (i=0 ; i<8 ; i++) {
-        if (outEngines[i]) {
-          GpDeactivate(outEngines[i]);
-          GpKillEngine(outEngines[i]);
-        }
-      }
       p_quit();
     }
     xPrefix= 0;
