@@ -18,7 +18,7 @@
 #include <string.h>
 #include <time.h>
 
-static char *cgmType= "binary CGM";
+static g_callbacks g_cgm_on = { "gist CGMEngine", 0, 0, 0, 0, 0, 0, 0, 0 };
 
 #define N_CGMFONTS 20
 
@@ -31,7 +31,7 @@ extern double floor(double);
 /* ------------------------------------------------------------------------ */
 
 static Octet *FormCommand(Octet *buffer, int klass, int id, long nbytes,
-			  long *lpart);
+                          long *lpart);
 static Octet *NextPartition(Octet *buffer, long nbytes, long *lpart);
 static Octet *Pascalify(Octet *buffer, const char *text, long len, int pad);
 static Octet *FormWords(Octet *buffer, const short *words, long nwords);
@@ -43,7 +43,7 @@ static void WriteError(CGMEngine *cgm, const char *msg);
 static int BeginMetafile(CGMEngine *cgm);
 static void SetPageDefaults(CGMEngine *cgmEngine);
 static void SetCGMTransform(GpTransform *toPixels, int landscape,
-			    GpReal scale);
+                            GpReal scale);
 static int BeginPage(CGMEngine *cgmEngine);
 static void EndClip(CGMEngine *cgmEngine);
 static void BeginClip(CGMEngine *cgmEngine, GpTransform *trans);
@@ -59,19 +59,19 @@ static int SetupLine(CGMEngine *cgmEngine);
 static int SetupEdge(CGMEngine *cgmEngine);
 static void CheckClip(CGMEngine *cgmEngine);
 static int DrawLines(Engine *engine, long n, const GpReal *px,
-		     const GpReal *py, int closed, int smooth);
+                     const GpReal *py, int closed, int smooth);
 static int SetupMarker(CGMEngine *cgmEngine);
 static int DrawMarkers(Engine *engine, long n, const GpReal *px,
-		       const GpReal *py);
+                       const GpReal *py);
 static int SetupText(CGMEngine *cgmEngine);
 static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text);
 static int DrawFill(Engine *engine, long n, const GpReal *px,
-		    const GpReal *py);
+                    const GpReal *py);
 static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
-		     GpReal qy, long width, long height, long nColumns,
-		     const GpColor *colors);
+                     GpReal qy, long width, long height, long nColumns,
+                     const GpColor *colors);
 static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
-			const GpReal *py, const GpReal *qx, const GpReal *qy);
+                        const GpReal *py, const GpReal *qx, const GpReal *qy);
 
 static void IncrementName(char *filename);
 
@@ -107,23 +107,23 @@ cgm_swap(short *words, long nwords)
 /* ------------------------------------------------------------------------ */
 
 static Octet *FormCommand(Octet *buffer, int klass, int id, long nbytes,
-			  long *lpart)
+                          long *lpart)
 {
   long nhere;
-  *buffer++= (klass<<4) | (id>>3);
+  *buffer++= (Octet)((klass<<4) | (id>>3));
   if (nbytes < 0x1f) {
-    *buffer++= (id<<5) | nbytes;
+    *buffer++= (Octet)((id<<5) | nbytes);
     nhere= nbytes;
   } else {
-    *buffer++= (id<<5) | (nbytes<0x1f? nbytes : 0x1f);
+    *buffer++= (Octet)((id<<5) | (nbytes<0x1f? nbytes : 0x1f));
     if (nbytes < MAX_PARTITION) {
       nhere= nbytes;
-      *buffer++= nhere>>8;
-      *buffer++= nhere&0xff;
+      *buffer++= (Octet)(nhere>>8);
+      *buffer++= (Octet)(nhere&0xff);
     } else {
       nhere= MAX_PARTITION;
-      *buffer++= (nhere>>8) | 0x80;
-      *buffer++= nhere&0xff;
+      *buffer++= (Octet)((nhere>>8) | 0x80);
+      *buffer++= (Octet)(nhere&0xff);
     }
   }
   *lpart= nhere;
@@ -135,12 +135,12 @@ static Octet *NextPartition(Octet *buffer, long nbytes, long *lpart)
   long nhere;
   if (nbytes < MAX_PARTITION) {
     nhere= nbytes;
-    *buffer++= nhere>>8;
-    *buffer++= nhere&0xff;
+    *buffer++= (Octet)(nhere>>8);
+    *buffer++= (Octet)(nhere&0xff);
   } else {
     nhere= MAX_PARTITION;
-    *buffer++= (nhere>>8) | 0x80;
-    *buffer++= nhere&0xff;
+    *buffer++= (Octet)((nhere>>8) | 0x80);
+    *buffer++= (Octet)(nhere&0xff);
   }
   *lpart= nhere;
   return buffer;
@@ -162,8 +162,8 @@ static Octet *Pascalify(Octet *buffer, const char *text, long len, int pad)
 static Octet *FormWords(Octet *buffer, const short *words, long nwords)
 {
   while (nwords--) {
-    *buffer++= *words >> 8;
-    *buffer++= *words++ & 0xff;
+    *buffer++= (Octet)(*words >> 8);
+    *buffer++= (Octet)(*words++ & 0xff);
   }
   return buffer;
 }
@@ -179,7 +179,7 @@ static void FormReal(short *fixed, GpReal x)
 static int WriteB(p_file *file, void *buffer, long nbytes)
 {
   if (!file) return 1;
-  return p_fwrite(file, buffer, nbytes)!=nbytes;
+  return p_fwrite(file, buffer, nbytes) != (unsigned long)nbytes;
 }
 
 static void WriteError(CGMEngine *cgm, const char *msg)
@@ -246,7 +246,7 @@ static int BeginMetafile(CGMEngine *cgm)
   len+= 4;
   len+= 4 + lpart+1;
   if (len&1) len++;
-  len+= 4;
+  len+= 6*4;
   len+= 8;
   len+= 784;
   len+= 4 + lfonts;
@@ -278,6 +278,26 @@ static int BeginMetafile(CGMEngine *cgm)
   now= FormCommand(now, 1, 2, lpart+1, &len);    /* METAFILE DESCRIPTION */
   now= Pascalify(now, description, lpart, 1);
 
+  now= FormCommand(now, 1, 3, 2L, &len);                     /* VDC TYPE */
+  value[0]= 0;
+  now= FormWords(now, value, 1L);
+
+  now= FormCommand(now, 1, 4, 2L, &len);            /* INTEGER PRECISION */
+  value[0]= 16;
+  now= FormWords(now, value, 1L);
+
+  now= FormCommand(now, 1, 6, 2L, &len);              /* INDEX PRECISION */
+  value[0]= 16;
+  now= FormWords(now, value, 1L);
+
+  now= FormCommand(now, 1, 7, 2L, &len);              /* COLOR PRECISION */
+  value[0]= 8;
+  now= FormWords(now, value, 1L);
+
+  now= FormCommand(now, 1, 8, 2L, &len);        /* COLOR INDEX PRECISION */
+  value[0]= 8;
+  now= FormWords(now, value, 1L);
+
   now= FormCommand(now, 1, 9, 1L, &len);          /* MAXIMUM COLOR INDEX */
   *now++= 255;
   *now++= 0;
@@ -378,7 +398,7 @@ static void SetPageDefaults(CGMEngine *cgmEngine)
 }
 
 static void SetCGMTransform(GpTransform *toPixels, int landscape,
-			    GpReal scale)
+                            GpReal scale)
 {
   toPixels->viewport.xmin= toPixels->viewport.ymin= 0.0;
   if (landscape) {
@@ -422,7 +442,7 @@ static int BeginPage(CGMEngine *cgmEngine)
   /* Set transform viewport to reflect current page orientation */
   if (cgmEngine->landscape != cgmEngine->e.landscape) {
     SetCGMTransform(&cgmEngine->e.transform, cgmEngine->e.landscape,
-		    cgmEngine->scale);
+                    cgmEngine->scale);
     cgmEngine->landscape= cgmEngine->e.landscape;
   }
 
@@ -463,9 +483,9 @@ static int BeginPage(CGMEngine *cgmEngine)
     now= FormCommand(now, 5, 34, 3*nColors+1, &len);    /* COLOR TABLE */
     *now++= 10;   /* begin at color index 10 */
     for (i=0 ; i<nColors ; i++) {
-      *now++= P_R(palette[i]);
-      *now++= P_G(palette[i]);
-      *now++= P_B(palette[i]);
+      *now++= (Octet)(P_R(palette[i]));
+      *now++= (Octet)(P_G(palette[i]));
+      *now++= (Octet)(P_B(palette[i]));
     }
     if (!(nColors&1)) *now++= 0;
 
@@ -533,12 +553,12 @@ static void BeginClip(CGMEngine *cgmEngine, GpTransform *trans)
     if (points[0].x > points[1].x) {
       GpReal xt= points[0].x;
       points[0].x= points[1].x;
-      points[1].x= xt;
+      points[1].x= (short)xt;
     }
     if (points[0].y > points[1].y) {
       GpReal yt= points[0].y;
       points[0].y= points[1].y;
-      points[1].y= yt;
+      points[1].y= (short)yt;
     }
 
     now= FormCommand(now, 3, 5, 8L, &len);           /* CLIP RECTANGLE */
@@ -590,7 +610,7 @@ static int EndPage(CGMEngine *cgmEngine)
     return 1;
   }
 
-  if (p_ftell(file) < cgmEngine->fileSize) {
+  if (p_ftell(file) < (unsigned long)cgmEngine->fileSize) {
     /* Just flush buffers if file still reasonable size */
     p_fflush(file);
     cgmEngine->state= 5;   /* Picture closed */
@@ -682,30 +702,30 @@ static int SetupColor(CGMEngine *cgmEngine, unsigned long color, int which)
     /* "CI index C"-- "CI" omitted if current color is indexed */
     GpColorCell *palette= cgmEngine->e.palette;
     if (nColors>0) {
-      if (color>=nColors) color= nColors-1;
-      if (cgmEngine->colorMode) c= color;  /* this page has a color table */
+      if (color>=(unsigned long)nColors) color= nColors-1;
+      if (cgmEngine->colorMode) c= (short)color;  /* page has color table */
       else {                              /* this is a 240 gray mono page */
-	c= (P_R(palette[color]) +
-            P_G(palette[color]) + P_B(palette[color]))/3;
-	c-= (c+8)/16;
+        c= (short)((P_R(palette[color]) +
+                    P_G(palette[color]) + P_B(palette[color]))/3);
+        c-= (c+8)/16;
       }
     } else {
       if (color>255) color= 255;
-      c= color;         /* No palette ==> no color table on page */
+      c= (short)color;         /* No palette ==> no color table on page */
     }
     c+= 10;
 
   } else if (color<256UL) {
     /* standard color command FG, RED, etc. */
     if (color<YELLOW_COLOR) color= FG_COLOR;
-    c= 255UL-color;  /* standard colors are 0 thru 9 */
+    c= (short)(255UL-color);  /* standard colors are 0 thru 9 */
   } else {
-    c = 255UL-FG_COLOR;
+    c = (short)(255UL-FG_COLOR);
   }
 
   FormCommand(command, 5, cgmColorID[which], 1L, &len);
         /* LINE COLOR (0) MARKER COLOR (1) TEXT COLOR (2) FILL COLOR (3) */
-  command[2]= c;
+  command[2]= (Octet)c;
   command[3]= 0;
   if (WriteB(file, command, 4L)) {
     WriteError(cgmEngine, "write to CGM failed in SetupColor");
@@ -828,7 +848,7 @@ static void CheckClip(CGMEngine *cgmEngine)
 }
 
 static int DrawLines(Engine *engine, long n, const GpReal *px,
-		     const GpReal *py, int closed, int smooth)
+                     const GpReal *py, int closed, int smooth)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -845,23 +865,23 @@ static int DrawLines(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
     if (closed) {
       if (firstPass) {
-	firstPoint= points[0];
-	firstPass= 0;
+        firstPoint= points[0];
+        firstPass= 0;
       }
       if (n==nPoints) {
-	n++;
-	points[nPoints++]= firstPoint;
+        n++;
+        points[nPoints++]= firstPoint;
       }
     }
 
     if (smooth) {
       /* There may be a GDP for Bezier curves, but I don't know how
-	 standard it is.  This is sure to be non-portable, but also
-	 sure to produces SOME acceptable output (identical, in fact,
-	 to what the Gist X engine displays).  */
+         standard it is.  This is sure to be non-portable, but also
+         sure to produces SOME acceptable output (identical, in fact,
+         to what the Gist X engine displays).  */
       short value= smooth;  /* identifier */
       now= FormCommand(command, 7, 2, 3L, &len);     /* APPLICATION DATA */
       now= FormWords(now, &value, 1L);
@@ -941,7 +961,7 @@ static int SetupMarker(CGMEngine *cgmEngine)
 }
 
 static int DrawMarkers(Engine *engine, long n, const GpReal *px,
-		       const GpReal *py)
+                       const GpReal *py)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -957,7 +977,7 @@ static int DrawMarkers(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
 
     now= FormCommand(command, 4, 3, 4*nPoints, &len);      /* POLYMARKER */
     if (WriteB(file, command, now-command)) {
@@ -1106,6 +1126,9 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
   long len;
   const char *t;
 
+  /* do not try to output zero size text */
+  if ((short)(0.8*gistA.t.height*cgmEngine->scale+0.5) <= 0) return 0;
+
   CheckClip(cgmEngine);
   if (SetupText(cgmEngine)) return 1;
   file= cgmEngine->file;
@@ -1132,10 +1155,10 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
   else { ymin= wind->ymax; ymax= wind->ymin; }
   if (gistA.t.orient==TX_RIGHT || gistA.t.orient==TX_LEFT) {
     if (x0<xmin-width || x0>xmax+width ||
-	y0<ymin-height || y0>ymax+height) return 0;
+        y0<ymin-height || y0>ymax+height) return 0;
   } else {
     if (x0<xmin-height || x0>xmax+height ||
-	y0<ymin-width || y0>ymax+width) return 0;
+        y0<ymin-width || y0>ymax+width) return 0;
   }
 
   /* Adjust y0 to represent topmost line */
@@ -1164,8 +1187,8 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
       now= FormWords(now, xyf, 3L);
       now= Pascalify(now, t, (long)count, 1);
       if (WriteB(file, cgmText, now-cgmText)) {
-	WriteError(cgmEngine, "write to CGM failed in DrwText");
-	return 1;
+        WriteError(cgmEngine, "write to CGM failed in DrwText");
+        return 1;
       }
     }
 
@@ -1182,7 +1205,7 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
 /* ------------------------------------------------------------------------ */
 
 static int DrawFill(Engine *engine, long n, const GpReal *px,
-		    const GpReal *py)
+                    const GpReal *py)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1202,7 +1225,7 @@ static int DrawFill(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
 
     now= FormCommand(command, 4, 7, 4*nPoints, &len);         /* POLYGON */
     if (WriteB(file, command, now-command)) {
@@ -1231,8 +1254,8 @@ static int DrawFill(Engine *engine, long n, const GpReal *px,
 /* ------------------------------------------------------------------------ */
 
 static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
-		     GpReal qy, long width, long height, long nColumns,
-		     const GpColor *colors)
+                     GpReal qy, long width, long height, long nColumns,
+                     const GpColor *colors)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1251,10 +1274,10 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
   /* Transform corner coordinates, clipping and adjusting width,
      height, nColumns, and colors as necessary.  */
   width= GpClipCells(&map->x, &px, &qx,
-		     gistT.window.xmin, gistT.window.xmax, width, &off);
+                     gistT.window.xmin, gistT.window.xmax, width, &off);
   colors+= off;
   height= GpClipCells(&map->y, &py, &qy,
-		      gistT.window.ymin, gistT.window.ymax, height, &off);
+                      gistT.window.ymin, gistT.window.ymax, height, &off);
   colors+= nColumns*off;
   if (width>0x7ffe) width= 0x7ffe;
   if (height>0x7ffe) height= 0x7ffe;
@@ -1305,13 +1328,13 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
   for (;;) {
     while (lPart--) {
       if (i>=width) {
-	if (pad) {
-	  *now++= 0;
-	  nCells--;
-	  if (!lPart--) break;
-	}
-	i= 0;
-	j+= nColumns;
+        if (pad) {
+          *now++= 0;
+          nCells--;
+          if (!lPart--) break;
+        }
+        i= 0;
+        j+= nColumns;
       }
       if (gistA.rgb) {
         color = (colors[3*(i+j)]+colors[3*(i+j)+1]+colors[3*(i+j)+2])/3;
@@ -1322,9 +1345,9 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
       i++;
       if (color>=nColors && nColors>0) color= nColors-1;
       if (!colorMode) {  /* this is default 240 gray mono page */
-	color= (P_R(palette[color]) +
+        color= (P_R(palette[color]) +
                 P_G(palette[color]) + P_B(palette[color]))/3;
-	color-= (color+8)/16;
+        color-= (color+8)/16;
       }
 
       *now++= 10+color;  /* skip the 10 standard colors */
@@ -1348,7 +1371,7 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
 /* ------------------------------------------------------------------------ */
 
 static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
-			const GpReal *py, const GpReal *qx, const GpReal *qy)
+                        const GpReal *py, const GpReal *qx, const GpReal *qy)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1364,7 +1387,7 @@ static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nSegs=
-	  GpIntSegs(map, maxSegs, n, px, py, qx, qy, &segs))) {
+          GpIntSegs(map, maxSegs, n, px, py, qx, qy, &segs))) {
 
     now= FormCommand(command, 4, 2, 8*nSegs, &len); /* DISJOINT POLYLINE */
     if (WriteB(file, command, now-command)) {
@@ -1397,7 +1420,7 @@ static void IncrementName(char *filename)
 {
   int i, len= filename? strlen(filename) : 0;
   if (len>4 && (strcmp(filename+len-4, ".cgm")==0 ||
-		strcmp(filename+len-4, ".CGM")==0)) i= len-4;
+                strcmp(filename+len-4, ".CGM")==0)) i= len-4;
   else i= len;
   while (i-- > 0) {
     if (filename[i]=='9') {
@@ -1413,7 +1436,7 @@ static void IncrementName(char *filename)
 /* ------------------------------------------------------------------------ */
 
 GpReal gCGMScale= 25545.24;   /* default CGM scale is 2400 dpi (round up) */
-long gCGMFileSize= 1000000;   /* default max file size is about 1 Meg */
+long gCGMFileSize= 100000000; /* default max file size is about 100 Meg */
 
 Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
 {
@@ -1427,11 +1450,11 @@ Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
   SetCGMTransform(&toPixels, landscape, gCGMScale);
 
   cgmEngine=
-    (CGMEngine *)GpNewEngine(engineSize, name, cgmType, &toPixels, landscape,
-			     &Kill, &Clear, &Flush, &GpComposeMap,
-			     &ChangePalette, &DrawLines, &DrawMarkers,
-			     &DrwText, &DrawFill, &DrawCells,
-			     &DrawDisjoint);
+    (CGMEngine *)GpNewEngine(engineSize, name, &g_cgm_on, &toPixels, landscape,
+                             &Kill, &Clear, &Flush, &GpComposeMap,
+                             &ChangePalette, &DrawLines, &DrawMarkers,
+                             &DrwText, &DrawFill, &DrawCells,
+                             &DrawDisjoint);
 
   if (!cgmEngine) {
     strcpy(gistError, "memory manager failed in GpCGMEngine");
@@ -1459,7 +1482,7 @@ Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
 
 CGMEngine *GisCGMEngine(Engine *engine)
 {
-  return (engine && engine->type==cgmType)? (CGMEngine *)engine : 0;
+  return (engine && engine->on==&g_cgm_on)? (CGMEngine *)engine : 0;
 }
 
 void GcgmSetScale(CGMEngine *cgmEngine, GpReal scale)
