@@ -15,7 +15,19 @@
 
 #include <string.h>
 
-static char *xType= "PLAY Window";
+static void g_on_expose(void *c, int *xy);
+static void g_on_destroy(void *c);
+static void g_on_resize(void *c,int w,int h);
+static void g_on_focus(void *c,int in);
+static void g_on_key(void *c,int k,int md);
+static void g_on_click(void *c,int b,int md,int x,int y, unsigned long ms);
+static void g_on_motion(void *c,int md,int x,int y);
+static void g_on_deselect(void *c);
+static void g_on_panic(p_scr *screen);
+
+static g_callbacks g_x_on = {
+  "gist XEngine", g_on_expose, g_on_destroy, g_on_resize, g_on_focus,
+  g_on_key, g_on_click, g_on_motion, g_on_deselect };
 
 static int ChangePalette(Engine *engine);
 static GpReal TextWidth(const char *text, int nc, const GpTextAttribs *t);
@@ -33,22 +45,22 @@ static void ChangeMap(Engine *engine);
 static void chk_clipping(XEngine *xeng);
 static int SetupLine(XEngine *xeng, GpLineAttribs *gistAl, int join);
 static int DrawLines(Engine *engine, long n, const GpReal *px,
-		     const GpReal *py, int closed, int smooth);
+                     const GpReal *py, int closed, int smooth);
 static int DrawMarkers(Engine *engine, long n, const GpReal *px,
-		       const GpReal *py);
+                       const GpReal *py);
 static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text);
 static int DrawFill(Engine *engine, long n, const GpReal *px,
-		    const GpReal *py);
+                    const GpReal *py);
 static int GetCells(GpMap *map, GpReal xmin, GpReal xmax,
                     GpReal px, GpReal qx, long width,
                     int *i0, int *di, int *ncols, int *x0, int *x1);
 static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
-		     GpReal qy, long width, long height, long nColumns,
-		     const GpColor *colors);
+                     GpReal qy, long width, long height, long nColumns,
+                     const GpColor *colors);
 static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
-			const GpReal *py, const GpReal *qx, const GpReal *qy);
+                        const GpReal *py, const GpReal *qx, const GpReal *qy);
 static void GetVisibleNDC(XEngine *xeng,
-			  GpReal *xn, GpReal *xx, GpReal *yn, GpReal *yx);
+                          GpReal *xn, GpReal *xx, GpReal *yn, GpReal *yx);
 
 /* Hook for hlevel.c error handling.  */
 extern void (*HLevelHook)(Engine *engine);
@@ -117,31 +129,31 @@ TextWidth(const char *text, int nc, const GpTextAttribs *t)
     for ( ; nc-- ; text++) {
       c= text[0];
       if ((nc && c=='!') || c=='^' || c=='_') {
-	if (txt<text)
+        if (txt<text)
           width += p_txwidth(current_scr, txt, (int)(text-txt),
                              gistA.t.font, current_fsize);
-	if (firstChunk) {
-	  nChunk= (int)(text-txt);
-	  chunkWidth= width;
-	  firstChunk= 0;
-	}
-	txt= text+1;
-	if (c=='!') {
-	  /* process single character escapes immediately */
-	  text= txt;
-	  nc--;
-	  c= txt[0];
-	  if (c!='!' && c!='^' && c!='_') {
-	    if (c==']') c= '^';
+        if (firstChunk) {
+          nChunk= (int)(text-txt);
+          chunkWidth= width;
+          firstChunk= 0;
+        }
+        txt= text+1;
+        if (c=='!') {
+          /* process single character escapes immediately */
+          text= txt;
+          nc--;
+          c= txt[0];
+          if (c!='!' && c!='^' && c!='_') {
+            if (c==']') c= '^';
             width += p_txwidth(current_scr, &c, 1,
                                current_fsym, current_fsize);
-	    txt++;
-	  }
-	} else if (c=='^') {
-	  supersub|= 1;
-	} else {
-	  supersub|= 2;
-	}
+            txt++;
+          }
+        } else if (c=='^') {
+          supersub|= 1;
+        } else {
+          supersub|= 2;
+        }
       }
     }
     if (txt<text)
@@ -173,6 +185,10 @@ GxJustifyText(GpXYMap *map, GpReal x0, GpReal y0, const char *text,
   if (x0<-16000. || x0>16000. || y0<-16000. || y0>16000.) return -1;
   current_state = 0;
 
+  /* call p_font before any calls to TextWidth
+   * - may improve efficiency of p_txwidth, p_txheight */
+  p_font(current_win, gistA.t.font, current_fsize, gistA.t.orient);
+
   /* Set nLines, maxWidth, nChars, prevWidth */
   firstTextLine = 1;
   nChars = prevWidth = chunkWidth = nChunk = supersub = 0;
@@ -282,8 +298,8 @@ GxJustifyNext(const char **text, int *ix, int *iy)
     /* scan for end of first chunk */
     if (gtDoEscapes) {
       for (nChunk=0 ; nChunk<nChars ; nChunk++) {
-	c= txt[nChunk];
-	if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
+        c= txt[nChunk];
+        if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
       }
     } else {
       nChunk= nChars;
@@ -333,52 +349,52 @@ GxJustifyNext(const char **text, int *ix, int *iy)
       nChunk= 1;
       c= txt[0];
       if (c=='!' || c=='^' || c=='_') {
-	/* chunk is just ordinary text */
-	for ( ; nChunk<nChars ; nChunk++) {
-	  c= txt[nChunk];
-	  if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
-	}
+        /* chunk is just ordinary text */
+        for ( ; nChunk<nChars ; nChunk++) {
+          c= txt[nChunk];
+          if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
+        }
         p_font(current_win, gistA.t.font, current_fsize, gistA.t.orient);
-	current_state&= 3;
+        current_state&= 3;
       } else {
-	/* chunk is single symbol char */
+        /* chunk is single symbol char */
         p_font(current_win, current_fsym, current_fsize, gistA.t.orient);
-	current_state|= 4;
+        current_state|= 4;
       }
 
     } else {
       for (nChunk=0 ; nChunk<nChars ; nChunk++) {
-	c= txt[nChunk];
-	if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
+        c= txt[nChunk];
+        if ((nChunk+1<nChars && c=='!') || c=='^' || c=='_') break;
       }
       if (nChunk)
         p_font(current_win, gistA.t.font, current_fsize, gistA.t.orient);
       if (c1=='^') {
-	if (current_state&1) {
-	  yadj+= dy_super;  /* return from super to normal */
-	  current_state= 0;
-	} else {
-	  if (current_state&2) yadj-= dy_sub;
-	  yadj-= dy_super;  /* move to superscript */
-	  current_state= 1;
-	}
+        if (current_state&1) {
+          yadj+= dy_super;  /* return from super to normal */
+          current_state= 0;
+        } else {
+          if (current_state&2) yadj-= dy_sub;
+          yadj-= dy_super;  /* move to superscript */
+          current_state= 1;
+        }
       } else if (c1=='_') {
-	if (current_state&2) {
-	  yadj-= dy_sub;  /* return from sub to normal */
-	  current_state= 0;
-	} else {
-	  if (current_state&1) yadj+= dy_super;
-	  yadj+= dy_sub;  /* move to subscript */
-	  current_state= 2;
-	}
+        if (current_state&2) {
+          yadj-= dy_sub;  /* return from sub to normal */
+          current_state= 0;
+        } else {
+          if (current_state&1) yadj+= dy_super;
+          yadj+= dy_sub;  /* move to subscript */
+          current_state= 2;
+        }
       } else {
-	/* just finished a symbol char */
-	current_state&= 3;
+        /* just finished a symbol char */
+        current_state&= 3;
       }
     }
 
     if (nChunk &&
-	(nChunk<nChars || alignH!=TH_LEFT || gistA.t.orient!=TX_RIGHT)) {
+        (nChunk<nChars || alignH!=TH_LEFT || gistA.t.orient!=TX_RIGHT)) {
       char caret= '^';
       if (nChunk==1 && (current_state&4) && txt[0]==']') txt= &caret;
       chunkWidth = p_txwidth(current_scr, txt, nChunk,
@@ -408,12 +424,39 @@ GxJustifyNext(const char **text, int *ix, int *iy)
 
 /* ------------------------------------------------------------------------ */
 
+/* notes for Kill() and g_on_destroy
+ * (1) Kill() is program-driven (e.g.- winkill)
+ *     g_on_destroy() is event-driven (e.g.- mouse click)
+ * (2) however, the p_destroy() function MIGHT or MIGHT NOT
+ *     result in a call to g_on_destroy()
+ *     under Windows, g_on_destroy() is naturally called
+ *       by the call p_destroy() must make to close the window
+ *     under UNIX/X, the play event handler calls p_destroy()
+ *       after g_on_destroy() returns
+ * (3) therefore g_on_destroy() must not call p_destroy(), since
+ *       this would cause infinite recursion on Windows, and a
+ *       double call on X.
+ * (4) worse, if this is the final window on an X display, gist
+ *       should disconnect from the display, but that can only
+ *       be done after the final p_destroy(), which is after
+ *       the g_on_destroy() in the event-driven case
+ *       -- thus, the p_disconnect must take place on the next
+ *          event, which is during the GhBeforeWait hlevel.c function
+ */
+/* hack to disconnect if last engine destroyed (see GhBeforeWait) */
+static void g_do_disconnect(void);
+extern void (*g_pending_task)(void);
+void (*g_pending_task)(void) = 0;
+
 static void
 Kill(Engine *engine)
 {
   XEngine *xeng= (XEngine *)engine;
+  p_win *w = xeng->win;
   ShutDown(xeng);
-  GpDelEngine(engine);
+  if (w) p_destroy(w);
+  /* for program-driven Kill(), can take care of p_disconnect immediately */
+  g_do_disconnect();
 }
 
 static int
@@ -553,14 +596,16 @@ ChangeMap(Engine *engine)
   if (landscape!=xeng->e.landscape) {
     /* this is probably insane if in animation mode... */
     SetXTransform(&xeng->e.transform, xeng->e.landscape, xeng->dpi);
+    xeng->width = (int)xeng->e.transform.window.xmax;
+    xeng->height = (int)xeng->e.transform.window.ymin;
     xeng->swapped = xeng->e.transform;
     /* make adjustments to allow for SetXTransform, then recenter */
     if (xeng->w != xeng->win) {
-      xeng->a_x += xeng->x;
-      xeng->a_y += xeng->y;
+      xeng->a_x += xeng->x+1;
+      xeng->a_y += xeng->y+1;
     }
-    xeng->x = xeng->y = 0;
-    GxRecenter(xeng, xeng->width, xeng->height);
+    xeng->x = xeng->y = -1;
+    GxRecenter(xeng, xeng->wtop+xeng->leftMargin, xeng->htop+xeng->topMargin);
   }
 
   /* do generic change map */
@@ -727,7 +772,7 @@ DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
   if (!w || !xeng->mapped) return 1;
   chk_clipping(xeng);
 
-  current_fsize = (xeng->dpi/ONE_INCH)*gistA.t.height;
+  current_fsize = (int)((xeng->dpi/ONE_INCH)*gistA.t.height);
   if (current_fsize<4) current_fsize = 4;     /* totally illegible */
   if (current_fsize>180) current_fsize = 180; /* way too big */
   current_fsym = T_SYMBOL | (gistA.t.font&3);
@@ -988,21 +1033,9 @@ DrawDisjoint(Engine *engine, long n, const GpReal *px,
 
 /* ------------------------------------------------------------------------ */
 
-static void g_on_expose(void *c, int *xy);
-static void g_on_destroy(void *c);
-static void g_on_resize(void *c,int w,int h);
-static void g_on_focus(void *c,int in);
-static void g_on_key(void *c,int k,int md);
-static void g_on_click(void *c,int b,int md,int x,int y, unsigned long ms);
-static void g_on_motion(void *c,int md,int x,int y);
-static void g_on_deselect(void *c);
-static void g_on_panic(p_scr *screen);
-
 static Engine *waiting_for = 0;
 static void (*wait_callback)(void) = 0;
 
-extern int gist_expose_wait(Engine *eng, void (*e_callback)(void));
-
 int
 gist_expose_wait(Engine *eng, void (*e_callback)(void))
 {
@@ -1011,6 +1044,9 @@ gist_expose_wait(Engine *eng, void (*e_callback)(void))
     wait_callback = 0;
     return 1;
   } else {
+    XEngine *xeng = GisXEngine(eng);
+    if (!xeng || !xeng->w) return 1;
+    if (xeng->mapped) return 2;
     waiting_for = eng;
     wait_callback = e_callback;
     return 0;
@@ -1021,74 +1057,104 @@ static void
 g_on_expose(void *c, int *xy)
 {
   XEngine *xeng = c;
-  if (c && c == waiting_for) {
-    waiting_for = 0;
-    if (wait_callback) wait_callback();
-    wait_callback = 0;
+  if (xeng->e.on==&g_x_on) {
+    if (c && c == waiting_for) {
+      waiting_for = 0;
+      if (wait_callback) wait_callback();
+      wait_callback = 0;
+    }
+    if (!xeng->w) return;
+    xeng->mapped = 1;
+    if (xeng->HandleExpose)
+      /* the alternate handler should probably call GxExpose */
+      xeng->HandleExpose(&xeng->e, xeng->e.drawing, xy);
+    else
+      GxExpose(&xeng->e, xeng->e.drawing, xy);
+  } else if (xeng->e.on && xeng->e.on->expose) {
+    xeng->e.on->expose(c, xy);
   }
-  if (!xeng->w) return;
-  xeng->mapped = 1;
-  if (xeng->HandleExpose)
-    /* the alternate handler should probably call GxExpose */
-    xeng->HandleExpose(&xeng->e, xeng->e.drawing, xy);
-  else
-    GxExpose(&xeng->e, xeng->e.drawing, xy);
 }
 
 static void
 g_on_click(void *c,int b,int md,int x,int y, unsigned long ms)
 {
   XEngine *xeng = c;
-  if (!xeng->w) return;
-  if (xeng->HandleClick)
-    xeng->HandleClick(&xeng->e, b, md, x, y, ms);
+  if (xeng->e.on==&g_x_on) {
+    if (!xeng->w) return;
+    if (xeng->HandleClick)
+      xeng->HandleClick(&xeng->e, b, md, x, y, ms);
+  } else if (xeng->e.on && xeng->e.on->click) {
+    xeng->e.on->click(c, b, md, x, y, ms);
+  }
 }
 
 static void
 g_on_motion(void *c,int md,int x,int y)
 {
   XEngine *xeng = c;
-  if (!xeng->w) return;
-  if (xeng->HandleMotion)
-    xeng->HandleMotion(&xeng->e, md, x, y);
+  if (xeng->e.on==&g_x_on) {
+    if (!xeng->w) return;
+    if (xeng->HandleMotion)
+      xeng->HandleMotion(&xeng->e, md, x, y);
+  } else if (xeng->e.on && xeng->e.on->motion) {
+    xeng->e.on->motion(c, md, x, y);
+  }
 }
 
 static void
 g_on_destroy(void *c)
 {
   XEngine *xeng = c;
-  p_win *w = xeng->w;
-  p_win *win = xeng->win;
-  xeng->win = xeng->w = 0;
-  if (w && w!=win) p_destroy(w);
-  xeng->mapped = 0;
+  if (xeng->e.on==&g_x_on) {
+    /* if xeng->win==0, assume ShutDown already called */
+    if (xeng->win) ShutDown(xeng);
+  } else if (xeng->e.on && xeng->e.on->destroy) {
+    xeng->e.on->destroy(c);
+  }
 }
 
 static void
 g_on_resize(void *c,int w,int h)
 {
   XEngine *xeng = c;
-  if (xeng->w) GxRecenter(xeng, w, h);
+  if (xeng->e.on==&g_x_on) {
+    if (xeng->w) GxRecenter(xeng, w, h);
+  } else if (xeng->e.on && xeng->e.on->resize) {
+    xeng->e.on->resize(c, w, h);
+  }
 }
 
 static void
 g_on_focus(void *c,int in)
 {
   XEngine *xeng = c;
-  if (xeng->w && xeng->HandleMotion && in==2)
-    xeng->HandleMotion(&xeng->e, 0, -1, -1);
+  if (xeng->e.on==&g_x_on) {
+    if (xeng->w && xeng->HandleMotion && in==2)
+      xeng->HandleMotion(&xeng->e, 0, -1, -1);
+  } else if (xeng->e.on && xeng->e.on->focus) {
+    xeng->e.on->focus(c, in);
+  }
 }
 
 static void
 g_on_key(void *c,int k,int md)
 {
   XEngine *xeng = c;
-  if (xeng->w && xeng->HandleKey)
-    xeng->HandleKey(&xeng->e, k, md);
+  if (xeng->e.on==&g_x_on) {
+    if (xeng->w && xeng->HandleKey)
+      xeng->HandleKey(&xeng->e, k, md);
+  } else if (xeng->e.on && xeng->e.on->key) {
+    xeng->e.on->key(c, k , md);
+  }
 }
 
 static void g_on_deselect(void *c)
 {
+  XEngine *xeng = c;
+  if (xeng->e.on==&g_x_on) {
+  } else if (xeng->e.on && xeng->e.on->deselect) {
+    xeng->e.on->deselect(c);
+  }
 }
 
 void
@@ -1100,10 +1166,10 @@ GxExpose(Engine *engine, Drauing *drawing, int *xy)
   /* xy=0 to redraw all, otherwise x0,y0,x1,y1 */
   if (xy) {
     GpXYMap *map = &engine->devMap;
-    damage.xmin= (xy[0]-xeng->x)/map->x.scale;
-    damage.xmax= (xy[2]-xeng->x)/map->x.scale;
-    damage.ymax= (xy[1]-xeng->y)/map->y.scale;
-    damage.ymin= (xy[3]-xeng->y)/map->y.scale;
+    damage.xmin= (xy[0]-map->x.offset)/map->x.scale;
+    damage.xmax= (xy[2]-map->x.offset)/map->x.scale;
+    damage.ymax= (xy[1]-map->y.offset)/map->y.scale;
+    damage.ymin= (xy[3]-map->y.offset)/map->y.scale;
   } else {
     damage.xmin = xeng->swapped.viewport.xmin;
     damage.xmax = xeng->swapped.viewport.xmax;
@@ -1179,6 +1245,8 @@ static int n_screens = 0;
 void
 g_initializer(int *pargc, char *argv[])
 {
+  extern char *g_argv0;
+  g_argv0 = argv? argv[0] : 0;
   p_gui(&g_on_expose, &g_on_destroy, &g_on_resize, &g_on_focus,
         &g_on_key, &g_on_click, &g_on_motion, &g_on_deselect,
         &g_on_panic);
@@ -1205,11 +1273,11 @@ g_connect(char *displayName)
   if (!len) displayName = 0;
   if (g_screens) {
     for (i=0 ; i<n_screens ; i++) {
-      if (g_screens[i].name)
-        for (j=0 ; j<len ; j++)
+      j = 0;
+      if (g_screens[i].name) {
+        for ( ; j<len ; j++)
           if (g_screens[i].s && g_screens[i].name[j]!=displayName[j]) break;
-      else
-        j = 0;
+      }
       if (j==len && (len? (!g_screens[i].name[j]) : !g_screens[i].name)) {
         if (number == g_screens[i].number) break;
         else if (i0<0) i0 = i;
@@ -1250,6 +1318,8 @@ g_disconnect(p_scr *s)
       }
     }
     p_disconnect(s);
+  } else {
+    g_do_disconnect();
   }
 }
 
@@ -1285,11 +1355,11 @@ GxEngine(p_scr *s, char *name, GpTransform *toPixels,
   gx_translate(&toPixels->window, x+leftMargin, y+topMargin);
 
   xEngine =
-    (XEngine *)GpNewEngine(engineSize, name, xType, toPixels, width>height,
-			   &Kill, &Clear, &Flush, &ChangeMap,
-			   &ChangePalette, &DrawLines, &DrawMarkers,
-			   &DrwText, &DrawFill, &DrawCells,
-			   &DrawDisjoint);
+    (XEngine *)GpNewEngine(engineSize, name, &g_x_on, toPixels, width>height,
+                           &Kill, &Clear, &Flush, &ChangeMap,
+                           &ChangePalette, &DrawLines, &DrawMarkers,
+                           &DrwText, &DrawFill, &DrawCells,
+                           &DrawDisjoint);
   if (!xEngine) {
     strcpy(gistError, "memory manager failed in GxEngine");
     return 0;
@@ -1391,7 +1461,7 @@ GxInput(Engine *engine,
 XEngine *
 GisXEngine(Engine *engine)
 {
-  return (engine && engine->type==xType)? (XEngine *)engine : 0;
+  return (engine && engine->on==&g_x_on)? (XEngine *)engine : 0;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1400,7 +1470,7 @@ int
 GxAnimate(Engine *engine, GpBox *viewport)
 {
   XEngine *xeng = GisXEngine(engine);
-  int x, y, x0, y0, x1, y1;
+  int x, y, x1, y1;
   GpBox *v, *w;
   GpReal xmin, xmax, ymin, ymax;
   GpReal scalx, offx, scaly, offy;
@@ -1458,9 +1528,12 @@ GxAnimate(Engine *engine, GpBox *viewport)
     w->ymax = 0.0;
   }
   GpDeviceMap((Engine *)xeng);
-  GetXRectangle(&xeng->e.devMap, v, &x0, &y0, &x1, &y1);
-  x1 -= x0;
-  y1 -= y0;
+  /* GetXRectangle(&xeng->e.devMap, v, &x0, &y0, &x1, &y1);
+     x1 -= x0;
+     y1 -= y0;
+  */
+  x1 = xeng->wtop;
+  y1 = xeng->htop;
 
   /* create the offscreen pixmap */
   xeng->w = p_offscreen(xeng->win, x1, y1);
@@ -1548,6 +1621,20 @@ GxDirect(Engine *engine)
 
 void (*HLevelHook)(Engine *engine)= 0;
 
+static void
+g_do_disconnect(void)
+{
+  if (g_screens) {
+    p_scr *s;
+    int i;
+    for (i=n_screens-1 ; i>=0 ; i--) {
+      s = g_screens[i].s;
+      if (s && !p_wincount(s)) g_disconnect(s);
+    }
+  }
+  g_pending_task = 0;
+}
+
 static void
 ShutDown(XEngine *xeng)
 {
@@ -1555,25 +1642,18 @@ ShutDown(XEngine *xeng)
   p_win *w = xeng->w;
   p_win *win = xeng->win;
   xeng->mapped= 0;
+  /* turn off all event callbacks (probably unnecessary) */
+  xeng->e.on = 0;
   /* destroy any hlevel references without further ado */
   if (HLevelHook) HLevelHook((Engine *)xeng);
-  if (w && w!=win) {
-    xeng->w = win;
-    p_destroy(w);
-  }
-  if (win) {
-    xeng->w = xeng->win = 0;
-    p_destroy(win);
-  }
+  xeng->w = xeng->win = 0;
+  xeng->s = 0;
+  /* note that p_destroy and GpDelEngine call on_destroy in Windows */
+  if (w && w!=win) p_destroy(w);
+  GpDelEngine(&xeng->e);
   if (s) {
-    /* disconnect if this is the last engine on the screen */
-    Engine *eng;
-    XEngine *xe2;
-    for (eng=GpNextEngine(0) ; eng ; eng=GpNextEngine(eng)) {
-      xe2 = GisXEngine(eng);
-      if (xe2 && xe2!=xeng && xe2->s==s) break;
-    }
-    if (!eng) g_disconnect(s);
+    if (!p_wincount(s))
+      g_pending_task = g_do_disconnect;
   }
 }
 
@@ -1620,12 +1700,26 @@ g_rgb_read(Engine *eng, GpColor *rgb, long *nx, long *ny)
 {
   XEngine *xeng = GisXEngine(eng);
   if (!xeng || !xeng->w || !xeng->win) return 1;
-  if (!rgb) {
-    *nx = xeng->wtop;
-    *ny = xeng->htop;
+  GpPreempt(eng);
+  GdDraw(1);       /* make sure screen updated */
+  GpPreempt(0);
+  if (xeng->w == xeng->win) {
+    /* not in animate mode */
+    if (!rgb) {
+      *nx = xeng->wtop;
+      *ny = xeng->htop;
+    } else {
+      p_rgb_read(xeng->win, rgb, xeng->leftMargin, xeng->topMargin,
+                 xeng->leftMargin+xeng->wtop, xeng->topMargin+xeng->htop);
+    }
   } else {
-    p_rgb_read(xeng->win, rgb, xeng->leftMargin, xeng->topMargin,
-               xeng->leftMargin+xeng->wtop, xeng->topMargin+xeng->htop);
+    /* in animate mode, read offscreen pixmap */
+    if (!rgb) {
+      *nx = xeng->a_width;
+      *ny = xeng->a_height;
+    } else {
+      p_rgb_read(xeng->w, rgb, 0, 0, xeng->a_width, xeng->a_height);
+    }
   }
   return 0;
 }
