@@ -8,44 +8,14 @@
 #include "dg.h"
 
 
-
-/*-----prototypes of functions defined here-----*/
-extern long ilaenv( long ispec, char *name, char *opts,
-		   long n1, long n2, long n3,long n4 );
-extern void dgetf2( long m, long n, double a[], long lda, long ipiv[],
-		   long *info );
-extern void dlaswp( long n, double a[], long lda, long k1, long k2,
-		   long ipiv[], long incx );
-extern void dgetrf( long m, long n, double a[], long lda, long ipiv[],
-		   long *info );
-extern void dgetrs( char trans, long n, long nrhs, double a[], long lda,
-		   long ipiv[], double b[], long ldb, long *info );
-extern void dgesv( long n, long nrhs, double a[], long lda, long ipiv[],
-		  double b[], long ldb, long *info );
-/*-----end of prototypes-----*/
-
-
-
 /*---blas routines---*/
-extern long  yidamax(long n,double dx[],long incx);
-extern void  ydswap (long n,double dx[],long incx,double dy[],long incy);
-extern void  ydscal(long n,double da,double dx[],long incx);
-extern void ydger  ( long m, long n, double alpha, double x[], long incx,
-		   double y[], long incy, double a[], long lda );
-extern void ydgemm ( char transa, char transb, long m, long n, long k,
-		   double alpha, double a[], long lda, double b[], long ldb,
-		   double beta, double c[], long ldc );
-extern void ydtrsm ( char side, char uplo, char transa, char diag,
-		   long m, long n, double alpha, double a[], long lda,
-		   double b[], long ldb );
+/* idamax dswap dscal dger dgemm dtrsm */
 
 
 
 /*---converted nutty string switches to single characters (lower case)---*/
 #define lsame(x,y) ((x)==(y))
 
-extern void xerbla(char *,long);
-
 extern int strcmp(const char *, const char *);
 
 
@@ -59,7 +29,7 @@ extern int strcmp(const char *, const char *);
 
 
 void dgesv( long n, long nrhs, double a[], long lda, long ipiv[],
-	   double b[], long ldb, long *info )
+           double b[], long ldb, long *info )
 {
   /**
    *  -- LAPACK driver routine (version 1.1) --
@@ -164,7 +134,7 @@ void dgesv( long n, long nrhs, double a[], long lda, long ipiv[],
      *        Solve the system A*X = B, overwriting B with X.
      **/
     dgetrs( 'n'/*o transpose*/, n, nrhs, a, lda, ipiv, b, ldb,
-	   info );
+           info );
   }
   return;
   /**
@@ -175,7 +145,7 @@ void dgesv( long n, long nrhs, double a[], long lda, long ipiv[],
 
 
 void dgetrs( char trans, long n, long nrhs, double a[], long lda,
-	    long ipiv[], double b[], long ldb, long *info )
+            long ipiv[], double b[], long ldb, long *info )
 {
   /**
    *  -- LAPACK routine (version 1.1) --
@@ -292,26 +262,26 @@ void dgetrs( char trans, long n, long nrhs, double a[], long lda,
     /**
      *        Solve L*X = B, overwriting B with X.
      **/
-    ydtrsm( 'l'/*eft*/, 'l'/*ower*/, 'n'/*o transpose*/, 'u'/*nit*/, n, nrhs,
-	  one, a, lda, b, ldb );
+    cblas_dtrsm(CblasColMajor, CblasLeft, CblasLower, CblasNoTrans,
+                CblasUnit, n, nrhs, one, a, lda, b, ldb );
     /**
      *        Solve U*X = B, overwriting B with X.
      **/
-    ydtrsm( 'l'/*eft*/, 'u'/*pper*/, 'n'/*o transpose*/, 'n'/*on-unit*/, n,
-	  nrhs, one, a, lda, b, ldb );
+    cblas_dtrsm(CblasColMajor, CblasLeft, CblasUpper, CblasNoTrans,
+                CblasNonUnit, n, nrhs, one, a, lda, b, ldb );
   } else {
     /**
      *        Solve A' * X = B.
      *
      *        Solve U'*X = B, overwriting B with X.
      **/
-    ydtrsm( 'l'/*eft*/, 'u'/*pper*/, 't'/*ranspose*/, 'n'/*on-unit*/, n, nrhs,
-	  one, a, lda, b, ldb );
+    cblas_dtrsm(CblasColMajor, CblasLeft, CblasUpper, CblasTrans,
+                CblasNonUnit, n, nrhs, one, a, lda, b, ldb );
     /**
      *        Solve L'*X = B, overwriting B with X.
      **/
-    ydtrsm( 'l'/*eft*/, 'l'/*ower*/, 't'/*ranspose*/, 'u'/*nit*/, n, nrhs, one,
-	  a, lda, b, ldb );
+    cblas_dtrsm(CblasColMajor, CblasLeft, CblasLower, CblasTrans,
+                CblasUnit, n, nrhs, one, a, lda, b, ldb );
     /**
      *        Apply row interchanges to the solution vectors.
      **/
@@ -445,9 +415,9 @@ void dgetrf( long m, long n, double a[], long lda, long ipiv[], long *info )
        *           Adjust INFO and the pivot indices.
        **/
       if( *info==0 && iinfo>0 )
-	*info = iinfo + j - 1;
+        *info = iinfo + j - 1;
       for (i=j ; i<=min( m, j+jb-1 ) ; i+=1) {
-	ipiv_1( i ) = j - 1 + ipiv_1( i );
+        ipiv_1( i ) = j - 1 + ipiv_1( i );
       }
       /**
        *           Apply interchanges to columns 1:J-1.
@@ -455,26 +425,25 @@ void dgetrf( long m, long n, double a[], long lda, long ipiv[], long *info )
       dlaswp( j-1, a, lda, j, j+jb-1, ipiv, 1 );
 
       if( j+jb<=n ) {
-	/**
-	 *              Apply interchanges to columns J+JB:N.
-	 **/
-	dlaswp( n-j-jb+1, &a_2( 1, j+jb ), lda, j, j+jb-1,
-	       ipiv, 1 );
-	/**
-	 *              Compute block row of U.
-	 **/
-	ydtrsm( 'l'/*eft*/, 'l'/*ower*/, 'n'/*o transpose*/, 'u'/*nit*/, jb,
-	      n-j-jb+1, one, &a_2( j, j ), lda, &a_2( j, j+jb ),
-	      lda );
-	if( j+jb<=m ) {
-	  /**
-	   *                 Update trailing submatrix.
-	   **/
-	  ydgemm( 'n'/*o transpose*/, 'n'/*o transpose*/, m-j-jb+1,
-		n-j-jb+1, jb, -one, &a_2( j+jb, j ), lda,
-		&a_2( j, j+jb ), lda, one, &a_2( j+jb, j+jb ),
-		lda );
-	}
+        /**
+         *              Apply interchanges to columns J+JB:N.
+         **/
+        dlaswp( n-j-jb+1, &a_2( 1, j+jb ), lda, j, j+jb-1,
+               ipiv, 1 );
+        /**
+         *              Compute block row of U.
+         **/
+        cblas_dtrsm(CblasColMajor, CblasLeft, CblasLower, CblasNoTrans,
+                    CblasUnit, jb, n-j-jb+1, one, &a_2( j, j ), lda,
+                    &a_2( j, j+jb ), lda );
+        if( j+jb<=m ) {
+          /**
+           *                 Update trailing submatrix.
+           **/
+          cblas_dgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, m-j-jb+1,
+                      n-j-jb+1, jb, -one, &a_2( j+jb, j ), lda,
+                      &a_2( j, j+jb ), lda, one, &a_2( j+jb, j+jb ), lda );
+        }
       }
     }
   }
@@ -487,7 +456,7 @@ void dgetrf( long m, long n, double a[], long lda, long ipiv[], long *info )
 
 
 void dlaswp( long n, double a[], long lda, long k1, long k2,
-	    long ipiv[], long incx )
+            long ipiv[], long incx )
 {
   /**
    *  -- LAPACK auxiliary routine (version 1.1) --
@@ -563,20 +532,20 @@ void dlaswp( long n, double a[], long lda, long k1, long k2,
     for (i=k1 ; i<=k2 ; i+=1) {
       ip = ipiv_1( i );
       if( ip!=i )
-	ydswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
+        cblas_dswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
     }
   } else if( incx>1 ) {
     for (i=k1 ; i<=k2 ; i+=1) {
       ip = ipiv_1( ix );
       if( ip!=i )
-	ydswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
+        cblas_dswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
       ix = ix + incx;
     }
   } else if( incx<0 ) {
     for (i=k2 ; i>=k1 ; i+=-1) {
       ip = ipiv_1( ix );
       if( ip!=i )
-	ydswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
+        cblas_dswap( n, &a_2( i, 1 ), lda, &a_2( ip, 1 ), lda );
       ix = ix + incx;
     }
   }
@@ -691,19 +660,19 @@ void dgetf2( long m, long n, double a[], long lda, long ipiv[], long *info )
     /**
      *        Find pivot and test for singularity.
      **/
-    jp = j - 1 + yidamax( m-j+1, &a_2( j, j ), 1 );
+    jp = j + cblas_idamax( m-j+1, &a_2( j, j ), 1 );
     ipiv_1( j ) = jp;
     if( a_2( jp, j )!=zero ) {
       /**
        *           Apply the interchange to columns 1:N.
        **/
       if( jp!=j )
-	ydswap( n, &a_2( j, 1 ), lda, &a_2( jp, 1 ), lda );
+        cblas_dswap( n, &a_2( j, 1 ), lda, &a_2( jp, 1 ), lda );
       /**
        *           Compute elements J+1:M of J-th column.
        **/
       if( j<m )
-	ydscal( m-j, one / a_2( j, j ), &a_2( j+1, j ), 1 );
+        cblas_dscal( m-j, one / a_2( j, j ), &a_2( j+1, j ), 1 );
 
     } else if( *info==0 ) {
 
@@ -714,8 +683,8 @@ void dgetf2( long m, long n, double a[], long lda, long ipiv[], long *info )
       /**
        *           Update trailing submatrix.
        **/
-      ydger( m-j, n-j, -one, &a_2( j+1, j ), 1, &a_2( j, j+1 ), lda,
-	   &a_2( j+1, j+1 ), lda );
+      cblas_dger(CblasColMajor, m-j, n-j, -one, &a_2( j+1, j ), 1,
+                 &a_2( j, j+1 ), lda, &a_2( j+1, j+1 ), lda );
     }
   }
   return;
@@ -727,7 +696,7 @@ void dgetf2( long m, long n, double a[], long lda, long ipiv[], long *info )
 
 
 long ilaenv( long ispec, char *name, char *opts,
-	    long n1, long n2, long n3,long n4 )
+            long n1, long n2, long n3,long n4 )
 {
   /**
    *  -- LAPACK auxiliary routine (preliminary version) --
@@ -870,8 +839,8 @@ long ilaenv( long ispec, char *name, char *opts,
     if( ic>=97 && ic<=122 ) {
       subnam[0] = ic-32;
       for (i=2 ; i<=6 ; i+=1) {
-	ic = subnam[i-1];
-	if( ic>=97 && ic<=122 ) subnam[i-1] = ic-32;
+        ic = subnam[i-1];
+        if( ic>=97 && ic<=122 ) subnam[i-1] = ic-32;
       }
     }
 
@@ -884,10 +853,10 @@ long ilaenv( long ispec, char *name, char *opts,
        ( ic>=162 && ic<=169 ) ) {
       subnam[0] = ic+64;
       for (i=2 ; i<=6 ; i+=1) {
-	ic = subnam[i-1];
-	if( ( ic>=129 && ic<=137 ) ||
-	   ( ic>=145 && ic<=153 ) ||
-	   ( ic>=162 && ic<=169 ) ) subnam[i-1] = ic+64;
+        ic = subnam[i-1];
+        if( ( ic>=129 && ic<=137 ) ||
+           ( ic>=145 && ic<=153 ) ||
+           ( ic>=162 && ic<=169 ) ) subnam[i-1] = ic+64;
       }
     }
 
@@ -898,8 +867,8 @@ long ilaenv( long ispec, char *name, char *opts,
     if( ic>=225 && ic<=250 ) {
       subnam[0] = ic-32;
       for (i=2 ; i<=6 ; i+=1) {
-	ic = subnam[i-1];
-	if( ic>=225 && ic<=250 ) subnam[i-1] = ic-32;
+        ic = subnam[i-1];
+        if( ic>=225 && ic<=250 ) subnam[i-1] = ic-32;
       }
     }
   }
@@ -932,50 +901,50 @@ long ilaenv( long ispec, char *name, char *opts,
   if( !strcmp(c2,"GE") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     } else if( !strcmp(c3,"QRF") || !strcmp(c3,"RQF") || !strcmp(c3,"LQF") ||
-	      !strcmp(c3,"QLF") ) {
+              !strcmp(c3,"QLF") ) {
       if( sname ) {
-	nb = 32;
+        nb = 32;
       } else {
-	nb = 32;
+        nb = 32;
       }
     } else if( !strcmp(c3,"HRD") ) {
       if( sname ) {
-	nb = 32;
+        nb = 32;
       } else {
-	nb = 32;
+        nb = 32;
       }
     } else if( !strcmp(c3,"BRD") ) {
       if( sname ) {
-	nb = 32;
+        nb = 32;
       } else {
-	nb = 32;
+        nb = 32;
       }
     } else if( !strcmp(c3,"TRI") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     }
   } else if( !strcmp(c2,"PO") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     }
   } else if( !strcmp(c2,"SY") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     } else if( sname && !strcmp(c3,"TRD") ) {
       nb = 1;
@@ -993,77 +962,77 @@ long ilaenv( long ispec, char *name, char *opts,
   } else if( sname && !strcmp(c2,"OR") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nb = 32;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nb = 32;
       }
     } else if( c3[0]=='M' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nb = 32;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nb = 32;
       }
     }
   } else if( cname && !strcmp(c2,"UN") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nb = 32;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nb = 32;
       }
     } else if( c3[0]=='M' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nb = 32;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nb = 32;
       }
     }
   } else if( !strcmp(c2,"GB") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	if( n4<=64 ) {
-	  nb = 1;
-	} else {
-	  nb = 32;
-	}
+        if( n4<=64 ) {
+          nb = 1;
+        } else {
+          nb = 32;
+        }
       } else {
-	if( n4<=64 ) {
-	  nb = 1;
-	} else {
-	  nb = 32;
-	}
+        if( n4<=64 ) {
+          nb = 1;
+        } else {
+          nb = 32;
+        }
       }
     }
   } else if( !strcmp(c2,"PB") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	if( n2<=64 ) {
-	  nb = 1;
-	} else {
-	  nb = 32;
-	}
+        if( n2<=64 ) {
+          nb = 1;
+        } else {
+          nb = 32;
+        }
       } else {
-	if( n2<=64 ) {
-	  nb = 1;
-	} else {
-	  nb = 32;
-	}
+        if( n2<=64 ) {
+          nb = 1;
+        } else {
+          nb = 32;
+        }
       }
     }
   } else if( !strcmp(c2,"TR") ) {
     if( !strcmp(c3,"TRI") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     }
   } else if( !strcmp(c2,"LA") ) {
     if( !strcmp(c3,"UUM") ) {
       if( sname ) {
-	nb = 64;
+        nb = 64;
       } else {
-	nb = 64;
+        nb = 64;
       }
     }
   } else if( sname && !strcmp(c2,"ST") ) {
@@ -1083,35 +1052,35 @@ long ilaenv( long ispec, char *name, char *opts,
     if( !strcmp(c3,"QRF") || !strcmp(c3,"RQF") || !strcmp(c3,"LQF") ||
        !strcmp(c3,"QLF") ) {
       if( sname ) {
-	nbmin = 2;
+        nbmin = 2;
       } else {
-	nbmin = 2;
+        nbmin = 2;
       }
     } else if( !strcmp(c3,"HRD") ) {
       if( sname ) {
-	nbmin = 2;
+        nbmin = 2;
       } else {
-	nbmin = 2;
+        nbmin = 2;
       }
     } else if( !strcmp(c3,"BRD") ) {
       if( sname ) {
-	nbmin = 2;
+        nbmin = 2;
       } else {
-	nbmin = 2;
+        nbmin = 2;
       }
     } else if( !strcmp(c3,"TRI") ) {
       if( sname ) {
-	nbmin = 2;
+        nbmin = 2;
       } else {
-	nbmin = 2;
+        nbmin = 2;
       }
     }
   } else if( !strcmp(c2,"SY") ) {
     if( !strcmp(c3,"TRF") ) {
       if( sname ) {
-	nbmin = 2;
+        nbmin = 2;
       } else {
-	nbmin = 2;
+        nbmin = 2;
       }
     } else if( sname && !strcmp(c3,"TRD") ) {
       nbmin = 2;
@@ -1123,29 +1092,29 @@ long ilaenv( long ispec, char *name, char *opts,
   } else if( sname && !strcmp(c2,"OR") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nbmin = 2;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nbmin = 2;
       }
     } else if( c3[0]=='M' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nbmin = 2;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nbmin = 2;
       }
     }
   } else if( cname && !strcmp(c2,"UN") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nbmin = 2;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nbmin = 2;
       }
     } else if( c3[0]=='M' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nbmin = 2;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nbmin = 2;
       }
     }
   }
@@ -1161,21 +1130,21 @@ long ilaenv( long ispec, char *name, char *opts,
     if( !strcmp(c3,"QRF") || !strcmp(c3,"RQF") || !strcmp(c3,"LQF") ||
        !strcmp(c3,"QLF") ) {
       if( sname ) {
-	nx = 128;
+        nx = 128;
       } else {
-	nx = 128;
+        nx = 128;
       }
     } else if( !strcmp(c3,"HRD") ) {
       if( sname ) {
-	nx = 128;
+        nx = 128;
       } else {
-	nx = 128;
+        nx = 128;
       }
     } else if( !strcmp(c3,"BRD") ) {
       if( sname ) {
-	nx = 128;
+        nx = 128;
       } else {
-	nx = 128;
+        nx = 128;
       }
     }
   } else if( !strcmp(c2,"SY") ) {
@@ -1189,17 +1158,17 @@ long ilaenv( long ispec, char *name, char *opts,
   } else if( sname && !strcmp(c2,"OR") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nx = 128;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nx = 128;
       }
     }
   } else if( cname && !strcmp(c2,"UN") ) {
     if( c3[0]=='G' ) {
       if( !strcmp(c4,"QR") || !strcmp(c4,"RQ") || !strcmp(c4,"LQ") ||
-	 !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
-	 !strcmp(c4,"BR") ) {
-	nx = 128;
+         !strcmp(c4,"QL") || !strcmp(c4,"HR") || !strcmp(c4,"TR") ||
+         !strcmp(c4,"BR") ) {
+        nx = 128;
       }
     }
   }
