@@ -182,3 +182,9 @@ p_rename(const char *unix_old, const char *unix_new)
   strncat(old, w_pathname(unix_old), P_WKSIZ);
   return -(!MoveFile(old, w_pathname(unix_new)));
 }
+
+char *
+p_native(const char *unix_name)
+{
+  return p_strcpy(w_pathname(unix_name));
+}
