@@ -32,28 +32,12 @@ static char *clip_text = 0;
 static HWND clip_owner = 0;
 static void clip_free(int force);
 
-LONG APIENTRY WndProc ( 
-    HWND         hWnd, 
-    UINT     iMessage, 
-    UINT         wParam, 
-    LONG         lParam) 
-{
-  PAINTSTRUCT ps;
-    switch (iMessage) {
-    case WM_PAINT:
-      BeginPaint(hWnd, &ps);
-      EndPaint(hWnd, &ps);
-      break;
-    default:
-      return DefWindowProc (hWnd, iMessage, wParam, lParam) ; 
-    }
-    return 0;
-}
+void (*p_on_connect)(int dis, int fd) = 0;
 
 p_scr *
 p_connect(char *server_name)
 {
-  WNDCLASSEX class_data;
+  static int registered = 0;
   HDC dc0 = GetDC(0);
   RECT r;
   int i;
@@ -72,7 +56,9 @@ p_connect(char *server_name)
     w_screen.height = r.bottom - r.top;
   }
 
-  w_screen.does_linetypes = GetDeviceCaps(dc0, LINECAPS) & LC_WIDESTYLED;
+  /* windows linetypes are unusably ugly for thin lines */
+  /*w_screen.does_linetypes = GetDeviceCaps(dc0, LINECAPS) & LC_WIDESTYLED;*/
+  w_screen.does_linetypes = 0;
   w_screen.does_rotext = GetDeviceCaps(dc0, TEXTCAPS) & TC_CR_90;
 
   w_screen.sys_colors[255-P_BLACK] = RGB(0,0,0);
@@ -85,10 +71,10 @@ p_connect(char *server_name)
   w_screen.sys_colors[255-P_YELLOW] = RGB(255,255,0);
   w_screen.sys_colors[255-P_BG] = GetSysColor(COLOR_WINDOW);
   w_screen.sys_colors[255-P_FG] = GetSysColor(COLOR_WINDOWTEXT);
-  w_screen.sys_colors[255-P_GUI_BG] = GetSysColor(COLOR_BTNFACE);
-  w_screen.sys_colors[255-P_GUI_FG] = GetSysColor(COLOR_BTNTEXT);
-  w_screen.sys_colors[255-P_GUI_LO] = GetSysColor(COLOR_BTNSHADOW);
-  w_screen.sys_colors[255-P_GUI_HI] = GetSysColor(COLOR_BTNHILIGHT);
+  w_screen.sys_colors[255-P_GRAYD] = RGB(100,100,100);
+  w_screen.sys_colors[255-P_GRAYC] = RGB(150,150,150);
+  w_screen.sys_colors[255-P_GRAYB] = RGB(190,190,190);
+  w_screen.sys_colors[255-P_GRAYA] = RGB(214,214,214);
   w_screen.sys_colors[255-P_XOR] = 0xffffff;  /* really fg^bg, but unused */
   i = GetDeviceCaps(dc0, NUMRESERVED);
   if (w_screen.depth==8 && i>1 && i<=32 &&
@@ -137,6 +123,7 @@ p_connect(char *server_name)
     w_screen.font_order[i] = i;
     w_screen.font_cache[i].hfont = 0;
   }
+  w_screen.font_win = 0;
 
   {
     char *sys_cursor[P_NONE] = {
@@ -153,37 +140,52 @@ p_connect(char *server_name)
 
   w_screen.first_menu = w_screen.active = 0;
 
-  class_data.cbSize = sizeof(WNDCLASSEX);
-  class_data.style = CS_OWNDC;
-  class_data.lpfnWndProc = (WNDPROC)w_winproc;
-  class_data.cbClsExtra = 0;
-  class_data.cbWndExtra = 0;
-  class_data.hInstance = w_app_instance;
-  class_data.hIcon = 0;
-  class_data.hCursor = 0;
-  class_data.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
-  class_data.lpszMenuName = 0;
-  class_data.lpszClassName = w_win_class;
-  class_data.hIconSm = 0;
-  if (!RegisterClassEx(&class_data)) return 0;
+  if (!registered) {
+    WNDCLASSEX class_data;
+    class_data.cbSize = sizeof(WNDCLASSEX);
+    class_data.style = CS_OWNDC;
+    class_data.lpfnWndProc = (WNDPROC)w_winproc;
+    class_data.cbClsExtra = 0;
+    class_data.cbWndExtra = 0;
+    class_data.hInstance = w_app_instance;
+    class_data.hIcon = 0;
+    class_data.hCursor = 0;
+    class_data.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
+    class_data.lpszMenuName = 0;
+    class_data.lpszClassName = w_win_class;
+    class_data.hIconSm = 0;
+    if (!RegisterClassEx(&class_data)) return 0;
 
-  class_data.cbSize = sizeof(WNDCLASSEX);
-  class_data.style = CS_OWNDC | CS_SAVEBITS;
-  class_data.lpfnWndProc = (WNDPROC)w_winproc;
-  class_data.cbClsExtra = 0;
-  class_data.cbWndExtra = 0;
-  class_data.hInstance = w_app_instance;
-  class_data.hIcon = 0;
-  class_data.hCursor = 0;
-  class_data.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
-  class_data.lpszMenuName = 0;
-  class_data.lpszClassName = w_menu_class;
-  class_data.hIconSm = 0;
-  if (!RegisterClassEx(&class_data)) return 0;
+    class_data.cbSize = sizeof(WNDCLASSEX);
+    class_data.style = CS_OWNDC | CS_SAVEBITS;
+    class_data.lpfnWndProc = (WNDPROC)w_winproc;
+    class_data.cbClsExtra = 0;
+    class_data.cbWndExtra = 0;
+    class_data.hInstance = w_app_instance;
+    class_data.hIcon = 0;
+    class_data.hCursor = 0;
+    class_data.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
+    class_data.lpszMenuName = 0;
+    class_data.lpszClassName = w_menu_class;
+    class_data.hIconSm = 0;
+    if (!RegisterClassEx(&class_data)) return 0;
+    registered = 1;
+  }
+
+  if (p_on_connect) p_on_connect(0, -1);
 
   return &w_screen;
 }
 
+/* expose MSWindows specific data required to create subwindow */
+HINSTANCE
+w_linker(LPCTSTR *pw_class, WNDPROC *pw_proc)
+{
+  *pw_proc = (WNDPROC)w_winproc;
+  *pw_class = w_win_class;
+  return w_app_instance;
+}
+
 int
 p_sshape(p_scr *s, int *width, int *height)
 {
@@ -204,6 +206,7 @@ void
 p_disconnect(p_scr *s)
 {
   int i;
+  if (p_on_connect) p_on_connect(1, -1);
   for (i=0 ; i<W_FONTS_CACHED && s->font_cache[i].hfont ; i++) {
     DeleteObject(s->font_cache[i].hfont);
     s->font_cache[i].hfont = 0;
@@ -301,6 +304,14 @@ p_raise(p_win *w)
   BringWindowToTop(w->w);
 }
 
+int w_nwins = 0;
+
+int
+p_wincount(p_scr *s)
+{
+  return (s==&w_screen)? w_nwins : 0;
+}
+
 LRESULT CALLBACK
 w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
 {
@@ -309,7 +320,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
     int button = 0;
 
     if (msg == WM_CHAR) {  /* horrible design */
-      if (won_key) {
+      if (won_key && pw->ctx) {
         int key = (int)(pw->keydown&&0xffff);
         if (!key) {
           int state = (int)(pw->keydown>>16);
@@ -328,7 +339,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
     switch (msg) {
     case WM_PAINT:
       /* call BeginPaint or ValidateRect to avoid another WM_PAINT */
-      if (won_expose) {
+      if (won_expose && pw->ctx) {
         RECT r;
         int xy[4];
         if (GetUpdateRect(hwnd, &r, 0)) {
@@ -349,7 +360,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
       p_clear(pw);
       return 1;
     case WM_SIZE:
-      if (won_resize && wp!=SIZE_MINIMIZED &&
+      if (won_resize && pw->ctx && wp!=SIZE_MINIMIZED &&
           wp!=SIZE_MAXHIDE && wp!=SIZE_MAXSHOW) {
         int width = LOWORD(lp);
         int height = HIWORD(lp);
@@ -375,7 +386,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
         DeleteObject(pw->palette);
         pw->palette = 0;
       }
-      if (won_destroy) won_destroy(pw->ctx);
+      if (won_destroy && pw->ctx) won_destroy(pw->ctx);
       pw->ctx = 0;
       SetWindowLong(hwnd, GWL_USERDATA, (LONG)0);
       if (pw->ancestor) {
@@ -397,7 +408,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
       }
       button = 1;
     case WM_KILLFOCUS:
-      if (won_focus)
+      if (won_focus && pw->ctx)
         won_focus(pw->ctx, button);
       return 0;
 
@@ -414,7 +425,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
       return button;
 
     case WM_KEYDOWN:
-      if (won_key) {  /* wp is virtual key code */
+      if (won_key && pw->ctx) {  /* wp is virtual key code */
         int key = 0;
         int state = 0; /* (int)((pw->keydown>>16)&P_META); */
         if (wp>=VK_NUMPAD0 && wp<=VK_F24) {
@@ -474,7 +485,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
       button++;
     case WM_LBUTTONUP:
       button++;
-      if (won_click) {
+      if (won_click && pw->ctx) {
         int x = LOWORD(lp);
         int y = HIWORD(lp);
         int state = 0;
@@ -493,7 +504,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
       return 0;
 
     case WM_MOUSEMOVE:
-      if (won_motion) {
+      if (won_motion && pw->ctx) {
         int x = LOWORD(lp);
         int y = HIWORD(lp);
         int state = 0;
@@ -521,7 +532,7 @@ w_winproc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
 
     case WM_DESTROYCLIPBOARD:
       clip_free(0);
-      if (won_deselect) won_deselect(pw->ctx);
+      if (won_deselect && pw->ctx) won_deselect(pw->ctx);
       return 0;
     }
 
