@@ -97,15 +97,6 @@ static PrtFunc *PrintD[]= { &FPrintD, &DPrintD };
 /* Two data types which are "foreign" to Yorick are defined in this
    file: the TextStream and the Bookmark.  */
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static UnaryOp PrintBM, PrintTX;
 
 /* Implement text streams as a foreign Yorick data type.  */
@@ -215,7 +206,7 @@ void Y_open(int nArgs)
     /* fail silently if optional errmode flag is set */
     p_free(fullname);
     permissions= 0;
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
 
   } else {
     /* blow up if optional errmode flag is not set */
@@ -247,7 +238,7 @@ void Y_close(int nArgs)
         (s->value.db->ops==&textOps || s->value.db->ops==&streamOps)) {
       s->ops= &intScalar;
       Unref(s->value.db);
-      s->value.db= Ref(&nilDB);
+      s->value.db= RefNC(&nilDB);
       s->ops= &dataBlockSym;
     }
   }
@@ -289,7 +280,6 @@ void Y_close(int nArgs)
  */
 
 extern void yr_reader(char *input_line);
-extern void yr_reset(void);
 extern Instruction *ym_suspend(void);
 extern void ym_resume(Instruction *pc);
 extern char *y_read_prompt;
@@ -336,21 +326,19 @@ yr_reader(char *input_line)
     yr_n--;
   } else if (yr_n > 0) {           /* read */
     yr_nlines++;
-    while (yr_n) {
-      /* note that if Scanner sets input_line to 0, this loop quickly
-       * zips along until yr_n == 0 */
-      while (!Scanner[fmtList[yr_j].typeID](&ioOps[yr_j].o,
-                                            fmtList[yr_j].format,
-                                            &input_line) && yr_n) {
-        yr_j++;
-        if (yr_j >= yr_nargs) {
-          yr_n--;
-          yr_j = 0;
-        }
-        if (!input_line[0]) goto finis;
+    /* Scanner returns 0 if found, 1 if not found,
+     * sets input_line==0 on matching failure, which should abort read */
+    while (!Scanner[fmtList[yr_j].typeID](&ioOps[yr_j].o,
+                                          fmtList[yr_j].format,
+                                          &input_line)) {
+      yr_j++;
+      if (yr_j >= yr_nargs) {
+        yr_n--;
+        if (!yr_n) break;
+        yr_j = 0;
       }
     }
-  finis:;
+    if (!input_line) yr_n = 0;  /* halt on matching failure */
   }
   if (p_signalling) p_abort();
   if (yr_n <= 0) {
@@ -477,7 +465,8 @@ ReadWorker(Operand *sourceOp, Symbol *stack, char *format, Symbol *stack0)
       ioOps[nArgs].s.ops = ioOps[nArgs].o.owner->ops;
       if (ioOps[nArgs].s.ops != &dataBlockSym) {
         /* int, long, double scalars read directly into ioOps */
-        if (s!=stack) ioOps[nArgs].s.index = stack->index;
+        if (s!=stack) ioOps[nArgs].s.index = stack->index,
+                        ioOps[nArgs].s.value = globTab[stack->index].value;
         ioOps[nArgs].o.value = &ioOps[nArgs].s.value;
       }
     }
@@ -575,7 +564,7 @@ void Y_rdline(int nArgs)
   long i, nLines= 0;
   Array *result;
   Dimension *dims;
-  char *keyNames[2];
+  char *keyNames[2], *q;
   Symbol *keySymbols[1];
   Symbol *stack0= sp-nArgs;
 
@@ -639,8 +628,11 @@ void Y_rdline(int nArgs)
     return;
   }
 
-  for (i=0 ; i<nLines ; i++)
-    result->value.q[i]= p_strcpy(NextInLine(&op, i));
+  for (i=0 ; i<nLines ; i++) {
+    q = NextInLine(&op, i);
+    if (!q) break;
+    result->value.q[i] = p_strcpy(q);
+  }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -819,29 +811,27 @@ WriteWorker(Operand *sinkOp, Symbol *stack, char *format)
     if (!sinkOp) {
       result->value.q[i]= p_strcpy(outBuf);
     } else {
-      /* YPrompt does NOT append \n, as desired (YputsOut does) */
-      if (i && ((nArgs>nConversions && nArgs>1) || nLine>lineSize)) {
-        if (sinkOp->ops) {
-          if (file) p_fputs(file, "\n");
-          else p_stdout("\n");
-        } else YPrompt("\n");
+      /* extra line break explained in help,write documentation
+       * nLine>nChars not mentioned in documentation prevents
+       * extra line break if single argument is longer than lineSize
+       */
+      if (i && ((nArgs>nConversions && nArgs>1) ||
+                (nLine>lineSize && nLine>nChars))) {
+        if (sinkOp->ops && file) p_fputs(file, "\n");
+        else p_stdout("\n");
         fmtTotal++;
         nLine= strlen(outBuf);
       }
-      if (sinkOp->ops) {
-        if (file) p_fputs(file, outBuf);
-        else p_stdout(outBuf);
-      } else YPrompt(outBuf);
+      if (sinkOp->ops && file) p_fputs(file, outBuf);
+      else p_stdout(outBuf);
       if (nChars && text[-1]=='\n') nLine= 0;
     }
   }
 
   /* add trailing newline if it seems reasonable to do so */
   if (sinkOp && nArgs>nConversions) {
-    if (sinkOp->ops) {
-      if (file) p_fputs(file, "\n");
-      else p_stdout("\n");
-    } else YPrompt("\n");
+    if (sinkOp->ops && file) p_fputs(file, "\n");
+    else p_stdout("\n");
     fmtTotal++;
   }
 
@@ -862,7 +852,7 @@ IOFileLink *yTextFiles= 0;
    at a time.  Since TextStream contains several pointers, the alignment
    of an TextStream must be at least as strict as a void*.  */
 static MemryBlock txtsBlock= {0, 0, sizeof(TextStream),
-                                  16*sizeof(TextStream)};
+                              16*sizeof(TextStream)};
 
 TextStream *NewTextStream(char *fullname, void *stream, int permissions,
                           long line, long pos)
@@ -1042,10 +1032,11 @@ static char *NextInLine(Operand *op, long n)
         YErrorIO("****ABORTING READ**** error reading input file");
       if (!hadEOF)
         YErrorIO("****ABORTING READ**** input file not ASCII text");
+    } else {
+      stream->lastLineRead++;
+      stream->lastPosition= stream->readPosition;
+      if (!(stream->permissions&16)) stream->readPosition= p_ftell(file);
     }
-    stream->lastLineRead++;
-    stream->lastPosition= stream->readPosition;
-    if (!(stream->permissions&16)) stream->readPosition= p_ftell(file);
   } else if (op->ops==&stringOps) {
     char **q= op->value;
     if (n<op->type.number) text= q[n];
@@ -1067,7 +1058,7 @@ static int opname(Operand *op, char *format, char **text) \
   type2 v;  int i, n; \
   if (*text) { \
     i= sscanf(*text, format, &v, &n); \
-    if (i==1) { *x= v; fmtAssigns++; *text+= n; i= 0; \
+    if (i==1) { *x= (type1)v; fmtAssigns++; *text+= n; i= 0; \
     } else if (i==0)    {         *text= 0;     i= 1; \
     } else {         *text+= strlen(*text);     i= 1; \
     } \
@@ -1093,7 +1084,7 @@ static int opname(Operand *op, char *format, char **text) \
   type2 v;  int i, n; \
   if (*text) { \
     i= retry_sscanf(*text, format, &v, &n); \
-    if (i==1) { *x= v; fmtAssigns++; *text+= n; i= 0; \
+    if (i==1) { *x= (type1)v; fmtAssigns++; *text+= n; i= 0; \
     } else if (i==0)    {         *text= 0;     i= 1; \
     } else {         *text+= strlen(*text);     i= 1; \
     } \
@@ -1153,15 +1144,17 @@ static int NilScanner(Operand *op, char *format, char **text)
 static int
 retry_sscanf(char *text, char *format, double *pv, int *pn)
 {
-  int i= sscanf(text, format, pv, pn);
-  int n= i? *pn : 0;
-  char c= text[n];
-  if (i==1 && (c=='D' || c=='d')) {
-    /* this may represent only limited success if the scan
-     * stopped at a "d" or "D" character */
-    text[n]= 'e';
-    i= sscanf(text, format, pv, pn);
-    text[n]= c;
+  int i = sscanf(text, format, pv, pn);
+  if (i == 1) {
+    int n = *pn;
+    char c = text[n];
+    if (c=='D' || c=='d') {
+      /* this may represent only limited success if the scan
+       * stopped at a "d" or "D" character */
+      text[n] = 'e';
+      i = sscanf(text, format, pv, pn);
+      text[n] = c;
+    }
   }
   return i;
 }
