@@ -21,47 +21,49 @@
 extern VMaction Power, Multiply, Divide, Modulo, Add, Subtract,
   ShiftL, ShiftR, Less, Greater, LessEQ, GreaterEQ, Equal, NotEqual;
 
+/* ..X versions now declared in ydata.h */
+
 extern StackOp AddII, AddIL, AddID, AddIB,
                AddLI, AddLL, AddLD, AddLB,
                AddDI, AddDL, AddDD, AddDB,
                AddBI, AddBL, AddBD, AddBB;
 extern BinaryOp AddC, AddS, AddI, AddL,
-                AddF, AddD, AddZ, AddQ, AddX;
+                AddF, AddD, AddZ, AddQ;
 
 extern StackOp SubtractII, SubtractIL, SubtractID, SubtractIB,
                SubtractLI, SubtractLL, SubtractLD, SubtractLB,
                SubtractDI, SubtractDL, SubtractDD, SubtractDB,
                SubtractBI, SubtractBL, SubtractBD, SubtractBB;
 extern BinaryOp SubtractC, SubtractS, SubtractI, SubtractL,
-                SubtractF, SubtractD, SubtractZ, SubtractX;
+                SubtractF, SubtractD, SubtractZ;
 
 extern StackOp MultiplyII, MultiplyIL, MultiplyID, MultiplyIB,
                MultiplyLI, MultiplyLL, MultiplyLD, MultiplyLB,
                MultiplyDI, MultiplyDL, MultiplyDD, MultiplyDB,
                MultiplyBI, MultiplyBL, MultiplyBD, MultiplyBB;
 extern BinaryOp MultiplyC, MultiplyS, MultiplyI, MultiplyL,
-                MultiplyF, MultiplyD, MultiplyZ, MultiplyX;
+                MultiplyF, MultiplyD, MultiplyZ;
 
 extern StackOp DivideII, DivideIL, DivideID, DivideIB,
                DivideLI, DivideLL, DivideLD, DivideLB,
                DivideDI, DivideDL, DivideDD, DivideDB,
                DivideBI, DivideBL, DivideBD, DivideBB;
 extern BinaryOp DivideC, DivideS, DivideI, DivideL,
-                DivideF, DivideD, DivideZ, DivideX;
+                DivideF, DivideD, DivideZ;
 
 extern StackOp ModuloII, ModuloIL, ModuloID, ModuloIB,
                ModuloLI, ModuloLL, ModuloLD, ModuloLB,
                ModuloDI, ModuloDL, ModuloDD, ModuloDB,
                ModuloBI, ModuloBL, ModuloBD, ModuloBB;
 extern BinaryOp ModuloC, ModuloS, ModuloI, ModuloL,
-                ModuloF, ModuloD, ModuloZ, ModuloX;
+                ModuloF, ModuloD, ModuloZ;
 
 extern StackOp PowerII, PowerIL, PowerID, PowerIB,
                PowerLI, PowerLL, PowerLD, PowerLB,
                PowerDI, PowerDL, PowerDD, PowerDB,
                PowerBI, PowerBL, PowerBD, PowerBB;
 extern BinaryOp PowerC, PowerS, PowerI, PowerL,
-                PowerF, PowerD, PowerZ, PowerX;
+                PowerF, PowerD, PowerZ;
 
 extern BinaryOp PowerXF, PowerXD, PowerXZ;
 
@@ -70,7 +72,7 @@ extern StackOp GreaterII, GreaterIL, GreaterID, GreaterIB,
                GreaterDI, GreaterDL, GreaterDD, GreaterDB,
                GreaterBI, GreaterBL, GreaterBD, GreaterBB;
 extern BinaryOp GreaterC, GreaterS, GreaterI, GreaterL,
-                GreaterF, GreaterD, GreaterZ, GreaterQ, GreaterX;
+                GreaterF, GreaterD, GreaterZ, GreaterQ;
 
 extern StackOp LessII, LessIL, LessID, LessIB,
                LessLI, LessLL, LessLD, LessLB,
@@ -82,7 +84,7 @@ extern StackOp GreaterEQII, GreaterEQIL, GreaterEQID, GreaterEQIB,
                GreaterEQDI, GreaterEQDL, GreaterEQDD, GreaterEQDB,
                GreaterEQBI, GreaterEQBL, GreaterEQBD, GreaterEQBB;
 extern BinaryOp GreaterEQC, GreaterEQS, GreaterEQI, GreaterEQL,
-                GreaterEQF, GreaterEQD, GreaterEQZ, GreaterEQQ, GreaterEQX;
+                GreaterEQF, GreaterEQD, GreaterEQZ, GreaterEQQ;
 
 extern StackOp LessEQII, LessEQIL, LessEQID, LessEQIB,
                LessEQLI, LessEQLL, LessEQLD, LessEQLB,
@@ -95,7 +97,7 @@ extern StackOp EqualII, EqualIL, EqualID, EqualIB,
                EqualBI, EqualBL, EqualBD, EqualBB;
 extern BinaryOp EqualC, EqualS, EqualI, EqualL,
                 EqualF, EqualD, EqualZ, EqualQ, EqualP,
-                EqualSI, EqualR, EqualX;
+                EqualSI, EqualR;
 
 extern StackOp NotEqualII, NotEqualIL, NotEqualID, NotEqualIB,
                NotEqualLI, NotEqualLL, NotEqualLD, NotEqualLB,
@@ -103,7 +105,7 @@ extern StackOp NotEqualII, NotEqualIL, NotEqualID, NotEqualIB,
                NotEqualBI, NotEqualBL, NotEqualBD, NotEqualBB;
 extern BinaryOp NotEqualC, NotEqualS, NotEqualI, NotEqualL,
                 NotEqualF, NotEqualD, NotEqualZ, NotEqualQ, NotEqualP,
-                NotEqualSI, NotEqualR, NotEqualX;
+                NotEqualSI, NotEqualR;
 
 /*--------------------------------------------------------------------------*/
 
@@ -456,7 +458,7 @@ void opname(Operand *l, Operand *r) \
     Popper(l->owner); \
   } else DivideError(); }
 
-OPERATION(DivideC, char, PopTo)
+OPERATION(DivideC, unsigned char, PopTo)
 OPERATION(DivideS, short, PopTo)
 OPERATION(DivideI, int, PopToI)
 OPERATION(DivideL, long, PopToL)
@@ -465,19 +467,21 @@ OPERATION(DivideD, double, PopToD)
 void DivideZ(Operand *l, Operand *r)
 { double *dst= BuildResult2(l, r);
   if (dst) {
-    long i, n= l->type.number;
-    double *lv= l->value, *rv= r->value;
-    double lr, li, rr, ri, nrm;          /* watch out for dst==lv or rv */
+    long i, n = l->type.number;
+    double *lv = l->value, *rv= r->value;
+    double lr, li, rr, ri;               /* watch out for dst==lv or rv */
     for (i=0 ; i<n ; i++) {
-      lr= lv[2*i];  li= lv[2*i+1];
-      rr= rv[2*i];  ri= rv[2*i+1];
+      lr = lv[2*i];  li = lv[2*i+1];
+      rr = rv[2*i];  ri = rv[2*i+1];
       if ((rr>0?rr:-rr)>(ri>0?ri:-ri)) { /* be careful about overflow... */
-	nrm=rr; rr=1.0; ri/=nrm; nrm*=1.0+ri*ri;
+        ri/=rr; rr=1.0/((1.0+ri*ri)*rr);
+        dst[2*i] = (lr+li*ri)*rr;
+        dst[2*i+1] = (li-lr*ri)*rr;
       } else {
-	nrm=ri; ri=1.0; rr/=nrm; nrm*=1.0+rr*rr;
+        rr/=ri; ri=1.0/((1.0+rr*rr)*ri);
+        dst[2*i] = (lr*rr+li)*ri;
+        dst[2*i+1] = (li*rr-lr)*ri;
       }
-      dst[2*i]= (lr*rr+li*ri)/nrm;
-      dst[2*i+1]= (li*rr-lr*ri)/nrm;
     }
     PopTo(l->owner);
   } else DivideError();
@@ -509,11 +513,11 @@ void ModuloLB(void)
 { Modulo_BB(FormOperandLS(sp-1, &lop), FormOperandDB(sp, &rop)); }
 
 void ModuloDI(void) { Symbol *spr= sp--;
-		      sp->value.d= RMOD(sp->value.d, spr->value.i); }
+                      sp->value.d= RMOD(sp->value.d, spr->value.i); }
 void ModuloDL(void) { Symbol *spr= sp--;
-		      sp->value.d= RMOD(sp->value.d, spr->value.l); }
+                      sp->value.d= RMOD(sp->value.d, spr->value.l); }
 void ModuloDD(void) { Symbol *spr= sp--;
-		      sp->value.d= RMOD(sp->value.d, spr->value.d); }
+                      sp->value.d= RMOD(sp->value.d, spr->value.d); }
 void ModuloDB(void)
 { Modulo_BB(FormOperandDS(sp-1, &lop), FormOperandDB(sp, &rop)); }
 
@@ -549,7 +553,7 @@ void opname(Operand *l, Operand *r) \
     Popper(l->owner); \
   } else ModuloError(); }
 
-OPERATION(ModuloC, char, PopTo)
+OPERATION(ModuloC, unsigned char, PopTo)
 OPERATION(ModuloS, short, PopTo)
 OPERATION(ModuloI, int, PopToI)
 OPERATION(ModuloL, long, PopToL)
@@ -605,8 +609,16 @@ void PowerLI(void) { Symbol *spr= sp--;
   sp->value.l= powLL(sp->value.l, spr->value.i); }
 void PowerLL(void) { Symbol *spr= sp--;
   sp->value.l= powLL(sp->value.l, spr->value.l); }
-void PowerLD(void) { Symbol *spr= sp--;
-  sp->value.d= pow(sp->value.l, spr->value.d); sp->ops= &doubleScalar; }
+void PowerLD(void) {
+  Symbol *spr= sp--;
+  errno = 0;
+  sp->value.d= pow(sp->value.l, spr->value.d);
+  if (errno) {
+    if (errno!=ERANGE || sp->value.d!=0.)
+      YError("mathlib pow() function signals error");
+  }
+  sp->ops= &doubleScalar;
+}
 void PowerLB(void)
 { Power_BB(FormOperandLS(sp-1, &lop), FormOperandDB(sp, &rop)); }
 
@@ -614,8 +626,15 @@ void PowerDI(void) { Symbol *spr= sp--;
   sp->value.d= powDL(sp->value.d, spr->value.i); }
 void PowerDL(void) { Symbol *spr= sp--;
   sp->value.d= powDL(sp->value.d, spr->value.l); }
-void PowerDD(void) { Symbol *spr= sp--;
-  sp->value.d= pow(sp->value.d, spr->value.d); }
+void PowerDD(void) {
+  Symbol *spr= sp--;
+  errno = 0;
+  sp->value.d= pow(sp->value.d, spr->value.d);
+  if (errno) {
+    if (errno!=ERANGE || sp->value.d!=0.)
+      YError("mathlib pow() function signals error");
+  }
+}
 void PowerDB(void)
 { Power_BB(FormOperandDS(sp-1, &lop), FormOperandDB(sp, &rop)); }
 
@@ -787,7 +806,7 @@ void opname(Operand *l, Operand *r) \
     PopToI(l->owner); \
   } else GreaterError(); }
 
-OPERATION(GreaterC, char)
+OPERATION(GreaterC, unsigned char)
 OPERATION(GreaterS, short)
 OPERATION(GreaterI, int)
 OPERATION(GreaterL, long)
@@ -931,7 +950,7 @@ void opname(Operand *l, Operand *r) \
     PopToI(l->owner); \
   } else GreaterEQError(); }
 
-OPERATION(GreaterEQC, char)
+OPERATION(GreaterEQC, unsigned char)
 OPERATION(GreaterEQS, short)
 OPERATION(GreaterEQI, int)
 OPERATION(GreaterEQL, long)
@@ -1123,9 +1142,9 @@ void EqualSI(Operand *l, Operand *r)
       long size= l->type.base->size;
       char *lv= l->value, *rv= r->value;
       for (i=0 ; i<n ; i++) {
-	dst[i]= memcmp(lv, rv, size)==0;
-	lv+= size;
-	rv+= size;
+        dst[i]= memcmp(lv, rv, size)==0;
+        lv+= size;
+        rv+= size;
       }
     } else {
       for (i=0 ; i<n ; i++) dst[i]= 0;
@@ -1265,9 +1284,9 @@ void NotEqualSI(Operand *l, Operand *r)
       long size= l->type.base->size;
       char *lv= l->value, *rv= r->value;
       for (i=0 ; i<n ; i++) {
-	dst[i]= memcmp(lv, rv, size)!=0;
-	lv+= size;
-	rv+= size;
+        dst[i]= memcmp(lv, rv, size)!=0;
+        lv+= size;
+        rv+= size;
       }
     } else {
       for (i=0 ; i<n ; i++) dst[i]= 1;
