@@ -23,22 +23,23 @@
            the minimum (or maximum) value along the index
        ptp:
            the difference between the maximum and minimum values
-	   along the index, positive if the maximum occurs for a
-	   larger index than the minimum, otherwise negative
+           along the index, positive if the maximum occurs for a
+           larger index than the minimum, otherwise negative
        sum:
            the sum of the values along the index
+           (concatentation for strings)
 
        avg:
        rms:
            the arithmetic mean (or square root of the arithmetic mean
-	   of the squares) of the values along the index, the result
-	   is always a double
+           of the squares) of the values along the index, the result
+           is always a double
 
        mnx:
        mxx:
            the index at which the minimum (or maximum) value along the
-	   index occured (the first such index if the extreme value
-	   occurs more than once), the result is always a long
+           index occured (the first such index if the extreme value
+           occurs more than once), the result is always a long
 
        Complex input is allowed for the sum and avg functions only.
 
@@ -46,24 +47,24 @@
     ---------------
        psum:
            the sums of the values at equal or lesser indices along the
-	   index
+           index
        dif:
            the pairwise differences between adjacent values along the
-	   index (there is one fewer pairwise difference than value)
+           index (there is one fewer pairwise difference than value)
        cum:
            psum, but first result value is 0 -- true inverse of dif
 
        zcen:
            the pairwise averages between adjacent values along the
-	   index (there is one fewer pairwise average than value),
-	   zcen is for "zone center"
+           index (there is one fewer pairwise average than value),
+           zcen is for "zone center"
        pcen:
            the pairwise averages between adjacent values along the
-	   index, with the first and last values copied (there is one
-	   more result than input than value), pcen is for "point center"
+           index, with the first and last values copied (there is one
+           more result than input than value), pcen is for "point center"
        uncp:
            the inverse of the pcen operation (there is one fewer result
-	   than input value), uncp is for "uncenter point"
+           than input value), uncp is for "uncenter point"
 
        Complex input is allowed for all rank preserving range functions.
 
@@ -74,6 +75,7 @@
 /* ------------------------------------------------------------------------ */
 
 #include "ydata.h"
+#include "pstdlib.h"
 
 /* A range function takes an Array and an integer indicating which index
    of the array to operate on (0 for fastest varying, 1 for next, etc.).
@@ -198,7 +200,7 @@ static StructDef *rfBase1[]= {
 
 static StructDef *rfBase3[]= {
   &longStruct, &longStruct, &longStruct, &longStruct,
-  &floatStruct, &doubleStruct, &complexStruct };
+  &floatStruct, &doubleStruct, &complexStruct, &stringStruct };
 
 static StructDef *rfBase4[]= {
   &doubleStruct, &doubleStruct, &doubleStruct, &doubleStruct,
@@ -421,20 +423,23 @@ RF_MXX(RFmxxD, double)
 static RawRF RFsumC, RFsumS, RFsumI, RFsumL, RFsumF, RFsumD;
 static RawRF *RawSum[]= {
   &RFsumC, &RFsumS, &RFsumI, &RFsumL, &RFsumF, &RFsumD };
+static void RFSumQ(Array *input, Array *result);
 
 int RFsum(Array *array, int index)
 {
   Operations *ops= array->ops;
   int typeID= ops->typeID;
   Array *result;
-  if (typeID > T_COMPLEX) YError("bad data type in sum: range function");
+  if (typeID>T_COMPLEX && typeID!=T_STRING)
+    YError("bad data type in sum: range function");
 
   /* set tmpDims to result dimensions */
   RankReduce(array->type.dims, index);
-  /* result data type long, float, or double (behavior #3) */
+  /* result data type long, float, or double, or string (behavior #3) */
   result= RFPush3(typeID);
 
-  if (typeID!=T_COMPLEX) RFLoop(RawSum[typeID], array, result);
+  if (typeID==T_STRING) RFSumQ(array, result);
+  else if (typeID!=T_COMPLEX) RFLoop(RawSum[typeID], array, result);
   else RFLoopC(&RFsumD, array, result);
   return 1;
 }
@@ -443,7 +448,7 @@ int RFsum(Array *array, int index)
 static void rfname(void *in, void *out) \
 { type1 *inp= in;  type2 *res= out; type3 cur= inp[0];  long i; \
   for (i=step ; i<number ; i+=step) cur+= inp[i]; \
-  res[0]= cur; }
+  res[0]= (type2)cur; }
 
 RF_SUM(RFsumC, unsigned char, long, long)
 RF_SUM(RFsumS, short, long, long)
@@ -452,6 +457,39 @@ RF_SUM(RFsumL, long, long, long)
 RF_SUM(RFsumF, float, float, double)
 RF_SUM(RFsumD, double, double, double)
 
+static void
+RFSumQ(Array *input, Array *result)
+{
+  char **inp = input->value.q;
+  char **out = result->value.q;
+  long inpNext = number-step;
+  /* long outNext = nout-step = 0; */
+  long i, j, k, m, len;
+
+  for (j=0 ; j<nslow ; j++) {
+    for (i=0 ; i<step ; i++) {
+      for (k=0,len=-1 ; k<number ; k+=step) {
+        if (!inp[k]) continue;
+        for (m=0 ; inp[k][m] ; m++);
+        if (len < 0) len = m;
+        else len += m;
+      }
+      if (len>=0) {
+        out[0] = p_malloc(len+1);
+        out[0][0] = '\0';
+        for (k=0,len=0 ; k<number ; k+=step) {
+          if (!inp[k]) continue;
+          for (m=0 ; (out[0][len] = inp[k][m]) ; len++,m++);
+        }
+      }
+      inp += 1;
+      out += 1;
+    }
+    inp += inpNext;
+    /* out += outNext; */
+  }
+}
+
 /* ---- ptp - returns peak-to-peak variation along index,
               >0 if minimum occurs before maximum, else <0 */
 
@@ -625,7 +663,7 @@ int RFavg(Array *array, int index)
 static void rfname(void *in, void *out) \
 { type1 *inp= in;  type2 *res= out; double cur= inp[0];  long i, n= 1; \
   for (i=step ; i<number ; i+=step) { cur+= inp[i]; n++; } \
-  res[0]= cur/n; }
+  res[0]= (type2)(cur/n); }
 
 RF_AVG(RFavgC, unsigned char, double)
 RF_AVG(RFavgS, short, double)
@@ -666,8 +704,8 @@ static void rfname(void *in, void *out) \
   if (mxa) { avg/= n;  tmp= ((type2)inp[0]-avg)/mxa;  cur= tmp*tmp; \
     for (i=step ; i<number ; i+=step) { \
       tmp= ((type2)inp[i]-avg)/mxa;  cur+= tmp*tmp; } \
-    res[0]= mxa*sqrt(cur/n); \
-  } else res[0]= 0.0; }
+    res[0]= (type2)(mxa*sqrt(cur/n)); \
+  } else res[0]= (type2)(0.0); }
 
 RF_RMS(RFrmsC, unsigned char, double)
 RF_RMS(RFrmsS, short, double)
@@ -704,7 +742,7 @@ int RFzcen(Array *array, int index)
 static void rfname(void *in, void *out) \
 { type1 *inp= in;  type2 *res= out;  long i; \
   for (i=0 ; i<number-step ; i+=step) res[i]= \
-    0.5*((type2)inp[i+step]+(type2)inp[i]); }
+    (type2)(0.5*((type2)inp[i+step]+(type2)inp[i])); }
 
 RF_ZCEN(RFzcenC, unsigned char, double)
 RF_ZCEN(RFzcenS, short, double)
@@ -741,7 +779,7 @@ static void rfname(void *in, void *out) \
 { type1 *inp= in;  type2 *res= out;  long i; \
   res[0]= inp[0]; \
   for (i=step ; i<number ; i+=step) res[i]= \
-    0.5*((type2)inp[i]+(type2)inp[i-step]); res[i]= inp[i-step]; }
+    (type2)(0.5*((type2)inp[i]+(type2)inp[i-step])); res[i]= inp[i-step]; }
 
 RF_PCEN(RFpcenC, unsigned char, double)
 RF_PCEN(RFpcenS, short, double)
@@ -777,8 +815,8 @@ int RFuncp(Array *array, int index)
 #define RF_UNCP(rfname, type1, type2) \
 static void rfname(void *in, void *out) \
 { type1 *inp= in;  type2 *res= out; double cur= inp[0];  long i; \
-  for (i=step,res[0]=cur ; i<number-step ; i+=step) \
-    res[i]= (cur= 2.0*inp[i]-cur); }
+  for (i=step,res[0]=(type2)cur ; i<number-step ; i+=step) \
+    res[i]= (type2)(cur= 2.0*inp[i]-cur); }
 
 RF_UNCP(RFuncpC, unsigned char, double)
 RF_UNCP(RFuncpS, short, double)
