@@ -37,7 +37,7 @@ extern BuiltIn Y_yorick_init, Y_set_path, Y_reshape, Y_array, Y_structof,
   Y_min, Y_max, Y_sum, Y_avg, Y_allof, Y_anyof, Y_noneof, Y_nallof, Y_where,
   Y_get_cwd, Y_get_home, Y_cd, Y_get_env, Y_get_argv, Y_use_origins;
 
-extern BuiltIn Y_set_site, Y_timer;
+extern BuiltIn Y_set_site, Y_timer, Y_get_path, Y_get_pkgnames;
 
 /*--------------------------------------------------------------------------*/
 
@@ -53,39 +53,37 @@ extern char **ym_argv;
 
 /*--------------------------------------------------------------------------*/
 
-static void YInit(BuiltIn **builtin, void **global,
-		  char **name, char **include);
+static void YInit(char *home, char *site, y_pkg_t **pkgs);
 
 extern int YGetLaunchDir(char *argv0);  /* sysdep.c */
-extern char *yLaunchDir;
-char *yLaunchDir= 0;
 
-/* Codger-generated arrays from ycode.c used in call to YorickInit */
-extern BuiltIn *yBuiltIns[];
-extern void *yGlobals[];
-extern char *yLinkNames[];
-extern char *initialIncludes[];
+char *yLaunchDir=0, *ySiteDir=0, *yHomeDir=0;
+static char *defaultPath = 0;
 
 extern int y_on_idle(void);
 extern void y_on_stdin(char *input_line);
 extern void y_on_exception(int signal, char *errmsg);
 extern int y_on_quit(void);
 extern void (*g_on_keyline)(char *);
+extern void (*g_stdout)(char *);
 
 extern int yBatchMode;  /* may be set with -batch, see std0.c */
 
 static void *y_mmfail(unsigned long n);
+static void y_on_keyline(char *msg);
 
 int
-on_launch(int argc, char *argv[])
+y_launch(int argc, char *argv[],
+         char *home, char *site, y_pkg_t **pkgs)
 {
   ym_argc= argc;
   ym_argv= argv;
   p_mmfail = y_mmfail;
-  YInit(yBuiltIns, yGlobals, yLinkNames, initialIncludes);
+  YInit(home, site, pkgs);
   if (!yBatchMode) {
     p_stdinit(y_on_stdin);
-    g_on_keyline = y_on_stdin;
+    g_on_keyline = y_on_keyline;
+    g_stdout = p_stdout;
   }
   p_idler(y_on_idle);
   p_handler(y_on_exception);
@@ -93,6 +91,14 @@ on_launch(int argc, char *argv[])
   return 0;
 }
 
+static void
+y_on_keyline(char *msg)
+{
+  p_stdout(msg);
+  p_stdout("\n");
+  y_on_stdin(msg);
+}
+
 static void *
 y_mmfail(unsigned long n)
 {
@@ -106,51 +112,42 @@ y_mmfail(unsigned long n)
 void Y_yorick_init(int nArgs)
 {
   Drop(nArgs);
-  YInit(yBuiltIns, yGlobals, yLinkNames, initialIncludes);
+  y_pkg_link(0);
 }
 
-static char *defaultPath;
-extern char *yUserPath;   /* defined in ycode.c */
-
-static void YInit(BuiltIn **builtin, void **global,
-		  char **name, char **include)
+static void
+YInit(char *home, char *site, y_pkg_t **pkgs)
 {
-  long index;
-  DataBlock *oldDB;
-
-  /* initialize Yorick timer */
+  /* initialize Yorick timer and user HOME directory */
   Y_timer(-1);
+  YGetHOME();
 
   /* initialize basic data types */
   InitStructDefs();                /* opsv.c */
 
-  /* initialize built-in functions */
-  while (*builtin) {
-    index= Globalize(*name++, 0L);
-    oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-    if (!oldDB || oldDB->ops!=&builtinOps ||
-	((BIFunction *)oldDB)->function!=*builtin) {
-      globTab[index].value.db= (DataBlock *)NewBIFunction(*builtin, index);
-      globTab[index].ops= &dataBlockSym;
-      Unref(oldDB);
-    }
-    builtin++;
-  }
+  /* add all statically linked packages, and link interpreter to them */
+  while (pkgs[0]) y_pkg_add(*pkgs++);
+  y_pkg_link(0);
 
-  /* initialize compiled variables */
-  while (*global) {
-    index= Globalize(*name++, 0L);
-    oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-    globTab[index].value.db=
-      (DataBlock *)NewLValueM((Array *)0, *global++,
-			      &charStruct, (Dimension *)0);
-    /* note: everything starts out as a scalar char at the correct address
-             -- the reshape operation when the associated package include
-	     file is read (below) will set the correct data type and
-	     dimension list */
-    globTab[index].ops= &dataBlockSym;
-    Unref(oldDB);
+  /* set startup include file search path:
+        yLaunchDir:ySiteDir/i0:ySiteDir/contrib
+
+     the switch to the normal include file search path is done in
+     stdx.i, which is included just before custom.i
+     -- create the default path here:
+        .:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
+  yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
+  if (yLaunchDir) {
+    int i;
+    for (i=0 ; yLaunchDir[i] ; i++);
+    while (i>0 && yLaunchDir[i]!='/') i--;
+    if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
   }
+  ySiteDir = site? p_strcpy(site) : p_strcpy("/");
+  YNameToHead(&ySiteDir);
+  yHomeDir = home? p_strcpy(home) : p_strcpy("/");
+  YNameToHead(&yHomeDir);
+  Y_set_site(0);
 
   /* set up to read user customizations
      If the first two arguments are -batch alternate.i, use alternate.i
@@ -169,31 +166,14 @@ static void YInit(BuiltIn **builtin, void **global,
     }
   }
 
-  /* set up to read std.i and package include files */
-  while (*include) YpPush(*include++);
+  /* final include file is always stdx.i, which returns to defaultPath */
+  YpPush("stdx.i");
 
-  /* first file read is always paths.i, which may override ySiteDir */
+  /* next include all the statically loaded packages */
+  y_pkg_include(0, 0);
+
+  /* first include file is always paths.i, which may override ySiteDir */
   YpPush("paths.i");
-
-  /* set startup include file search path:
-        yLaunchDir:ySiteDir/i0:ySiteDir/contrib
-
-     the switch to the normal include file search path is done in
-     stdx.i, which is included just before custom.i
-     -- create the default path here:
-        .:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
-  yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
-  if (yLaunchDir) {
-    int i;
-    for (i=0 ; yLaunchDir[i] ; i++);
-    while (i>0 && yLaunchDir[i]!='/') i--;
-    if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
-  }
-  ySiteDir = ySiteDir? p_strcpy(ySiteDir) : p_strcpy("/");
-  YNameToHead(&ySiteDir);
-  Y_set_site(0);
-
-  /* SetSignals(signalFlags); */
 }
 
 void Y_set_path(int nArgs)
@@ -205,11 +185,49 @@ void Y_set_path(int nArgs)
   YpSetPaths(path);
 }
 
-void Y_set_site(int nArgs)
+void Y_get_path(int nArgs)
+{
+  char *path, *p;
+  Array *rslt;
+  long i, len = 1;
+  if (nArgs!=1 || !yarg_nil(0)) YError("get_path takes one nil argument");
+  for (i=0 ; i<nYpPrefixes ; i++) len += strlen(ypPrefixes[i])+1;
+  rslt = (Array *)PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
+  rslt->value.q[0] = path = p_malloc(len);
+  for (i=0 ; i<nYpPrefixes ; i++) {
+    p = ypPrefixes[i];
+    while (p[0]) *path++ = *p++;
+    *path++ = ':';
+  }
+  if (nYpPrefixes) path--;
+  path[0] = '\0';
+}
+
+void
+Y_get_pkgnames(int nargs)
+{
+  int i, npkg;
+  Array *rslt;
+  Dimension *tmp = tmpDims;
+  if (nargs!=1) YError("get_pkgnames takes exactly one argument");
+  npkg = y_pkg_count(yarg_si(0));
+  if (npkg<=0) YError("get_pkgnames -- no packages loaded yet");
+  tmpDims = 0;
+  FreeDimension(tmp);
+  tmpDims = NewDimension((long)npkg, 1L, (Dimension *)0);
+  rslt = (Array *)PushDataBlock(NewArray(&stringStruct, tmpDims));
+  for (i=0 ; i<npkg ; i++)
+    rslt->value.q[i] = p_strcpy(y_pkg_name(i));
+}
+
+static char *y_make_ipath(char *ylaunch, char *ysite, char *yhome);
+
+void
+Y_set_site(int nArgs)
 {
   /* This function should ONLY be called from paths.i, or by YInit  */
 
-  char *path, *pathtmp, *site;
+  char *path;
   Array *dirName;
   DataBlock *oldDB;
   long index;
@@ -217,23 +235,27 @@ void Y_set_site(int nArgs)
   if (nArgs>2) YError("expecting zero, one, or two arguments");
 
   /* record Y_LAUNCH in global variable */
-  index= Globalize("Y_LAUNCH", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_LAUNCH", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yLaunchDir);
+  dirName->value.q[0] = p_strcpy(yLaunchDir);
 
   if (nArgs>=1) {
-    path= YGetString(sp-nArgs+1);
-    if (nArgs==2) yHomeDir= YGetString(sp);
+    ySiteDir = p_strcpy(YGetString(sp-nArgs+1));
+    YNameToHead(&ySiteDir);
+    if (nArgs==2) {
+      yHomeDir = p_strcpy(YGetString(sp));
+      YNameToHead(&yHomeDir);
+    }
   } else {
 #ifndef NO_FALLBACK_TEST
     p_file *test = 0;
     if (ySiteDir && ySiteDir[0]) {
-      path= p_strncat(ySiteDir, "i0/std.i", 0);
-      test= p_fopen(path, "r");
+      path = p_strncat(ySiteDir, "i0/std.i", 0);
+      test = p_fopen(path, "r");
       p_free(path);
     }
     if (test) {
@@ -245,14 +267,14 @@ void Y_set_site(int nArgs)
         for (i-=2 ; i>0 ; i-=2)
           if (yLaunchDir[i-1]!='/' || yLaunchDir[i]!='.') break;
         for (; i>0 ; i--) if (yLaunchDir[i]=='/') break;
-        path = p_malloc(i+9);
+        path = p_malloc(i+10);
         strncpy(path, yLaunchDir, i+1);
         strcpy(path+i+1, "i0/std.i");
         test= p_fopen(path, "r");
         if (test) {
           p_fclose(test);
           path[i+1] = '\0';
-          ySiteDir= path;
+          ySiteDir = path;
           /* if ySiteDir was bad, so is yHomeDir, guess the same */
           yHomeDir = p_strcpy(ySiteDir);  
         } else {
@@ -261,85 +283,76 @@ void Y_set_site(int nArgs)
       }
     }
 #endif
-    path= ySiteDir;
   }
 
   /* record Y_VERSION in global variable */
-  index= Globalize("Y_VERSION", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_VERSION", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yVersion);
+  dirName->value.q[0] = p_strcpy(yVersion);
 
   /* record Y_HOME in global variable */
-  index= Globalize("Y_HOME", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_HOME", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yHomeDir);
-  YNameToHead(dirName->value.q);
+  dirName->value.q[0] = p_strcpy(yHomeDir);
 
   /* record Y_SITE in global variable */
-  index= Globalize("Y_SITE", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
+  index = Globalize("Y_SITE", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
   globTab[index].ops= &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(path);
+  dirName->value.q[0] = p_strcpy(ySiteDir);
   YNameToHead(dirName->value.q);
-  site= dirName->value.q[0];
 
-  /* construct startup include file search path:
-        yLaunchDir:ySiteDir/i0:ySiteDir/contrib        */
-  if (yLaunchDir && strlen(yLaunchDir) &&
-      strcmp(yLaunchDir, site)) pathtmp= p_strncat(yLaunchDir, PATH_SEP, 0);
-  else pathtmp= 0;
-  path= p_strncat(pathtmp, site, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, "i0", 0);
-  p_free(path);
-  path= p_strncat(pathtmp, PATH_SEP, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "contrib", 0);
-  p_free(pathtmp);
-
-  /* set the initial include path */
+  /* set the initial include path for paths.i */
+  path = y_make_ipath(yLaunchDir, ySiteDir, yHomeDir);
   YpSetPaths(path);
   p_free(path);
 
-  /* the switch to the normal include file search path is done in
-     stdx.i, which is included just before custom.i
-     -- create the default path here:
-        .:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
-  if (yUserPath)
-    path= p_strncat(yUserPath, PATH_SEP, 0);
-  else
-    path= 0;
-  /* if (yLaunchDir && yLaunchDir[0] &&
-   *     strcmp(yLaunchDir, site)) {
-   *   pathtmp= p_strncat(path, yLaunchDir, 0);
-   *   p_free(path);
-   *   path= p_strncat(pathtmp, "i" PATH_SEP, 0);
-   *   p_free(pathtmp);
-   * }
-   */
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "i" PATH_SEP, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "contrib", 0);
-  p_free(pathtmp);
+  defaultPath = y_make_ipath(0, ySiteDir, yHomeDir);
+}
 
-  defaultPath= path;
+static char *
+y_make_ipath(char *ylaunch, char *ysite, char *yhome)
+{
+  /* ~/yorick:~/Yorick:Y_SITE/i:Y_SITE/contrib:Y_SITE/i0:Y_HOME/lib */
+  char *path1 = 0, *path2 = 0;
+  if (ylaunch && ylaunch[0]) {
+    /* prepend Y_LAUNCH for paths.i only */
+    long len = strlen(ylaunch);
+    path1 = (ylaunch[len-1]=='/')? p_strncat(0, ylaunch, len-1) :
+      p_strcpy(ylaunch);
+    path2 = p_strncat(path1, PATH_SEP, 0);
+    p_free(path1);
+  }
+  path1 = p_strncat(path2, "~/yorick" PATH_SEP "~/Yorick" PATH_SEP, 0);
+  if (path2) p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "i" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "contrib" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "i0" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, yhome, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "lib", 0);
+  p_free(path2);
+  return path1;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -360,7 +373,7 @@ void Y_reshape(int nArgs)
   if (!nArgs) {
     /* reshape, var
        same as var=[], but works for LValues as well */
-    globTab[index].value.db= Ref(&nilDB);
+    globTab[index].value.db= RefNC(&nilDB);
     if (globTab[index].ops==&dataBlockSym) { Unref(db); }
     else globTab[index].ops= &dataBlockSym;
     Drop(1);
@@ -376,43 +389,43 @@ void Y_reshape(int nArgs)
     if (arg->ops==&dataBlockSym) {
       adb= arg->value.db;
       if (adb->ops==&structDefOps) {
-	/* reshape, var, ->type, dimlist
-	   uses address of the current value of var */
-	base= (StructDef *)adb;
-	db= ForceToDB(&globTab[index]);
-	if (db->ops==&lvalueOps) {
-	  LValue *lvalue= (LValue *)db;
-	  if (lvalue->type.base->file)
-	    YError("cannot reshape a disk-resident variable");
-	  address= lvalue->address.m;
-	  owner= lvalue->owner;
-	} else if (db->ops->isArray) {
-	  Array *array= (Array *)db;
-	  address= array->value.c;
-	  owner= array;
-	} else {
-	  YError("cannot reshape non-array object");
-	}
+        /* reshape, var, ->type, dimlist
+           uses address of the current value of var */
+        base= (StructDef *)adb;
+        db= ForceToDB(&globTab[index]);
+        if (db->ops==&lvalueOps) {
+          LValue *lvalue= (LValue *)db;
+          if (lvalue->type.base->file)
+            YError("cannot reshape a disk-resident variable");
+          address= lvalue->address.m;
+          owner= lvalue->owner;
+        } else if (db->ops->isArray) {
+          Array *array= (Array *)db;
+          address= array->value.c;
+          owner= array;
+        } else {
+          YError("cannot reshape non-array object");
+        }
 
       } else if ((adb->ops==&pointerOps ||
-		  adb->ops==&intOps || adb->ops==&longOps) &&
-		 !((Array *)adb)->type.dims) {
-	/* reshape, var, ->address, type, dimlist */
-	Array *array= (Array *)adb;
-	if (adb->ops==&pointerOps) {
-	  address= array->value.p[0];
-	  owner= Pointee(address);
-	} else if (adb->ops==&intOps) {
-	  address= (char *)0 + array->value.i[0];
-	  owner= 0;
-	} else {
-	  address= (char *)0 + array->value.l[0];
-	  owner= 0;
-	}
+                  adb->ops==&intOps || adb->ops==&longOps) &&
+                 !((Array *)adb)->type.dims) {
+        /* reshape, var, ->address, type, dimlist */
+        Array *array= (Array *)adb;
+        if (adb->ops==&pointerOps) {
+          address= array->value.p[0];
+          owner= Pointee(address);
+        } else if (adb->ops==&intOps) {
+          address= (char *)0 + array->value.i[0];
+          owner= 0;
+        } else {
+          address= (char *)0 + array->value.l[0];
+          owner= 0;
+        }
 
       } else {
       badd:
-	YError("reshape address must be scalar pointer, int, or long");
+        YError("reshape address must be scalar pointer, int, or long");
       }
     } else if (arg->ops==&intScalar) {
       /* reshape, var, ->address, type, dimlist */
@@ -433,9 +446,9 @@ void Y_reshape(int nArgs)
       /* reshape, var, address, ->type, dimlist */
       if (arg->ops==&referenceSym) ReplaceRef(arg);
       if (arg->ops==&dataBlockSym && arg->value.db->ops==&structDefOps)
-	base= (StructDef *)arg->value.db;
+        base= (StructDef *)arg->value.db;
       else
-	YError("bad data type specified for reshape");
+        YError("bad data type specified for reshape");
       arg++;
       nArgs--;
     }
@@ -445,12 +458,12 @@ void Y_reshape(int nArgs)
 
     if (owner) {
       char *end= owner->value.c +
-	owner->type.number*owner->type.base->size;
+        owner->type.number*owner->type.base->size;
       char *last= result->address.m +
-	result->type.number*result->type.base->size;
+        result->type.number*result->type.base->size;
       if (last>end) {
-	Drop(1);
-	YError("reshape beyond bounds of array owner");
+        Drop(1);
+        YError("reshape beyond bounds of array owner");
       }
     }
 
@@ -539,19 +552,19 @@ static DataBlock *GetInfo(Symbol *s)
     } else if (s->ops==&dataBlockSym) {
       db= s->value.db;
       if (db->ops==&lvalueOps) {
-	LValue *lvalue= (LValue *)db;
-	type.base= lvalue->type.base;
-	type.dims= lvalue->type.dims;
-	type.number= lvalue->type.number;
+        LValue *lvalue= (LValue *)db;
+        type.base= lvalue->type.base;
+        type.dims= lvalue->type.dims;
+        type.number= lvalue->type.number;
       } else if (db->ops->isArray) {
-	Array *array= (Array *)db;
-	type.base= array->type.base;
-	type.dims= array->type.dims;
-	type.number= array->type.number;
+        Array *array= (Array *)db;
+        type.base= array->type.base;
+        type.dims= array->type.dims;
+        type.number= array->type.number;
       } else {
-	type.base= 0;
-	type.dims= 0;
-	type.number= 0;
+        type.base= 0;
+        type.dims= 0;
+        type.number= 0;
       }
       break;
     } else if (s->ops==&referenceSym) {
@@ -569,7 +582,7 @@ void Y_structof(int nArgs)
   if (nArgs != 1) YError("structof takes exactly one argument");
   db= GetInfo(sp);
   if (!db) PushDataBlock(Ref(type.base));
-  else PushDataBlock(Ref(&nilDB));
+  else PushDataBlock(RefNC(&nilDB));
 }
 
 Dimension *dimsofDims= 0;
@@ -618,7 +631,7 @@ void Y_dimsof(int nArgs)
       tmp= tmp->next;
     }
   } else {
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   }
 }
 
@@ -644,7 +657,7 @@ void Y_orgsof(int nArgs)
       type.dims= type.dims->next;
     }
   } else {
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   }
 }
 
@@ -723,7 +736,7 @@ void Y_nameof(int nArgs)
 
   Drop(2);
   if (!name) {
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   } else {
     Array *result= PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
     result->value.q[0]= p_strcpy(name);
@@ -764,6 +777,7 @@ void Y_is_func(int nArgs)
       Operations *ops= s->value.db->ops;
       if (ops==&functionOps) isFunc= 1;
       else if (ops==&builtinOps) isFunc= 2;
+      else if (ops==&auto_ops) isFunc= 3;
       else isFunc= 0;
       break;
     } else if (s->ops!=&referenceSym) {
@@ -785,7 +799,7 @@ void Y_is_void(int nArgs)
   for (;;) {
     if (s->ops==&dataBlockSym) {
       Operations *ops= s->value.db->ops;
-      isVoid= (ops==&voidOps);
+      isVoid= (ops==&voidOps || ops==&auto_ops);
       break;
     } else if (s->ops!=&referenceSym) {
       isVoid= 0;
@@ -1042,7 +1056,7 @@ void Y_atan(int nArgs)
     (sp-1)->ops->FormOperand(sp-1, &opy);
     ops= opy.ops->Promote[opx.ops->promoteID](&opy, &opx);
     if (!ops ||
-	ops->promoteID>T_DOUBLE) YError("illegal data type to atan(y, x)");
+        ops->promoteID>T_DOUBLE) YError("illegal data type to atan(y, x)");
     dst= BuildResult2(&opy, &opx);
     if (!dst) YError("operands not conformable in binary atan");
     y= opy.value;
@@ -1167,8 +1181,8 @@ static AbsLooper absCLoop, absSLoop, absILoop, absLLoop,
   absFLoop, absDLoop, absZLoop;
 
 static void absCLoop(void *d, void *s, long n)
-{ long i; char *dst= d; char *src= s;
-  for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -src[i] : src[i]; }
+{ long i; unsigned char *dst= d; unsigned char *src= s;
+  for (i=0 ; i<n ; i++) dst[i]= src[i]; }
 static void absSLoop(void *d, void *s, long n)
 { long i; short *dst= d; short *src= s;
   for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -src[i] : src[i]; }
@@ -1189,7 +1203,7 @@ static void absZLoop(void *d, void *s, long n)
   for (i=0 ; i<n ; i++) dst[i]= hypot(src[2*i], src[2*i+1]); }
 
 static AbsLooper *absLoop[]= { &absCLoop, &absSLoop, &absILoop, &absLLoop,
-		               &absFLoop, &absDLoop };
+                               &absFLoop, &absDLoop };
 
 void Y_abs(int nArgs)
 {
@@ -1222,9 +1236,9 @@ void Y_abs(int nArgs)
       (sp-1)->ops->FormOperand(sp-1, &opy);
       ops= opy.ops->Promote[opx.ops->promoteID](&opy, &opx);
       if (!ops || ops->promoteID>T_DOUBLE) {
-	if (!ops ||
-	    ops->promoteID>T_COMPLEX) YError("illegal data type to abs(x,y)");
-	else YError("no abs(complex,complex), take abs of argument(s)");
+        if (!ops ||
+            ops->promoteID>T_COMPLEX) YError("illegal data type to abs(x,y)");
+        else YError("no abs(complex,complex), take abs of argument(s)");
       }
       dst= BuildResult2(&opy, &opx);
       if (!dst) YError("operands not conformable in binary abs");
@@ -1244,8 +1258,8 @@ static AbsLooper signCLoop, signSLoop, signILoop, signLLoop,
   signFLoop, signDLoop, signZLoop;
 
 static void signCLoop(void *d, void *s, long n)
-{ long i; char *dst= d; char *src= s;
-  for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1 : 1; }
+{ long i; char *dst= d;
+  for (i=0 ; i<n ; i++) dst[i]= 1; }
 static void signSLoop(void *d, void *s, long n)
 { long i; short *dst= d; short *src= s;
   for (i=0 ; i<n ; i++) dst[i]= src[i]<0? -1 : 1; }
@@ -1267,8 +1281,8 @@ static void signZLoop(void *d, void *s, long n)
   for (i=0 ; i<n ; i+=2) signZ(&dst[i], &src[i]); }
 
 static AbsLooper *signLoop[]= { &signCLoop, &signSLoop, &signILoop,
-		                &signLLoop, &signFLoop, &signDLoop,
-				&signZLoop };
+                                &signLLoop, &signFLoop, &signDLoop,
+                                &signZLoop };
 
 void Y_sign(int nArgs)
 {
@@ -1334,7 +1348,7 @@ static Array *Force1D(Operand *op)
     if (op->references) {
       array= PushDataBlock(NewArray(op->type.base, tmpDims));
       op->type.base->Copy(op->type.base, array->value.c, op->value,
-			  op->type.number);
+                          op->type.number);
       PopTo(op->owner);
       op->value= array->value.c;
     } else {
@@ -1483,16 +1497,17 @@ void Y_sum(int nArgs)
   int promoteID;
   if (nArgs != 1) YError("sum requires exactly one argument");
   sp->ops->FormOperand(sp, &op);
-  promoteID= op.ops->promoteID;
-  if (promoteID>T_COMPLEX) YError("sum requires numeric argument");
+  promoteID= (op.ops->typeID!=T_STRING)? op.ops->promoteID : T_STRING;
+  if (promoteID>T_COMPLEX && promoteID!=T_STRING)
+    YError("sum requires numeric argument");
   array= Force1D(&op);
-  if (!array) {			/* scalars are trivial */
+  if (!array) {                 /* scalars are trivial */
     PopTo(sp-1);
     return;
   }
   RFsum(array, 0);
   /* sum range function may change data type */
-  PopToX(sp-2, sp->value.db->ops->promoteID);
+  PopToX(sp-2, (promoteID!=T_STRING)?sp->value.db->ops->promoteID:T_STRING);
   Drop(1);
 }
 
@@ -1508,7 +1523,7 @@ void Y_avg(int nArgs)
   promoteID= op.ops->promoteID;
   if (promoteID>T_COMPLEX) YError("avg requires numeric argument");
   array= Force1D(&op);
-  if (!array) {			/* scalars are trivial */
+  if (!array) {                 /* scalars are trivial */
     PopTo(sp-1);
     return;
   }
@@ -1651,7 +1666,7 @@ void Y_cd(int nArgs)
   p_free(name);
   if (notOK) {
     if (amSubroutine) YError("cd failed -- no such directory");
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   } else if (!amSubroutine) {
     Y_get_cwd(nArgs);
   }
@@ -1705,15 +1720,6 @@ struct OriginStatus {
 extern OriginStatus *NewOrg(int force);
 extern void FreeOrg(void *ob);  /* ******* Use Unref(bm) ******* */
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static UnaryOp PrintOB;
 
 Operations orgsOps = {
@@ -1728,7 +1734,7 @@ Operations orgsOps = {
 };
 
 static MemryBlock orgBlock= {0, 0, sizeof(OriginStatus),
-				16*sizeof(OriginStatus)};
+                                16*sizeof(OriginStatus)};
 
 void Y_use_origins(int nArgs)
 {
@@ -1760,7 +1766,7 @@ static void PrintOB(Operand *op)
   OriginStatus *ob= op->value;
   char line[80];
   sprintf(line, "origin_status: index origins %s",
-	  ob->force? "force" : "default");
+          ob->force? "force" : "default");
   ForceNewline();
   PrintFunc(line);
   ForceNewline();
