@@ -33,7 +33,7 @@ extern void SetSequentialWrite(IOStream *file, long last);  /* yrdwr.c */
 
 static void GrabVariable(IOStream *file, long iFile, long iGlob);
 static void StuffVariable(IOStream *file, long iFile,
-			  Operand *op, int notNew);
+                          Operand *op, int notNew);
 
 extern int YtestPDB(IOStream *file, int familyOK);
 extern int YinitPDB(IOStream *file, int close102);
@@ -74,11 +74,11 @@ void Y_save(int nArgs)
     if (globIndex<0) {
       /* get name and value to be saved */
       if (stack->ops!=&referenceSym)
-	YError("save needs simple variable references as arguments");
+        YError("save needs simple variable references as arguments");
       name= globalTable.names[stack->index];
       stack->ops->FormOperand(stack, &op);
       if (!op.ops || (!op.ops->isArray && op.ops!=&structDefOps))
-	YError("save can output array data only");
+        YError("save can output array data only");
       stack++;
       nArgs--;
 
@@ -91,11 +91,11 @@ void Y_save(int nArgs)
       name= globalTable.names[globIndex++];
       gops= global->ops;
       if (gops==&dataBlockSym) {
-	Operations *ops= global->value.db->ops;
-	if (!ops->isArray && ops!=&lvalueOps && ops!=&structDefOps) continue;
+        Operations *ops= global->value.db->ops;
+        if (!ops->isArray && ops!=&lvalueOps && ops!=&structDefOps) continue;
       } else if (gops!=&doubleScalar &&
-		 gops!=&longScalar && gops!=&intScalar) {
-	continue;
+                 gops!=&longScalar && gops!=&intScalar) {
+        continue;
       }
       gops->FormOperand(global, &op);
     }
@@ -105,13 +105,13 @@ void Y_save(int nArgs)
       if (structsOnly) YError("file has no current record for save");
       notNew= AddVariable(file, -1L, name, op.type.base, op.type.dims);
       if (notNew>1)
-	YError("data type (struct) name conflict in save to binary file");
+        YError("data type (struct) name conflict in save to binary file");
 
       StuffVariable(file, hashIndex, &op, notNew);
 
     } else {
       if (!CopyStruct(file, (StructDef *)op.value))
-	YError("problem saving struct to binary file (name conflict?)");
+        YError("problem saving struct to binary file (name conflict?)");
     }
   }
 
@@ -145,7 +145,7 @@ void Y_restore(int nArgs)
     do {
       /* get name and value to be restore */
       if (stack->ops!=&referenceSym)
-	YError("restore needs simple variable references as arguments");
+        YError("restore needs simple variable references as arguments");
       index= stack->index;
       name= globalTable.names[index];
       stack++;
@@ -153,11 +153,11 @@ void Y_restore(int nArgs)
 
       /* look up name in dataTable (in child if a current record) */
       if (HashFind(&file->dataTable, name, 0L))
-	GrabVariable(file, hashIndex, index);
+        GrabVariable(file, hashIndex, index);
       else if (parent && HashFind(&parent->dataTable, name, 0L))
-	GrabVariable(parent, hashIndex, index);
+        GrabVariable(parent, hashIndex, index);
       else
-	YError("restore variable does not exist in binary file");
+        YError("restore variable does not exist in binary file");
 
     } while (nArgs);
 
@@ -229,23 +229,23 @@ void Y_add_record(int nArgs)
     if (stack<=sp) {
       ncyc= YGet_L(stack++, 1, &dims);
       if (ncyc) {
-	if (nRecs) {
-	  if (nRecs!=TotalNumber(dims))
-	    YError("inconsistent number of ncycs in add_record");
-	} else {
-	  nRecs= TotalNumber(dims);
-	}
+        if (nRecs) {
+          if (nRecs!=TotalNumber(dims))
+            YError("inconsistent number of ncycs in add_record");
+        } else {
+          nRecs= TotalNumber(dims);
+        }
       }
       if (stack<=sp) {
-	address= YGet_L(stack++, 1, &dims);
-	if (address) {
-	  if (nRecs) {
-	    if (nRecs!=TotalNumber(dims))
-	      YError("inconsistent number of addresses in add_record");
-	  } else {
-	    nRecs= TotalNumber(dims);
-	  }
-	}
+        address= YGet_L(stack++, 1, &dims);
+        if (address) {
+          if (nRecs) {
+            if (nRecs!=TotalNumber(dims))
+              YError("inconsistent number of addresses in add_record");
+          } else {
+            nRecs= TotalNumber(dims);
+          }
+        }
       }
     }
   }
@@ -261,7 +261,7 @@ void Y_add_record(int nArgs)
   flags= (time? 1 : 0)|(ncyc? 2 : 0);
   while (nRecs--) {
     AddRecord(history, flags, time? time[0]:0.0, ncyc? ncyc[0]:0L,
-	      (address && address[0]>=0)? address[0]:-1L);
+              (address && address[0]>=0)? address[0]:-1L);
     if (time) time++;
     if (ncyc) ncyc++;
     if (address) address++;
@@ -294,7 +294,7 @@ static void GetTNworker(int nArgs, int cycs)
   Dimension *dims;
   if (nArgs!=1) YError("get_times/get_ncycs takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
   history= file->history;
 
   if (history) {
@@ -303,7 +303,7 @@ static void GetTNworker(int nArgs, int cycs)
     else times= history->time;
   }
   if (n<=0 || (cycs? (!ncycs) : (!times))) {
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
     return;
   }
 
@@ -344,30 +344,34 @@ void Y__jt(int nArgs)
     if (nArgs>1) {
       sp->ops->FormOperand(sp, &op);
       if (op.ops==&rangeOps) {        /* jt, file, - */
-	/* look for   jt, file, -    (backup 1 record) */
-	Range *range= op.value;
-	if ((range->nilFlags&(~(R_MINNIL|R_MAXNIL|R_PSEUDO))) ||
-	    range->inc!=1)
-	  YError("did you want   jt, file, -   (jump to previous record)?");
-	noRecord= recNumber>=0? JumpRecord(history, recNumber-1) :
-	                        JumpRecord(history, history->nRecords-1);
+        /* look for   jt, file, -    (backup 1 record) */
+        Range *range= op.value;
+        if ((range->nilFlags&(~(R_MINNIL|R_MAXNIL|R_PSEUDO))) ||
+            range->inc!=1)
+          YError("did you want   jt, file, -   (jump to previous record)?");
+        noRecord= recNumber>=0? JumpRecord(history, recNumber-1) :
+                                JumpRecord(history, history->nRecords-1);
 
       } else if (op.ops==&voidOps) {  /* jt, file, [] */
-	noRecord= recNumber>=0? JumpRecord(history, recNumber+1) :
-	                        JumpRecord(history, 0);
+        noRecord= recNumber>=0? JumpRecord(history, recNumber+1) :
+                                JumpRecord(history, 0);
 
       } else {                        /* jt, file, time */
-	targetTime= YGetReal(sp);
-	noRecord= JumpToTime(history, targetTime);
-	if (noRecord) {
-	  noRecord= 2;
-	  targetTime= history->time[history->recNumber];
-	}
+        targetTime= YGetReal(sp);
+        noRecord= JumpToTime(history, targetTime);
+        if (noRecord) {
+          if (history->time) {
+            noRecord= 2;
+            targetTime= history->time[history->recNumber];
+          } else {
+            noRecord= 3;
+          }
+        }
       }
 
     } else {                          /* jt, file */
       noRecord= recNumber>=0? JumpRecord(history, recNumber+1) :
-	                      JumpRecord(history, 0);
+                              JumpRecord(history, 0);
     }
 
   } else {                            /* jt, time */
@@ -387,7 +391,7 @@ void Y__jt(int nArgs)
   }
 
   if (!CalledAsSubroutine()) {
-    if (noRecord>2) PushDataBlock(Ref(&nilDB));
+    if (noRecord>2) PushDataBlock(RefNC(&nilDB));
     if (noRecord>1) PushDoubleValue(targetTime);
     else if (noRecord) PushIntValue(0);
     else PushIntValue(1);
@@ -401,7 +405,7 @@ void Y__jc(int nArgs)
   int noRecord;
   if (nArgs!=2) YError("jc takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history)
     YError("binary file passed to jc has no history records");
@@ -409,7 +413,7 @@ void Y__jc(int nArgs)
   noRecord= JumpToCycle(history, YGetInteger(sp));
 
   if (!CalledAsSubroutine()) {
-    if (noRecord) PushDataBlock(Ref(&nilDB));
+    if (noRecord) PushDataBlock(RefNC(&nilDB));
     else PushLongValue(history->ncyc[history->recNumber]);
   }
 }
@@ -422,7 +426,7 @@ void Y__jr(int nArgs)
   int amSubroutine= CalledAsSubroutine();
   if (nArgs!=2) YError("jr takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history) {
     if (amSubroutine)
@@ -451,7 +455,7 @@ void Y_get_vars(int nArgs)
   Dimension *dims;
   if (nArgs!=1) YError("get_vars takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   child= file->history? file->history->child : 0;
 
@@ -571,7 +575,7 @@ void Y_get_addrs(int nArgs)
   Dimension *dims;
   if (nArgs!=1) YError("get_addrs takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   history= file->history;
   child= history? history->child : 0;
@@ -631,7 +635,7 @@ void Y_set_filesize(int nArgs)
   long size;
   if (nArgs!=2) YError("set_filesize takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   history= file->history;
   if (!history)
     YError("binary file passed to set_filesize has no history records");
@@ -647,7 +651,7 @@ void Y_set_blocksize(int nArgs)
   long size, nbytes;
   if (nArgs!=2) YError("set_block takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
 
   nbytes= YGetInteger(sp);
   size= 4096;
@@ -695,7 +699,7 @@ void Y_edit_times(int nArgs)
     else nKeep= 0;
     for (i=0 ; i<nKeep ; i++) {
       if (keepList[i]<0 || keepList[i]>=nRecs || keepList[i]<=lastKept)
-	YError("keep_list out of range or not increasing in edit_times");
+        YError("keep_list out of range or not increasing in edit_times");
       lastKept= keepList[i];
     }
     if (nArgs>2) {
@@ -703,13 +707,13 @@ void Y_edit_times(int nArgs)
       newTimes= YGet_D(stack, 1, &dims);
       i= TotalNumber(dims);
       if (newTimes && (nKeep? (i!=nKeep) : (i!=nRecs)))
-	YError("new_times has wrong length in edit_times");
+        YError("new_times has wrong length in edit_times");
       if (nArgs>3) {
-	stack++;
-	newNcycs= YGet_L(stack, 1, &dims);
-	i= TotalNumber(dims);
-	if (newNcycs && (nKeep? (i!=nKeep) : (i!=nRecs)))
-	  YError("new_ncycs has wrong length in edit_times");
+        stack++;
+        newNcycs= YGet_L(stack, 1, &dims);
+        i= TotalNumber(dims);
+        if (newNcycs && (nKeep? (i!=nKeep) : (i!=nRecs)))
+          YError("new_ncycs has wrong length in edit_times");
       }
     }
   }
@@ -735,15 +739,15 @@ void Y_edit_times(int nArgs)
       double downTo;
       downTo= times[nRecs-1];
       for (i=nRecs-2 ; i>=0 ; i--) {
-	if (times[i]<downTo) downTo= times[i];
-	else { times[i]= bad; nKeep--; }  /* mark records to be deleted */
+        if (times[i]<downTo) downTo= times[i];
+        else { times[i]= bad; nKeep--; }  /* mark records to be deleted */
       }
     } else if (ncycs) {
       long downTo;
       downTo= ncycs[nRecs-1];
       for (i=nRecs-2 ; i>=0 ; i--) {
-	if (ncycs[i]<downTo) downTo= ncycs[i];
-	else { ncycs[i]= nbad; nKeep--; }  /* mark records to be deleted */
+        if (ncycs[i]<downTo) downTo= ncycs[i];
+        else { ncycs[i]= nbad; nKeep--; }  /* mark records to be deleted */
       }
     }
   }
@@ -752,44 +756,48 @@ void Y_edit_times(int nArgs)
   if (nKeep && nKeep<nRecs) {
     if (keepList) {
       for (i=0 ; i<nKeep ; i++) {
-	j= keepList[i];
-	ifiles[j]= ifiles[i];
-	offsets[j]= offsets[i];
-	if (times) times[j]= times[i];
-	if (ncycs) ncycs[j]= ncycs[i];
+        j= keepList[i];
+        ifiles[i]= ifiles[j];
+        offsets[i]= offsets[j];
+        if (times) times[i]= times[j];
+        if (ncycs) ncycs[i]= ncycs[j];
       }
       history->nRecords= nRecs= nKeep;
 
     } else if (times) {
       for (i=j=0 ; i<nRecs ; i++) {
-	if (times[i]!=bad) {
-	  ifiles[j]= ifiles[i];
-	  offsets[j]= offsets[i];
-	  if (ncycs) ncycs[j]= ncycs[i];
-	  times[j++]= times[i];
-	}
+        if (times[i]!=bad) {
+          ifiles[j]= ifiles[i];
+          offsets[j]= offsets[i];
+          if (ncycs) ncycs[j]= ncycs[i];
+          times[j++]= times[i];
+        }
       }
       history->nRecords= nRecs= j;
 
     } else if (ncycs) {
       for (i=j=0 ; i<nRecs ; i++) {
-	if (ncycs[i]!=nbad) {
-	  ifiles[j]= ifiles[i];
-	  offsets[j]= offsets[i];
-	  ncycs[j++]= ncycs[i];
-	}
+        if (ncycs[i]!=nbad) {
+          ifiles[j]= ifiles[i];
+          offsets[j]= offsets[i];
+          ncycs[j++]= ncycs[i];
+        }
       }
       history->nRecords= nRecs= j;
     }
   }
 
   /* install new times */
-  if (newTimes && times) {
+  if (newTimes && nRecs>0) {
+    if (!times) times = history->time =
+                  p_malloc(sizeof(double)*(16+((nRecs-1)&~0xf)));
     for (i=0 ; i<nRecs ; i++) times[i]= newTimes[i];
   }
 
   /* install new ncycs */
-  if (newNcycs && ncycs) {
+  if (newNcycs && nRecs>0) {
+    if (!ncycs) ncycs = history->ncyc =
+                  p_malloc(sizeof(long)*(16+((nRecs-1)&~0xf)));
     for (i=0 ; i<nRecs ; i++) ncycs[i]= newNcycs[i];
   }
 }
@@ -804,7 +812,7 @@ void Y__read(int nArgs)
   StructDef *base;
   if (nArgs!=3) YError("_read takes exactly three arguments");
 
-  file= YGetFile(sp-2);
+  file= yarg_file(2);
   if (file->history) file= file->history->child;
 
   address= YGetInteger(sp-1);
@@ -888,7 +896,7 @@ void Y__write(int nArgs)
     YWrite(op.value, address, base, op.type.number, (Strider *)0);
   }
 
-  PushDataBlock(Ref(&nilDB));
+  PushDataBlock(RefNC(&nilDB));
 }
 
 /*--------------------------------------------------------------------------*/
@@ -959,7 +967,7 @@ void Y_install_struct(int nArgs)
     if (nArgs==6) {
       layout= YGet_L(stack, 1, &dims);
       if (!layout || TotalNumber(dims)!=7)
-	YError("layout argument must be array of 7 longs in install_struct");
+        YError("layout argument must be array of 7 longs in install_struct");
       fpLayout.sgnAddr= (int)layout[0];
       fpLayout.expAddr= (int)layout[1];
       fpLayout.expSize= (int)layout[2];
@@ -993,7 +1001,7 @@ void Y_install_struct(int nArgs)
       addressType= 2;
     }
     if (DefinePrimitive(base, size, (int)align, addressType, (int)order,
-			layout? &fpLayout : 0, model, (Converter *)0))
+                        layout? &fpLayout : 0, model, (Converter *)0))
       YError("failed to define primitive data type in add_member");
   }
 
@@ -1012,7 +1020,7 @@ void Y_get_primitives(int nArgs)
   int i, j;
   if (nArgs!=1) YError("get_primitives requires exactly one argument");
 
-  file = YGetFile(sp);
+  file = yarg_file(0);
   dims = tmpDims;
   tmpDims = 0;
   FreeDimension(dims);
@@ -1081,7 +1089,7 @@ void Y_data_align(int nArgs)
   int alignment;
   if (nArgs!=2) YError("data_align takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   alignment= (int)YGetInteger(sp);
 
   file->dataAlign= alignment>0? alignment : 0;
@@ -1094,10 +1102,10 @@ void Y_struct_align(int nArgs)
   int alignment;
   if (nArgs!=2) YError("struct_align takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   alignment= (int)YGetInteger(sp);
 
-  file->structAlign= alignment>0? alignment : yStructAlign;
+  file->structAlign= (alignment!=0)? alignment : yStructAlign;
   Drop(2);
 }
 
@@ -1106,9 +1114,9 @@ void Y_struct_align(int nArgs)
 static int doNotRecurse= 1;  /* dummy for ReadGather/WriteScatter */
 
 extern void ReadGather(void *dst, void *srcM, long srcD, StructDef *base,
-		       long number, const Strider *strider);
+                       long number, const Strider *strider);
 extern void WriteScatter(void *src, void *dstM, long dstD, StructDef *base,
-			 long number, const Strider *strider);
+                         long number, const Strider *strider);
 
 /* Grab the variable file->dataTable.names[iFile] and stuff its value
    into globTab[iGlob].  */
@@ -1160,7 +1168,7 @@ static void GrabVariable(IOStream *file, long iFile, long iGlob)
 }
 
 static void StuffVariable(IOStream *file, long iFile,
-			  Operand *op, int notNew)
+                          Operand *op, int notNew)
 {
   StructDef *base= file->types[iFile].base;
   long address= file->addresses[iFile]+file->offset;
@@ -1176,7 +1184,7 @@ static void StuffVariable(IOStream *file, long iFile,
   if (base->addressType==2)
     SetSequentialWrite(file, address+base->size*op->type.number);
   WriteScatter(op->value, &doNotRecurse, address, base, op->type.number,
-	       (Strider *)0);
+               (Strider *)0);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1190,7 +1198,7 @@ void Y__not_pdb(int nArgs)
   int familyOK, notOK;
   if (nArgs!=2) YError("_not_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   familyOK= (int)YGetInteger(sp);
 
   if (!pdb_open) pdb_open= Globalize("yPDBopen", 0L);
@@ -1221,7 +1229,7 @@ void Y__init_pdb(int nArgs)
   int close102;
   if (nArgs!=2) YError("_init_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   close102= ((int)YGetInteger(sp)) & 017;
 
   if (YinitPDB(file, close102))
@@ -1235,7 +1243,7 @@ void Y__set_pdb(int nArgs)
   int close102;
   if (nArgs!=2) YError("_set_pdb takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   close102= ((int)YGetInteger(sp)) & 017;
 
   if (file->history) {
@@ -1252,7 +1260,7 @@ void Y__init_clog(int nArgs)
   IOStream *file;
   if (nArgs!=1) YError("_init_clog takes exactly one argument");
 
-  file= YGetFile(sp);
+  file= yarg_file(0);
 
   if (file->history) {
     file->history->parent->CloseHook=
@@ -1269,7 +1277,7 @@ void Y_dump_clog(int nArgs)
   char *name;
   if (nArgs!=2) YError("dump_clog takes exactly two arguments");
 
-  file= YGetFile(sp-1);
+  file= yarg_file(1);
   name= YGetString(sp);
 
   if (DumpClogFile(file, name))
@@ -1283,8 +1291,8 @@ void Y_read_clog(int nArgs)
   char *name;
   if (nArgs!=2) YError("read_clog takes exactly two arguments");
 
-  file= YGetFile(sp-1);
-  name= YGetString(sp);
+  file= yarg_file(1);
+  name= yarg_sq(0);
 
   if (ReadClogFile(file, name))
     YError("read_clog failed -- unable to read input file");
@@ -1335,11 +1343,11 @@ void Y_eq_nocopy(int nArgs)
       Array *owner= lvalue->owner;
       /* actually only want to fetch a non-trivial LValue */
       if (!owner || lvalue->strider ||
-	  lvalue->type.dims!=owner->type.dims ||
-	  lvalue->type.base!=owner->type.base) {
-	array= FetchLValue(array, sp);
+          lvalue->type.dims!=owner->type.dims ||
+          lvalue->type.base!=owner->type.base) {
+        array= FetchLValue(array, sp);
       } else {
-	array= owner;
+        array= owner;
       }
     }
     glob->value.db= (DataBlock *)Ref(array);
@@ -1362,15 +1370,6 @@ struct y_tmpobj {
 
 static void y_zap_tmpobj(void *to);
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static Operations yo_tmpobj = {
   &y_zap_tmpobj, T_OPAQUE, 0, T_STRING, "tmpobj",
   {&PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX, &PromXX},
@@ -1515,7 +1514,7 @@ Y_lsdir(int nArgs)
     dlist->fils = 0;
     p_free(list);
   } else {
-    stack->value.db = (DataBlock *)(Ref(&nilDB));
+    stack->value.db = (DataBlock *)(RefNC(&nilDB));
     stack->ops = &dataBlockSym;
   }
 
@@ -1542,7 +1541,7 @@ Y_lsdir(int nArgs)
       dlist->subs = 0;
       p_free(list);
     } else {
-      glob->value.db = (DataBlock *)(Ref(&nilDB));
+      glob->value.db = (DataBlock *)(RefNC(&nilDB));
       glob->ops = &dataBlockSym;
     }
   }
