@@ -32,13 +32,12 @@ extern int YpParse(void *func);  /* argument non-zero for YpReparse */
 /*--------------------------------------------------------------------------*/
 
 extern BuiltIn Y_quit, Y_include, Y_require, Y_help, Y_exit, Y_error, Y_batch;
+extern BuiltIn Y_plug_in, Y_plug_dir;
 
 extern void YRun(void);
 extern void YHalt(void);
 
 extern int CheckForTasks(int wait);
-extern void IncludeNow(void);
-extern void ResetStack(int hard);
 
 extern VMaction Eval, Return, PushVariable;
 
@@ -67,12 +66,12 @@ extern Instruction *AbortReturn(void);
 extern long ypBeginLine;
 
 extern TextStream *NewTextStream(char *fullname,
-				 void *stream, int permissions,
-				 long line, long pos);
+                                 void *stream, int permissions,
+                                 long line, long pos);
 
 typedef struct DebugBlk DebugBlk;
 extern DebugBlk *NewDebugBlk(Function *f, long pcerr, char *errFile,
-			     long lnum);
+                             long lnum);
 
 /* stuff to implement catch */
 extern BuiltIn Y_catch;
@@ -97,7 +96,6 @@ static int caughtTask= 0;
 
 /* stuff to implement set_idler */
 extern BuiltIn Y_set_idler;
-extern Function *y_idler_function;
 Function *y_idler_function= 0;
 
 /*--------------------------------------------------------------------------*/
@@ -105,6 +103,7 @@ Function *y_idler_function= 0;
 Instruction *pc= 0;
 
 static int ym_state= 0;
+static int ym_fatal = 0;
 extern int ym_dbenter;
 int ym_dbenter = 0;
 #define Y_QUITTING 1
@@ -116,6 +115,7 @@ void
 YRun(void)
 {
   register VMaction *Action;
+  int run_state = ym_state & Y_RUNNING;
   ym_state |= Y_RUNNING;
   ym_dbenter = 0;
 
@@ -123,13 +123,23 @@ YRun(void)
     Action = (pc++)->Action;
     Action();
   }
+  if (p_signalling==-1 && !(ym_state&Y_RUNNING))
+    p_signalling = 0;      /* p_signalling set by YHalt (?? see below) */
 
-  if (!(ym_state&Y_RUNNING) && p_signalling==-1) {
-    p_signalling = 0;    /* p_signalling set by YHalt (?? see below) */
-  } else {
-    ym_state &= ~Y_RUNNING;
-    p_abort();           /* p_signalling set by real signal */
-  }
+  /* reset Y_RUNNING to value on entry -- allows YRun to recurse */
+  ym_state = (ym_state & ~Y_RUNNING) | run_state;
+
+  if (p_signalling)
+    p_abort();             /* p_signalling set by real signal */
+}
+
+extern void ym_escape(void);
+void
+ym_escape(void)
+{
+  Y_quit(0);
+  if (p_signalling==-1) p_signalling = 0;
+  p_abort();
 }
 
 void
@@ -143,7 +153,6 @@ YHalt(void)
 }
 
 /* if read() interrupted by uncaught error, have a serious problem */
-extern void yr_reset(void);
 extern char *y_read_prompt;
 extern Instruction *ym_suspend(void);
 extern void ym_resume(Instruction *pc);
@@ -172,6 +181,15 @@ ym_resume(Instruction *ipc)
   if (!ipc) YError("(BUG) ym_resume from null pc");
   pc = ipc;
   caughtTask = 1;
+  /* protect against following sequence:
+   * (1) task suspends during include file, causing y_on_idle to return 0
+   *     this marks idler_eligible==0 in p_on_idle (alarms.c)
+   * (2) event arrives (e.g.- expose) and is handled as a pending event
+   *     before calling p_timeout; this event calls ym_resume
+   * (3) p_timeout is called, which should resume execution, but cannot
+   *     because idler_eligible has never been reset
+   */
+  p_timeout();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -179,72 +197,30 @@ ym_resume(Instruction *ipc)
 static Function **tasks= 0;
 static int nTasks= 0;
 static int maxTasks= 0;
-static int ym_immediate_mode = 0;
-
-static Function **ym_q = 0;
-static int ym_q_n = 0, ym_q_mx = 0, ym_q_i = 0;
 
 void ClearTasks(void)
 {
-  int i;
-  ym_immediate_mode = 0;
   while (nTasks) { nTasks--;  Unref(tasks[nTasks]); }
   if (maxTasks>16) {
     maxTasks= 0;
     p_free(tasks);
     tasks= 0;
   }
-  while (ym_q_n) {
-    ym_q_n--;
-    i = ym_q_i++;
-    if (ym_q_i >= ym_q_mx) ym_q_i = 0;
-    Unref(ym_q[i]);
-  }
-  if (ym_q_mx > 16) {
-    ym_q_mx = ym_q_i = 0;
-    p_free(ym_q);
-    ym_q = 0;
-  }
-}
-
-static void PushTask(Function *task);
-static void PushTask(Function *task)
-{
-  if (p_signalling) p_abort();
-  if (nTasks>=maxTasks) {
-    int newSize= maxTasks+16;
-    tasks= p_realloc(tasks, sizeof(Function *)*newSize);
-    maxTasks= newSize;
-  }
-  tasks[nTasks++]= Ref(task);  /* WARNING-- this is the reference for when
-				  DoTask pushes the task onto the stack--
-				  you are still giving your reference to the
-				  Function* away when you call PushTask */
 }
 
 void
-ym_queue(Function *task)
+PushTask(Function *task)
 {
-  if (!ym_immediate_mode) {
-    int j;
-    if (p_signalling) p_abort();
-    j = ym_q_i + ym_q_n;
-    if (ym_q_n >= ym_q_mx) {
-      int n = ym_q_mx + 16;
-      ym_q = p_realloc(ym_q, sizeof(Function *)*n);
-      ym_q_mx = n;
-      for (n=ym_q_mx-16 ; n<j ; n++) {
-        ym_q[n] = ym_q[n-ym_q_mx];
-        ym_q[n-ym_q_mx] = 0;
-      }
-      for ( ; n<ym_q_mx ; n++) ym_q[n] = 0;
-    }
-    if (j >= ym_q_mx) j -= ym_q_mx;
-    ym_q[j] = Ref(task); /* see note for PushTask */
-    ym_q_n++;
-  } else {
-    PushTask(task);
+  if (p_signalling) p_abort();
+  if (nTasks>=maxTasks) {
+    int newSize = maxTasks+16;
+    tasks = p_realloc(tasks, sizeof(Function *)*newSize);
+    maxTasks = newSize;
   }
+  tasks[nTasks++] = Ref(task);  /* WARNING-- this is the reference for when
+                                 * DoTask pushes the task onto the stack--
+                                 * you are still giving your reference to the
+                                 * Function* away when you call PushTask */
 }
 
 /* The task pseudo-code MUST be static, since YError may p_abort out
@@ -257,17 +233,10 @@ int DoTask(void)
   extern Operations debugOps;
   if (caughtTask) {
     caughtTask= 0;
-  } else if (nTasks>0 || ym_q_n>0) {
+  } else if (nTasks>0) {
     Function *task;
     if (p_signalling) p_abort();
-    if (nTasks > 0) {
-      task = tasks[--nTasks];
-    } else {
-      task = ym_q[ym_q_i];
-      ym_q[ym_q_i] = 0;
-      ym_q_n--;
-      if (++ym_q_i >= ym_q_mx) ym_q_i -= ym_q_mx;
-    }
+    task = tasks[--nTasks];
 
     CheckStack(1);
     (sp+1)->ops= &dataBlockSym;
@@ -284,7 +253,7 @@ int DoTask(void)
     /* actually, something is terribly wrong if this is not *main* */
     Drop(1);
   }
-  return nTasks+ym_q_n;
+  return nTasks;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -305,47 +274,71 @@ int yBatchMode= 0;
 static int y_was_idle= 0;
 
 static void ym_prompter(void);
+extern char *y_read_prompt;     /* ascio.c */
+extern int yg_blocking;         /* graph.c */
+/* yp_did_prompt reset by y_on_stdin */
+extern int yp_did_prompt;
+int yp_did_prompt= 0;
 
 static void
 ym_prompter(void)
 {
-  extern void y_do_prompt(void);  /* yorick.c */
-  extern char *y_read_prompt;     /* ascio.c */
-  if (y_read_prompt) {
-    if (y_read_prompt[0]) p_stdout(y_read_prompt);
-  } else {
-    y_do_prompt();
+  if (!yp_did_prompt) {
+    if (y_read_prompt) {
+      if (y_read_prompt[0]) {
+        p_stdout(y_read_prompt);
+        yp_did_prompt = 1;
+      }
+    } else {
+      extern void y_do_prompt(void);  /* yorick.c */
+      if (!yg_blocking) {
+        y_do_prompt();
+        yp_did_prompt = 1;
+      }
+    }
   }
 }
 
 int
 y_on_idle(void)
 {
-  int more_work= 0;
+  int more_work = 0;
+  int pending_stdin = 0;
 
   if (!taskCodeInit) {
-    taskCode[0].Action= &Eval;
-    taskCode[1].count= 0;
-    taskCode[2].Action= &YHalt;
-    taskCode[3].index= 0;
-    taskCodeInit= 1;
+    taskCode[0].Action = &Eval;
+    taskCode[1].count = 0;
+    taskCode[2].Action = &YHalt;
+    taskCode[3].index = 0;
+    taskCodeInit = 1;
   }
 
-  if (ym_state & Y_QUITTING) return 0;
-  if (!(nTasks+ym_q_n+caughtTask)) {
+  if (ym_state & Y_QUITTING) {
+  die_now:
+    y_cleanup();
+    p_quit();
+    return 0;
+  }
+  if (!(nTasks+caughtTask)) {
+    extern int y_pending_stdin(void);  /* yinput.c */
     if (nYpIncludes || nYpInputs) {
-      ym_immediate_mode = 0;
       YpParse((void *)0);
-    } else if (y_idler_function) {
-      Function *f= y_idler_function;
-      y_idler_function= 0;
-      PushTask(f);
-      Unref(f);
+    } else {
+      pending_stdin = y_pending_stdin();
+      if (!(nTasks+caughtTask || nYpIncludes || nYpInputs) &&
+          y_idler_function) {
+        Function *f = y_idler_function;
+        y_idler_function = 0;
+        PushTask(f);
+        Unref(f);
+      }
     }
   }
 
-  if (nTasks+ym_q_n+caughtTask) DoTask();
-  more_work= (nTasks+ym_q_n+caughtTask || nYpIncludes || nYpInputs);
+  if (nTasks+caughtTask) DoTask();
+  if (ym_state & Y_QUITTING) goto die_now;
+  more_work = (nTasks+caughtTask || nYpIncludes || nYpInputs ||
+               pending_stdin || y_idler_function);
 
   /* non-0 return means we want to run again
    * -- need to prompt if nothing left to do, but want to check
@@ -353,12 +346,27 @@ y_on_idle(void)
    * thus, first time we are out of tasks return non-0 anyway,
    * but prompt and return 0 second consecutive time we are out */
   if (more_work)
-    y_was_idle= 0;
+    {
+      y_was_idle = 0;
+
+      /* probably incorrect -- any tasks created before the suspend
+       * but not yet run will be blocked
+       * or maybe that's correct behavior?  what are the implied
+       * dependencies of one task on another?
+       */
+
+      /* block on window,wait=1 or mouse() from #include file */
+      /* if (yg_blocking) more_work=0; */
+      /* block whenever suspended for any reason from #include file */
+      if (ym_state&Y_SUSPENDED) more_work = 0;
+      /* deliver prompt on read() or rdline() from #include file */
+      if (y_read_prompt) ym_prompter();
+    }
   else if (y_was_idle)
-    y_was_idle= 0, ym_prompter();
+    y_was_idle=0, ym_prompter();
   else
-    y_was_idle= more_work= !yBatchMode;
-  if (!(nTasks+ym_q_n+caughtTask)) {
+    y_was_idle = more_work = !yBatchMode;
+  if (!(nTasks+caughtTask)) {
     extern void yg_before_wait(void);
     yg_before_wait();
   }
@@ -369,55 +377,58 @@ void Y_quit(int nArgs)
 {
   ym_state|= Y_QUITTING;
   ResetStack(0);
-  y_cleanup();
-  p_quit();
   YHalt();
 }
 
+static int did_cleanup = 0;
+
 int
 y_on_quit(void)
 {
-  return 0;
+  if (!did_cleanup) y_cleanup();
+  return ym_fatal;
 }
 
-static volatile int detectRecursion= 0;
-static IOStream *yclean_file= 0;
+static volatile int detectRecursion = 0;
+static IOStream *yclean_file = 0;
 
 void
 y_cleanup(void)
 {
   if (detectRecursion) {
-    if (detectRecursion!=2) {
-      detectRecursion= 2;
-      Unref(yclean_file);
-    }
-    detectRecursion= 0;
-  }
-  /* close all binary files properly */
-  while (yBinaryFiles) {
-    yclean_file= yBinaryFiles->ios;
-    yclean_file->references= 0;
-    if (yclean_file==yBinaryFiles->ios) {
-      detectRecursion= 1;
+    if (detectRecursion < 2) {
+      detectRecursion = 3;
       RemoveIOLink(yBinaryFiles, yclean_file);
-      detectRecursion= 0;
+    } else {
+      yBinaryFiles = 0;
     }
+    detectRecursion = 0;
+  }
+  /* attempt to close all binary files properly */
+  while (yBinaryFiles) {
+    yclean_file = yBinaryFiles->ios;
+    yclean_file->references = 0;
+    detectRecursion = 1;
+    Unref(yclean_file);
+    detectRecursion = 2;
+    RemoveIOLink(yBinaryFiles, yclean_file);
   }
 
   if (CleanUpForExit) CleanUpForExit();
+  did_cleanup = 1;
 }
 
 void RunTaskNow(Function *task)
 {
-  Instruction *pcHere= pc;
-  int t0= nTasks;
+  Instruction *pcHere = pc;
+  int t0 = nTasks;
   if (ym_state & Y_SUSPENDED)
-    YError("illegal operation while suspended waiting for event");
+    YError("RunTaskNow called while suspended waiting for event");
   PushTask(Ref(task));
-  ym_immediate_mode = 1;
-  while (nTasks>t0 && !(ym_state&Y_QUITTING)) DoTask();
-  ym_immediate_mode = 0;
-  pc= pcHere;           /* may have been clobbered by DoTask */
+  while (nTasks>t0 && !(ym_state&(Y_QUITTING|Y_SUSPENDED))) DoTask();
+  if (ym_state & Y_SUSPENDED)
+    YError("(read, pause, wait=1, etc.) suspended during RunTaskNow");
+  pc = pcHere;           /* may have been clobbered by DoTask */
   if (ym_state&Y_QUITTING) YHalt();
 }
 
@@ -427,37 +438,267 @@ void IncludeNow(void)
   int i0= nYpIncludes;
   int t0= nTasks;
   if (ym_state & Y_SUSPENDED)
-    YError("require or include while suspended waiting for event");
-  ym_immediate_mode = 1;
+    YError("IncludeNow called while suspended waiting for event");
   for (;;) {
     while (nTasks<=t0 && (nYpIncludes>i0 || (nYpIncludes==i0 &&
-	   ypIncludes[i0-1].file))) YpParse((void *)0);
-    while (nTasks>t0 && !(ym_state&Y_QUITTING)) DoTask();
-    if ((ym_state&Y_QUITTING) || nYpIncludes<i0 ||
+           ypIncludes[i0-1].file))) YpParse((void *)0);
+    while (nTasks>t0 && !(ym_state&(Y_QUITTING|Y_SUSPENDED))) DoTask();
+    if ((ym_state&(Y_QUITTING|Y_SUSPENDED)) || nYpIncludes<i0 ||
         !ypIncludes[i0-1].file) break;
   }
-  ym_immediate_mode = 0;
+  if (ym_state & Y_SUSPENDED)
+    YError("(read, pause, wait=1, etc.) suspended during IncludeNow");
   pc= pcHere;           /* may have been clobbered by DoTask */
   if (ym_state&Y_QUITTING) YHalt();
 }
 
-void Y_include(int nArgs)
+void
+Y_include(int nArgs)
 {
   long now;
   char *name;
   if (nArgs!=1 && nArgs!=2)
     YError("include function takes exactly one or two arguments");
-  if (nArgs>1) now= YGetInteger(sp);
-  else now= 0;
-  name= YGetString(sp-nArgs+1);
-  if (now>=0 && !YpPushInclude(name))
-    YError("missing include file specified in include function");
-  else if (now<0)
-    YpPush(name);          /* defer until all pending input parsed */
+  now = (nArgs>1)? YGetInteger(sp) : 0;
+  name = YGetString(sp-nArgs+1);
+  if (name[0]) {  /* name=="" special hack cleans out pending includes */
+    if (now >= 0) {
+      if (!YpPushInclude(name)) {
+        if (!(now&2))
+          YError("missing include file specified in include function");
+        now = 0;
+      }
+    } else {
+      YpPush(name);          /* defer until all pending input parsed */
+    }
+  }
   Drop(nArgs);
   if (now>0) IncludeNow(); /* parse and maybe execute file to be included
-			      -- without now, this won't happen until the
-			      next line is parsed naturally */
+                            * -- without now, this won't happen until the
+                            * next line is parsed naturally */
+}
+
+static char **yplug_path = 0;
+
+void
+Y_plug_dir(int nArgs)
+{
+  Dimension *dims = 0;
+  char **d = (nArgs==1)? YGet_Q(sp, 1, &dims) : 0;
+  long nd = d? TotalNumber(dims) : 0;
+  int i;
+  if (nArgs > 1)
+    YError("plug_dir function takes at most one argument");
+  if (yplug_path) {
+    for (i=0 ; yplug_path[i] ; i++) {
+      p_free(yplug_path[i]);
+      yplug_path[i] = 0;
+    }
+    p_free(yplug_path);
+    yplug_path = 0;
+  }
+  yplug_path = p_malloc(sizeof(char *)*(nd+2));
+  for (i=0 ; i<=nd ; i++) {
+    if (i < nd) {
+      yplug_path[i] = YExpandName(d[i]);
+      YNameToHead(&yplug_path[i]);
+    } else {
+      yplug_path[i] = p_strncat(yHomeDir, "lib/", 0L);
+    }
+  }
+  yplug_path[nd+1] = 0;
+}
+
+struct y_package_t {
+  char *name;
+  y_pkg_t *init;
+  char **ifiles;
+  BuiltIn **code;
+  void **data;
+  char **varname;
+};
+static struct y_package_t *y_pkg_list = 0;
+static int y_npkg = 0;
+static int y_n0pkg = 0;
+
+char *
+y_pkg_name(int i)
+{
+  return (i<0 || i>=y_npkg || !y_pkg_list)? 0 : y_pkg_list[i].name;
+}
+
+int
+y_pkg_count(int i)
+{
+  return i? y_npkg : y_n0pkg;
+}
+
+y_pkg_t *
+y_pkg_lookup(char *name)
+{
+  int i;
+  for (i=0 ; i<y_npkg ; i++)
+    if (!strcmp(y_pkg_list[i].name, name))
+      return y_pkg_list[i].init;
+  return 0;
+}
+
+void
+y_pkg_add(y_pkg_t *init)
+{
+  char **ifiles, **varname;
+  BuiltIn **code;
+  void **data;
+  char *pkgname = init(&ifiles, &code, &data, &varname);
+  if (!y_pkg_list)
+    y_pkg_list = p_malloc(sizeof(struct y_package_t)*8);
+  else if (!(y_npkg & 7))
+    y_pkg_list = p_malloc(sizeof(struct y_package_t)*(y_npkg+8));
+  y_pkg_list[y_npkg].name = pkgname;
+  y_pkg_list[y_npkg].init = init;
+  y_pkg_list[y_npkg].ifiles = ifiles;
+  y_pkg_list[y_npkg].code = code;
+  y_pkg_list[y_npkg].data = data;
+  y_pkg_list[y_npkg].varname = varname;
+  y_npkg++;
+}
+
+static void y_pkg_0link(char **varname, BuiltIn **code, void **data);
+
+void
+y_pkg_link(char *name)
+{
+  /* use name==0 to relink every package */
+  int i;
+  if (!y_n0pkg) y_n0pkg = y_npkg;
+  for (i=0 ; i<y_npkg ; i++) {
+    if (!name || !strcmp(y_pkg_list[i].name, name)) {
+      y_pkg_0link(y_pkg_list[i].varname,
+                  y_pkg_list[i].code, y_pkg_list[i].data);
+      if (name) break;
+    }
+  }
+}
+
+static void
+y_pkg_0link(char **varname, BuiltIn **code, void **data)
+{
+  long index;
+  DataBlock *db;
+
+  /* initialize built-in functions */
+  if (code) while (*code) {
+    index = Globalize(*varname++, 0L);
+    db = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+    if (!db || db->ops!=&builtinOps ||
+        ((BIFunction *)db)->function!=*code) {
+      globTab[index].value.db = (DataBlock *)NewBIFunction(*code, index);
+      globTab[index].ops = &dataBlockSym;
+      Unref(db);
+    }
+    code++;
+  }
+
+  /* initialize compiled variables */
+  if (data) while (*data) {
+    index = Globalize(*varname++, 0L);
+    db = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+    if (!db || db->ops!=&lvalueOps || ((LValue *)db)->type.base->file ||
+        ((LValue *)db)->owner || ((LValue *)db)->address.m!=*data) {
+      /* note: everything starts out as a scalar char at the correct address
+       * - subsequent reshape will set true data type and dimensions
+       */
+      globTab[index].value.db =
+        (DataBlock *)NewLValueM((Array *)0, *data++,
+                                &charStruct, (Dimension *)0);
+      globTab[index].ops = &dataBlockSym;
+    }
+    Unref(db);
+  }
+}
+
+void
+y_pkg_include(char *name, int now)
+{
+  char **ifiles, *msg;
+  int i;
+  /* use name==0 to relink every package */
+  if (now) {
+    for (i=0 ; i<y_npkg ; i++) {
+      if (!name || !strcmp(y_pkg_list[i].name, name)) {
+        ifiles = y_pkg_list[i].ifiles;
+        if (ifiles) while (ifiles[0]) {
+          if (YpPushInclude(*ifiles++)) {
+            IncludeNow();
+          } else {
+            msg= p_strncat("missing include file ", ifiles[-1], 0);
+            YWarning(msg);
+            p_free(msg);
+          }
+        }
+        if (name) break;
+      }
+    }
+  } else {
+    /* when not immediate (e.g.- at startup) push in reverse order */
+    for (i=y_npkg-1 ; i>=0 ; i--) {
+      if (!name || !strcmp(y_pkg_list[i].name, name)) {
+        ifiles = y_pkg_list[i].ifiles;
+        if (ifiles) {
+          while (ifiles[0]) ifiles++;
+          for (ifiles-- ; ifiles>=y_pkg_list[i].ifiles ; ifiles--)
+            YpPush(ifiles[0]);
+        }
+        if (name) break;
+      }
+    }
+  }
+}
+
+void
+Y_plug_in(int nArgs)
+{
+  char *pname, *pkgname;
+  y_pkg_t *init = 0;
+  void *plug;
+  int i;
+  if (nArgs!=1) YError("plug_in function takes exactly one argument");
+  pkgname = YGetString(sp-nArgs+1);
+  if (!pkgname || !pkgname[0]) 
+    YError("plug_in: package name argument is null");
+  for (i=1 ; pkgname[i] ; i++);
+  for (pname=pkgname+i-1 ; pname>pkgname ; pname--)
+    if (pname[0]=='/' || pname[0]=='\\') break;
+
+  /* become a no-op if pkgname already plugged in */
+  if (y_pkg_lookup(pname)) return;
+
+  if (!yplug_path) Y_plug_dir(0);
+
+  plug = p_dlopen(pkgname);
+  if (!plug && !YIsAbsolute(pkgname)) {
+    /* check in plug_dir and Y_HOME/lib before giving up */
+    char *tmp;
+    for (i=0 ; yplug_path[i] ; i++) {
+      tmp = p_strncat(yplug_path[i], pkgname, 0L);
+      plug = p_dlopen(tmp);
+      p_free(tmp);
+      if (plug) break;
+    }
+  }
+  if (plug) {
+    char *tmp = p_strncat("yk_", pname, 0);
+    int failed = p_dlsym(plug, tmp, 0, &init);
+    p_free(tmp);
+    if (failed || !init)
+      YError("plug_in: dynamic library missing yk_<pkgname> function");
+  }
+  if (init) {
+    y_pkg_add(init);
+    y_pkg_link(pname);
+  } else {
+    YError("plug_in: unable to find dynamic library file");
+  }
 }
 
 void Y_require(int nArgs)
@@ -560,7 +801,10 @@ y_on_exception(int signal, char *errmsg)
       YError((errmsg&&errmsg[0])? errmsg :
              "Unrecognized signal delivered to y_on_exception");
   }
-  if (ym_state&Y_QUITTING) p_quit();
+  if (ym_state&Y_QUITTING) {
+    y_cleanup();
+    p_quit();
+  }
 }
 
 void YWarning(const char *msg)
@@ -607,6 +851,7 @@ YError(const char *msg)
   if (recursing>8 || yImpossible>8) {
     YputsErr("****FATAL**** YError looping -- quitting now");
     ym_state|= Y_QUITTING;
+    ym_fatal = 3;
     if (!no_abort) p_abort();
     return;
   }
@@ -625,8 +870,8 @@ YError(const char *msg)
     YputsErr("****OOPS**** error on read resume or throw/catch");
   }
 
-  func= ((ym_state&Y_RUNNING) && !recursing && pcUp!=&taskCode[2])?
-    FuncContaining(pcDebug) : 0;
+  func= (((ym_state&Y_RUNNING)||no_abort) && !recursing &&
+         pcUp!=&taskCode[2])? FuncContaining(pcDebug) : 0;
   name= func? globalTable.names[func->code[0].index] : "VM idle or lost";
 
   /* Clear out include stack, but remember current include file name.
@@ -652,14 +897,14 @@ YError(const char *msg)
   if (recursing<3) {
     p_qclear();
     ClearTasks();
-  } else if (nTasks+ym_q_n) {
+  } else if (nTasks) {
     YputsErr("WARNING unable to free task pointers in YError");
-    nTasks = ym_q_n = 0;
+    nTasks = 0;
   }
 
   /* Print error message, with name of current Yorick function prepended:
         ERROR (yorick_function) msg passed to YError
-	ERROR (VM idle or lost) msg passed to YError
+        ERROR (VM idle or lost) msg passed to YError
      The second form is used when the virtual machine is idle at the
      time of the error.  */
   if (!pcUp || recursing)
@@ -675,9 +920,12 @@ YError(const char *msg)
   if (recursing) {
     func= 0;
     YputsErr("WARNING aborting on recursive calls to YError");
-  } else if (y_read_prompt) {
+  } else if (ym_state&Y_SUSPENDED) {
     func= 0;
-    YputsErr("WARNING aborting on YError during keyboard read()");
+    if (y_read_prompt)
+      YputsErr("WARNING aborting on YError during keyboard read()");
+    else
+      YputsErr("WARNING aborting on YError after mouse() pause() or wait=1");
   }
 
   if (func) {
@@ -688,20 +936,20 @@ YError(const char *msg)
       name= includeFile;
     } else {
       char *mess= YpReparse(func);
-      if (nTasks+ym_q_n) ClearTasks();
+      if (nTasks) ClearTasks();
       if (mess[0]!='*') {
-	/* reparse succeeded, skip to filename */
-	name= 0;
-	while (mess[0] && mess[0]!=':') mess++;
-	if (mess[0]) do { mess++; } while (mess[0] && mess[0]!=':');
-	if (mess[0]) {
-	  mess++;
-	  if (mess[0]==' ') mess++;
-	  if (mess[0]) name= mess;
-	}
+        /* reparse succeeded, skip to filename */
+        name= 0;
+        while (mess[0] && mess[0]!=':') mess++;
+        if (mess[0]) do { mess++; } while (mess[0] && mess[0]!=':');
+        if (mess[0]) {
+          mess++;
+          if (mess[0]==' ') mess++;
+          if (mess[0]) name= mess;
+        }
       } else {
-	/* reparse failed */
-	name= 0;
+        /* reparse failed */
+        name= 0;
       }
       beginLine= 0;  /* used only for *main* from includeFile */
     }
@@ -719,10 +967,10 @@ YError(const char *msg)
 
     if (!yBatchMode && !pcUp && (!yAutoDebug || yDebugLevel>1)) {
       if (yDebugLevel>1) {
-	YputsErr(" To enter recursive debug level, type <RETURN> now");
+        YputsErr(" To enter recursive debug level, type <RETURN> now");
       } else {
-	YputsErr(" To enter debug mode, type <RETURN> now"
-		 " (then dbexit to get out)");
+        YputsErr(" To enter debug mode, type <RETURN> now"
+                 " (then dbexit to get out)");
       }
       ym_dbenter = 1;
     }
@@ -734,9 +982,10 @@ YError(const char *msg)
     if (recursing<5) ResetStack(y_read_prompt!=0);
     else {
       YputsErr("****SEVERE**** YError unable to reset stack -- "
-	       "probably lost variables");
+               "probably lost variables");
       sp= spBottom;
     }
+    ym_state &= ~Y_SUSPENDED;
     if (y_read_prompt) yr_reset();
     yg_got_expose();  /* in case window,wait=1 or pause */
     p_clr_alarm(0, 0);
@@ -751,6 +1000,7 @@ YError(const char *msg)
   if (yBatchMode || !strncmp(msg,"(FATAL)",7)) {
     if (yBatchMode) YputsErr("yorick: quitting on error in batch mode");
     else YputsErr("yorick: quitting on fatal error");
+    ym_fatal = yBatchMode? 1 : 2;
     ResetStack(0);
     ym_state|= Y_QUITTING;
   }
@@ -787,15 +1037,15 @@ void Y_help(int nArgs)
       DataBlock *db= stack->value.db;
       Operations *ops= db->ops;
       if (ops==&functionOps) {
-	Function *f= (Function *)db;
-	index= f->code[0].index;
+        Function *f= (Function *)db;
+        index= f->code[0].index;
       } else if (ops==&structDefOps) {
-	StructDef *base= (StructDef *)db;
-	while (base->model) base= base->model;
-	index= Globalize(yStructTable.names[base->index], 0L);
+        StructDef *base= (StructDef *)db;
+        while (base->model) base= base->model;
+        index= Globalize(yStructTable.names[base->index], 0L);
       } else if (ops==&builtinOps) {
-	BIFunction *f= (BIFunction *)db;
-	index= f->index;
+        BIFunction *f= (BIFunction *)db;
+        index= f->index;
       }
     }
     Drop(nAbove);
@@ -804,7 +1054,7 @@ void Y_help(int nArgs)
     /* no legal arguments, help function itself is target */
     BIFunction *f= (BIFunction *)(sp-nArgs)->value.db;
     index= f->index;
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   }
 
   /* move help topic argument off stack into help_topic extern variable */
@@ -814,17 +1064,17 @@ void Y_help(int nArgs)
   if (!help_worker) {
     long help_index= Globalize("help_worker", 0L);
     if (globTab[help_index].ops!=&dataBlockSym ||
-	(help_worker= (Function *)Ref(globTab[help_index].value.db))->ops
-	!=&functionOps)
+        (help_worker= (Function *)Ref(globTab[help_index].value.db))->ops
+        !=&functionOps)
       YError("(BUG) help_worker function not found -- help unavailable");
   }
 
   /* create help_file extern variable */
   if (index>=0 && (file= OpenSource(index)))
     PushDataBlock(NewTextStream(p_strcpy(ypIncludes[nYpIncludes-1].filename),
-				file, 1, ypBeginLine-1, p_ftell(file)));
+                                file, 1, ypBeginLine-1, p_ftell(file)));
   else
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   worker_arg= Globalize("help_file", 0L);
   PopTo(&globTab[worker_arg]);  /* help_file */
 
@@ -922,7 +1172,8 @@ static Catcher *CatchScan(const char *msg, int category)
       ClearStack();
       if (spCatch >= sp) break;
       pcRet= AbortReturn();
-      if (!pcRet || pcRet==&taskCode[2]) YError("(BUG) impossible catch");
+      if (!pcRet || pcRet==&taskCode[2])
+        YError("catch does not work outside immediate include or require");
     }
     if (spCatch!=sp) YError("(BUG) impossible catch or corrupt stack");
     pc= catchers[i].pc;
