@@ -43,16 +43,20 @@ if test -z "$Y_HOME"; then
   fi
   Y_HOME="$Y_SITE/$Y_PLATFORM"
 fi
+if test -z "$Y_BINDIR"; then
+  Y_BINDIR='$(Y_HOME)/bin'
+fi
 
-Y_VERSION="$VERSION".`tail -1 ../VERSION`
+Y_VERSION="$VERSION".`tail -n 1 ../VERSION`
 echo "Y_VERSION=$Y_VERSION" >>../Make.cfg
 echo "Y_SITE=$Y_SITE" >>../Make.cfg
 echo "Y_HOME=$Y_HOME" >>../Make.cfg
+echo "Y_BINDIR=$Y_BINDIR" >>../Make.cfg
 
 # extract global variables from ../Make.cfg
 CC=`grep '^CC=' ../Make.cfg | sed -e s/CC=//`
-COPTIONS=`grep '^COPTIONS=' ../Make.cfg | sed -e s/COPTIONS=//`
-LDOPTIONS=`grep '^LDOPTIONS=' ../Make.cfg | sed -e s/LDOPTIONS=//`
+COPTIONS=`grep '^Y_CFLAGS=' ../Make.cfg | sed -e s/Y_CFLAGS=//`
+LDOPTIONS=`grep '^Y_LDFLAGS=' ../Make.cfg | sed -e s/Y_LDFLAGS=//`
 MATHLIB=`grep '^MATHLIB=' ../Make.cfg | sed -e s/MATHLIB=//`
 FPELIB=`grep '^FPELIB=' ../Make.cfg | sed -e s/FPELIB=//`
 if test -z "$FPELIB"; then
@@ -60,9 +64,14 @@ if test -z "$FPELIB"; then
 else
   LOWLIBS="$MATHLIB" "$FPELIB"
 fi
-XLIB=`grep '^XLIB=' ../Make.cfg | sed -e s/XLIB=//`
+X11LIB=`grep '^X11LIB=' ../Make.cfg | sed -e s/X11LIB=//`
 RANLIB=`grep '^RANLIB=' ../Make.cfg | sed -e s/RANLIB=//`
+AR=`grep '^AR=' ../Make.cfg | sed -e s/AR=//`
 
+if test "$HACK103" = "yes"; then
+  echo "will emulate hypot to avoid system sqrt (HACK103)"
+  NO_HYPOT="-DNO_HYPOT"
+else
 cat >cfg.c <<EOF
 int main(int argc, char *argv[])
 {
@@ -77,6 +86,7 @@ else
   echo "libm does not contain hypot, will emulate"
   NO_HYPOT="-DNO_HYPOT"
 fi
+fi
 echo "NO_HYPOT=$NO_HYPOT" >>../Make.cfg
 
 cat >cfg.c <<EOF
@@ -106,33 +116,6 @@ FORTRAN_LINKAGE=-Df_linkage_
 CXX=CC
 echo "FORTRAN_LINKAGE=$FORTRAN_LINKAGE" >>../Make.cfg
 
-echo "OPT_DIRS=drat hex" >>../Make.cfg
-
-# eventually should just cat Make.cfg onto Maketmpl.in ??
-# create and apply sed script to go from Maketmpl.in to Maketmpl
-cat >cfg.sed <<EOF
-s%@Y_VERSION@%$Y_VERSION%
-s%@Y_SITE@%$Y_SITE%
-s%@Y_HOME@%$Y_HOME%
-s%@CC@%$CC%
-s%@DEFS@%%
-s%@LDOPTS@%%
-s%@LOWLIBS@%$LOWLIBS%
-s%@XLOAD@%$XLIB -lX11%
-s%@D_NO_XLIB@%%
-s%@NO_HYPOT@%$NO_HYPOT%
-s%@NO_EXP10@%$NO_EXP10%
-s%@RANLIB@%$RANLIB%
-s%@D_NO_STRTOUL@%$D_NO_STRTOUL%
-s%@FC@%$FC%
-s%@FORTRAN_LIBS@%$FORTRAN_LIBS%
-s%@FORTRAN_LINKAGE@%$FORTRAN_LINKAGE%
-s%@CXX@%$CXX%
-EOF
-
-sed -f cfg.sed <Maketmpl.in >Maketmpl
-rm -f cfg.sed
-
 #----------------------------------------------------------------------
 # clean up, issue warning if compiler gave fishy output
 rm -f cfg cfg.c cfg.o
@@ -146,7 +129,6 @@ for f in cfg.[0-9]*; do
 done
 
 echo "appended to ../Make.cfg"
-echo "generated yorick/Maketmpl"
 echo ""
 echo "  ============= end yorick configuration =============="
 
