@@ -21,9 +21,9 @@ extern BuiltIn Y_yorick_stats, Y_symbol_def, Y_symbol_set;
 
 /* Required for FetchLValue, StoreLValue */
 extern void ReadGather(void *dst, void *srcM, long srcD, StructDef *base,
-		       long number, const Strider *strider);
+                       long number, const Strider *strider);
 extern void WriteScatter(void *src, void *dstM, long dstD, StructDef *base,
-			 long number, const Strider *strider);
+                         long number, const Strider *strider);
 
 /* function defined in yrdwr.c required for StoreLValue */
 extern void SetSequentialWrite(IOStream *file, long last);
@@ -53,8 +53,8 @@ Array *GrowArray(Array *array, long extra)
 }
 
 Function *NewFunction(Symbol *consts, long nConsts, int nPos, int nKey,
-		      int nLocal, long hasPL, int maxStackDepth,
-		      Instruction *code, long codeSize)
+                      int nLocal, long hasPL, int maxStackDepth,
+                      Instruction *code, long codeSize)
 {
   Function *func= p_malloc(sizeof(Function)+codeSize*sizeof(Instruction));
   long frameSize= 1+nPos+(hasPL&1)+nKey+nLocal;
@@ -98,7 +98,7 @@ void FreeFunction(void *v)  /* ******* Use Unref(func) ******* */
    at a time.  Since Range contains an ops pointer, the alignment
    of a Range must be at least as strict as a void*.  */
 static MemryBlock rangeBlock= {0, 0, sizeof(Range),
-				   64*sizeof(Range)};
+                                   64*sizeof(Range)};
 
 Range *NewRange(long min, long max, long inc, int nilFlags)
 {
@@ -122,7 +122,7 @@ void FreeRange(void *range)  /* ******* Use Unref(range) ******* */
    at a time.  Since LValue contains several pointers, the alignment
    of an LValue must be at least as strict as a void*.  */
 static MemryBlock lvalueBlock= {0, 0, sizeof(LValue),
-				    64*sizeof(LValue)};
+                                    64*sizeof(LValue)};
 
 LValue *NewLValueD(long address, StructDef *base, Dimension *dims)
 {
@@ -140,7 +140,7 @@ LValue *NewLValueD(long address, StructDef *base, Dimension *dims)
 }
 
 LValue *NewLValueM(Array *owner, void *address,
-		   StructDef *base, Dimension *dims)
+                   StructDef *base, Dimension *dims)
 {
   long number= TotalNumber(dims);
   LValue *lvalue= NextUnit(&lvalueBlock);
@@ -168,7 +168,7 @@ void FreeLValue(void *v)  /* ******* Use Unref(lvalue) ******* */
 /* Set up a block allocator which grabs space for 16 BIFunction
    objeccts at a time. */
 static MemryBlock bifBlock= {0, 0, sizeof(BIFunction),
-		                 16*sizeof(BIFunction)};
+                                 16*sizeof(BIFunction)};
 
 BIFunction *NewBIFunction(BuiltIn *bi, long index)
 {
@@ -187,7 +187,7 @@ void FreeBIFunction(void *bif)
   BIFunction *func= bif;
   strcpy(warning, "freeing builtin function ");
   strncat(warning,
-	  func->index>=0? globalTable.names[func->index] : "<nameless>", 40L);
+          func->index>=0? globalTable.names[func->index] : "<nameless>", 40L);
   YWarning(warning);
   FreeUnit(&bifBlock, func);
 }
@@ -213,7 +213,7 @@ Array *FetchLValue(void *db, Symbol *dsts)
   data= darray->value.c;
 
   ReadGather(data, memory, lvalue->address.d,
-	     base, lvalue->type.number, lvalue->strider);
+             base, lvalue->type.number, lvalue->strider);
   if (file && file->pointeeList.table.nItems) ClearPointees(file, 0);
 
   PopTo(dsts);
@@ -255,7 +255,7 @@ long Globalize(const char *name, long n)
   if (!HashAdd(&globalTable, name, n)) {
     HASH_MANAGE(globalTable, Symbol, globTab);
     globTab[hashIndex].ops= &dataBlockSym;
-    globTab[hashIndex].value.db= Ref(&nilDB);
+    globTab[hashIndex].value.db= RefNC(&nilDB);
   }
   return hashIndex;
 }
@@ -452,13 +452,13 @@ static int EvenConform(Dimension *ldims, Dimension *rdims)
     rdims= rdims->next;
     if (lnum!=rnum) {
       if (lnum==1) {
-	casts|= 2;  /* must broadcast left operand */
-	lnum= rnum;
+        casts|= 2;  /* must broadcast left operand */
+        lnum= rnum;
       } else if (rnum==1) {
-	casts|= 1;  /* must broadcast right operand */
+        casts|= 1;  /* must broadcast right operand */
       } else {
-	casts|= 4;  /* not conformable */
-	lnum= -lnum-rnum;  /* error marker */
+        casts|= 4;  /* not conformable */
+        lnum= -lnum-rnum;  /* error marker */
       }
     }
     tmpDims= NewDimension(lnum, 1L, tmpDims);
@@ -505,6 +505,19 @@ int CalledAsSubroutine(void)
   return pc->Action==&DropTop;
 }
 
+Operand *
+yarg_op(int iarg, Operand *op)
+{
+  if (iarg>=0) {
+    Symbol *s = sp - iarg;
+    if (s->ops) s->ops->FormOperand(s, op);
+    else op->ops=0, op=0;
+  } else {
+    op = 0;
+  }
+  return op;
+}
+
 long YGetInteger(Symbol *s)
 {
   Operand op;
@@ -518,6 +531,12 @@ long YGetInteger(Symbol *s)
   return *(long *)op.value;
 }
 
+long
+yarg_sl(int iarg)
+{
+  return (iarg>=0)? YGetInteger(sp-iarg) : 0;
+}
+
 double YGetReal(Symbol *s)
 {
   Operand op;
@@ -531,6 +550,12 @@ double YGetReal(Symbol *s)
   return *(double *)op.value;
 }
 
+double
+yarg_sd(int iarg)
+{
+  return (iarg>=0)? YGetReal(sp-iarg) : 0;
+}
+
 char *YGetString(Symbol *s)
 {
   Operand op;
@@ -541,6 +566,12 @@ char *YGetString(Symbol *s)
   return *(char **)op.value;
 }
 
+char *
+yarg_sq(int iarg)
+{
+  return (iarg>=0)? YGetString(sp-iarg) : 0;
+}
+
 int YNotNil(Symbol *s)
 {
   if (!s) return 0;  /* for use with YGetKeywords */
@@ -548,8 +579,14 @@ int YNotNil(Symbol *s)
   return !(s->ops==&dataBlockSym && s->value.db==&nilDB);
 }
 
+int
+yarg_nil(int iarg)
+{
+  return (iarg>=0)? !YNotNil(sp-iarg) : 1;
+}
+
 Symbol *YGetKeywords(Symbol *stack, int nArgs, char **keyNames,
-		     Symbol **symbols)
+                     Symbol **symbols)
 {
   int i;
   char *globName;
@@ -571,6 +608,25 @@ Symbol *YGetKeywords(Symbol *stack, int nArgs, char **keyNames,
   return s0? s0 : stack+1;
 }
 
+int
+yarg_keys(int iarg, char **knames, Symbol **ksymbols)
+{
+  char *key;
+  int i;
+  while (iarg > 0) {
+    if (sp[-iarg].ops) break;
+    key = globalTable.names[sp[-(iarg--)].index];
+    for (i=0 ; knames[i] ; i++) {
+      if (ksymbols[i]) continue;
+      if (strcmp(key, knames[i]) == 0) break;
+    }
+    if (!knames[i])
+      YError("unrecognized or duplicate keyword");
+    ksymbols[i] = sp - (iarg--);
+  }
+  return iarg;
+}
+
 IOStream *YGetFile(Symbol *stack)
 {
   IOStream *file;
@@ -583,6 +639,12 @@ IOStream *YGetFile(Symbol *stack)
   return file;
 }
 
+IOStream *
+yarg_file(int iarg)
+{
+  return (iarg>=0)? YGetFile(sp-iarg) : 0;
+}
+
 /*--------------------------------------------------------------------------*/
 
 /* Retrieve array arguments for foreign code wrappers,
@@ -601,6 +663,12 @@ char *YGet_C(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+char *
+yarg_c(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_C(sp-iarg, 0, dims) : 0;
+}
+
 short *YGet_S(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -612,6 +680,12 @@ short *YGet_S(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+short *
+yarg_s(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_S(sp-iarg, 0, dims) : 0;
+}
+
 int *YGet_I(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -627,6 +701,12 @@ int *YGet_I(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+int *
+yarg_i(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_I(sp-iarg, 0, dims) : 0;
+}
+
 long *YGet_L(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -642,6 +722,12 @@ long *YGet_L(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+long *
+yarg_l(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_L(sp-iarg, 0, dims) : 0;
+}
+
 float *YGet_F(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -653,6 +739,12 @@ float *YGet_F(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+float *
+yarg_f(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_F(sp-iarg, 0, dims) : 0;
+}
+
 double *YGet_D(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -668,6 +760,12 @@ double *YGet_D(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+double *
+yarg_d(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_D(sp-iarg, 0, dims) : 0;
+}
+
 double *YGet_Z(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -679,6 +777,12 @@ double *YGet_Z(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+double *
+yarg_z(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_Z(sp-iarg, 0, dims) : 0;
+}
+
 char **YGet_Q(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -690,6 +794,12 @@ char **YGet_Q(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+char **
+yarg_q(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_Q(sp-iarg, 0, dims) : 0;
+}
+
 void **YGet_P(Symbol *s, int nilOK, Dimension **dims)
 {
   Operand op;
@@ -701,6 +811,12 @@ void **YGet_P(Symbol *s, int nilOK, Dimension **dims)
   return op.value;
 }
 
+void **
+yarg_p(int iarg, Dimension **dims)
+{
+  return (iarg>=0)? YGet_P(sp-iarg, 0, dims) : 0;
+}
+
 int YGet_dims(const Dimension *dims, long *dlist, int maxDims)
 {
   int i, n= CountDims(dims);
@@ -753,7 +869,7 @@ void Y_yorick_stats(int nArgs)
   tmpDims= NewDimension(14L, 1L, (Dimension *)0);
   result= PushDataBlock(NewArray(&longStruct, tmpDims));
   result->value.l[0]= globalTable.nItems;
-  result->value.l[1]= yStackSize;
+  result->value.l[1]=  yStackSize;
   result->value.l[2]= p_nallocs-p_nfrees;
   result->value.l[3]= p_asmall;
   result->value.l[4]= ExtraNilRefs();
@@ -799,7 +915,7 @@ void Y_symbol_set(int nArgs)
       Array *result= NewArray(array->type.base, array->type.dims);
       glob->value.db= (DataBlock *)result;
       array->type.base->Copy(array->type.base, result->value.c,
-			     array->value.c, array->type.number);
+                             array->value.c, array->type.number);
     } else {
       if (array->ops==&lvalueOps) FetchLValue(array, sp);
       glob->value.db= Ref(sp->value.db);
