@@ -22,15 +22,6 @@ extern void YError(const char *msg);
 extern int yDebugLevel;        /* declared in ydata.h */
 extern HashTable globalTable;  /* declared in ydata.h */
 
-extern long ypBeginLine; /* defined in this file -- line number where
-			    most recent parse began for YError */
-
-extern int ypSkipIncludes; /* defined in this file -- flag set when
-			      scanning for a variable definition */
-
-extern long YpStandby(void);    /* used by ScanForFunc in yorick.y */
-extern long ScanForFunc(const char *fname, int notExtern);
-
 extern char *MakeErrorLine(long lineNumber, const char *filename);
 
 extern long YpLineNumber(void);
@@ -67,6 +58,16 @@ static int GetNextLine(p_file *file, int context);
 static long FindSource(long index);
 extern long ReopenSource(long index, int notExtern);
 
+extern int y_pending_stdin(void);
+static void y_add_line(char *line);
+static void y_remove_line(void);
+typedef struct y_line_t y_line_t;
+struct y_line_t {
+  y_line_t *next;
+  char *line;
+};
+static y_line_t *y_lhead=0, *y_ltail=0;
+
 static int yp_chk_hash(p_file *file, int context, char *line);
 static char *yp_strtok_text = 0;
 static char *yp_strtok(char *text, char *delim);
@@ -90,7 +91,9 @@ yp_strtok(char *text, char *delim)
   }
   start = text;
   /* the whole point is to skip Windows drive letter */
-  if (text[1]==':') text += 2;
+  if (text[1]==':' &&
+      ((text[0]>='A' && text[0]<='Z') ||
+       (text[0]>='a' && text[0]<='z'))) text += 2;
   /* scan until delimiter */
   while (text[0]) {
     for (i=0 ; delim[i] ; i++) if (text[0]==delim[i]) break;
@@ -177,12 +180,16 @@ static p_file *PushInclude(const char *filename, int clear)
     char *tmp;
     for (i=0 ; i<=nYpPrefixes ; i++) {
       if (i<nYpPrefixes) {
-	tmp= p_strncat(ypPrefixes[i], filename, 0);
-	name= YExpandName(tmp);
-	p_free(tmp);
+        tmp= p_strncat(ypPrefixes[i], filename, 0);
+        name= YExpandName(tmp);
+        p_free(tmp);
       } else {
-	name= YExpandName(filename);
-	if (!YIsAbsolute(name)) break;
+        /* this branch is probably a bug --
+         * if . is not on path probably should not find file...
+         * maybe protects against empty path?
+         */
+        name= YExpandName(filename);
+        if (!YIsAbsolute(name)) break;
       }
       file= p_fopen(name, "r");
       if (file) break;
@@ -201,7 +208,8 @@ static p_file *PushInclude(const char *filename, int clear)
 
   ypIncludes[nYpIncludes].file= file;
   ypIncludes[nYpIncludes].filename= name;
-  ypIncludes[nYpIncludes++].lastLineRead= 0;
+  ypIncludes[nYpIncludes].lastLineRead= 0;
+  ypIncludes[nYpIncludes++].index = -1;
   prevErrLine= -1;
   return file;
 }
@@ -214,9 +222,6 @@ void YpClearIncludes(void)
   if (nYpIncludes>0) {
     do {
       nYpIncludes--;
-      /* Note <stdio.h> might not contain any prototype for fclose, or it
-	 might contain the correct ANSI prototype, or it might contain
-	 a K+R style function declaration...  */
       if (ypIncludes[nYpIncludes].file)
         p_fclose(ypIncludes[nYpIncludes].file);
       p_free(ypIncludes[nYpIncludes].filename);
@@ -227,6 +232,9 @@ void YpClearIncludes(void)
   YaltNextLine= 0;
   if (nYpInputs>0) do p_free(ypInputs[--nYpInputs]); while (nYpInputs);
   else nYpInputs= 0;
+
+  /* also clear pending stdin lines */
+  while (y_lhead) y_remove_line();
 }
 
 static void ClearSourceList(const char *name)
@@ -314,11 +322,11 @@ static int GetNextLine(p_file *file, int context)
       ypIncludes[nYpIncludes-1].file= 0;
       /* Any errors here are serious enough to warrant a panic stop.  */
       if (hadError)
-	YError("****ABORTING PARSE**** error reading include file");
+        YError("****ABORTING PARSE**** error reading include file");
       if (!hadEOF)
-	YError("****ABORTING PARSE**** include file not ASCII text");
+        YError("****ABORTING PARSE**** include file not ASCII text");
     }
-    return 0;			/* just a normal EOF */
+    return 0;                   /* just a normal EOF */
   }
   if (nYpIncludes) {
     long lnum= ++ypIncludes[nYpIncludes-1].lastLineRead;
@@ -349,6 +357,40 @@ extern int yp_continue;
 int yp_continue = 0;
 extern int yp_did_prompt;
 
+static void
+y_add_line(char *line)
+{
+  y_line_t *yline = p_malloc(sizeof(y_line_t));
+  if (yline) {
+    yline->next = 0;
+    yline->line = p_strcpy(line);
+    if (y_ltail) {
+      y_ltail->next = yline;
+      y_ltail = yline;
+    } else {
+      y_lhead = y_ltail = yline;
+    }
+  }
+}
+
+static void
+y_remove_line(void)
+{
+  if (y_lhead) {
+    y_line_t *yline = y_lhead;
+    y_line_t *next = yline->next;
+    char *line = yline->line;
+    y_lhead = next;
+    if (next)
+      yline->next = 0;
+    else
+      y_ltail = 0;
+    yline->line = 0;
+    p_free(yline);
+    if (line) p_free(line);
+  }
+}
+
 void
 y_on_stdin(char *input_line)
 {
@@ -357,37 +399,51 @@ y_on_stdin(char *input_line)
   if (!y_read_prompt) {
     if (input_line)
       while (input_line[0]==' ' || input_line[0]=='\t') input_line++;
-    if (flag) {
-      if (input_line && input_line[0]) {
-        extern void ResetStack(int hard);
-        extern void yr_reset(void);
-        extern void yg_got_expose(void);
-        yp_continue = 0;
-        if (p_signalling) p_abort();
-        /* drop current debug block plus task and clean up vm state */
-        ResetStack(0);
-        yr_reset();
-        yg_got_expose();
-        p_clr_alarm(0, 0);
-      }
+    if (flag && input_line && input_line[0]) {
+      extern void yg_got_expose(void);
+      yp_continue = 0;
+      if (p_signalling) p_abort();
+      /* drop current debug block plus task and clean up vm state */
+      ResetStack(0);
+      yr_reset();
+      yg_got_expose();
+      p_clr_alarm(0, 0);
     }
     if (yg_blocking) {
       if (yg_blocking==2) {
         extern void yg_got_expose(void);
         yg_got_expose();  /* escape from pause on keyboard input */
+        if (input_line && input_line[0])
+          YputsErr("WARNING discarding keyboard input that aborts pause");
+        return;
       } else {
         YputsErr("WARNING discarding keyboard input during window wait");
         return;
       }
     }
-    flag = yp_chk_hash(0, yp_continue, input_line);
-    flag = yp_parse_keybd((flag&1)? 0 : input_line, !yp_continue);
-    yp_continue = (flag!=0 && flag!=1);
+    y_add_line(input_line);
   } else {
     yr_reader(input_line);
   }
 }
 
+int
+y_pending_stdin(void)
+{
+  int flag;
+  if (y_lhead) {
+    do {
+      flag = yp_chk_hash(0, yp_continue, y_lhead->line);
+      flag = yp_parse_keybd((flag&1)? 0 : y_lhead->line, !yp_continue);
+      yp_continue = (flag!=0 && flag!=1);
+      y_remove_line();
+    } while (y_lhead && yp_continue);
+    return (y_lhead!=0);
+  } else {
+    return 0;
+  }
+}
+
 /* possible results:
  * 0 -- this line was not #include (or ypSkipIncludes), #if, or #endif
  *      or was a syntax error
@@ -457,7 +513,7 @@ yp_chk_hash(p_file *file, int context, char *line)
               while (*line && (*line==' ' || *line=='\t')) line++;
               if (line[0]=='i' && line[1]=='f' &&
                   (line[2]==' ' || line[2]=='\t')) {
-                count++;	/* nested #if (don't bother checking 0) */
+                count++;        /* nested #if (don't bother checking 0) */
               } else {
                 char *endi = "endif";
                 while (*line && *endi && (*line++ == *endi++));
@@ -507,11 +563,12 @@ YpNextLine(int context)
   /* get the current include file */
   if (nYpIncludes==0) {
     file = 0;
-  } else do {
+  } else for (;;) {
     if ((file=ypIncludes[nYpIncludes-1].file)) break;
     need_endif = 0;
     p_free(ypIncludes[nYpIncludes-1].filename);
-  } while (--nYpIncludes);
+    if (!(--nYpIncludes)) break;
+  }
   if (!file && nYpInputs) do file = YpPop(); while (!file && nYpInputs);
 
   /* quit if input is not from include file.  */
@@ -524,7 +581,7 @@ YpNextLine(int context)
     /* if first line of file begins with #! discard it
      * (allows yorick source files to be scripts on most UNIX systems) */
     if (nYpIncludes && ypIncludes[nYpIncludes-1].lastLineRead==1 &&
-	ypBuffer.line[0]=='#' && ypBuffer.line[1]=='!') {
+        ypBuffer.line[0]=='#' && ypBuffer.line[1]=='!') {
       if (!GetNextLine(file, context)) return 0;
     }
 
