@@ -19,7 +19,7 @@
 /* Print action defined here */
 extern VMaction Print;
 
-extern UnaryOp EvalFN, EvalBI;   /* required by Print */
+extern UnaryOp EvalFN, EvalBI, eval_auto;   /* required by Print */
 extern BuiltIn Y_print, Y_print_format;
 
 /* required by print() called as function */
@@ -99,7 +99,7 @@ int (*YputsErr)(char *s)= &YDputsErr;
 char *Ygets(YgetsLine *getsLine, p_file *file)
 {
   char *prev, *line= getsLine->line;
-  int n, maxChars= getsLine->max;
+  int ateof, n, maxChars= getsLine->max;
 
   if (!file) YError("(BUG) file==0 to Ygets no longer legal");
 
@@ -111,12 +111,12 @@ char *Ygets(YgetsLine *getsLine, p_file *file)
   }
 
   /* Take care not to try to read a stream that has already returned EOF.  */
-  if (Yfeof(file) ||
-      !Yfgets(line, maxChars+1, file) ||
-      Yferror(file)) goto abort;
+  if (Yfeof(file) || !Yfgets(line, maxChars+1, file)) goto abort;
+  ateof = Yfeof(file);
+  if (!ateof && Yferror(file)) goto abort;
 
   n= strlen(line);
-  while (line[n-1]!='\n' && !Yfeof(file)) {
+  while (line[n-1]!='\n' && !ateof) {
     if (maxChars>=MAX_LINE) goto abort;
     prev= line;
     getsLine->line= line= p_malloc(maxChars+INC_LINE+1);
@@ -127,9 +127,10 @@ char *Ygets(YgetsLine *getsLine, p_file *file)
 
     /* EOF is possible, in which case Yfgets returns 0 (ignored here)
        and leaves line unmodified.  The following strlen is guaranteed
-       to return 0, and the loop terminates on the Yfeof test above.  */
+       to return 0, and the loop terminates on the ateof test above.  */
     Yfgets(&line[n], maxChars+1-n, file);
-    if (Yferror(file)) goto abort;
+    ateof = Yfeof(file);
+    if (!ateof && Yferror(file)) goto abort;
     n+= strlen(&line[n]);   /* faster than n= strlen(line) */
   }
 
@@ -176,8 +177,8 @@ int YpEscapeSeq(const char *s, char **endp)
     int isD, isU= 0;
     val= 0;
     while ((isD= ((c= *s)<='9' && c>='0')) ||
-	   (isU= (c<='F' && c>='A')) ||
-	   (c<='f' && c>='a')) {
+           (isU= (c<='F' && c>='A')) ||
+           (c<='f' && c>='a')) {
       val<<= 4;
       if (isD) val|= c-'0';
       else if (isU) val|= c-('A'-10);
@@ -207,7 +208,7 @@ extern UnaryOp PrintC, PrintS, PrintI, PrintL, PrintF, PrintD, PrintZ,
   PrintQ, PrintP, PrintSI, PrintR, PrintVD, PrintSD, PrintFN, PrintBI,
   PrintIO;
 
-extern UnaryOp PrintXFN, PrintXBI, PrintX;
+extern UnaryOp PrintXFN, PrintXBI;
 
 int printLength= 79;   /* maximum number of characters to put on a line */
 long maxPrintLines= 5000;
@@ -251,6 +252,10 @@ void Print(void)
     /* put operand into bogus form expected by Eval (see Eval in ops3.c) */
     op.references= 0;   /* intentionally misused */
     EvalFN(&op);
+  } else if (op.ops==&auto_ops) {
+    /* put operand into bogus form expected by Eval (see Eval in ops3.c) */
+    op.references= 0;   /* intentionally misused */
+    eval_auto(&op);
   } else {
     PrintInit(YputsOut);
     op.ops->Print(&op);
@@ -270,12 +275,13 @@ void Y_print(int nArgs)
   Symbol *stack= sp-nArgs+1;
   if (CalledAsSubroutine()) {
     PrintInit(YputsOut);
-    PushDataBlock(Ref(&nilDB));
+    PushDataBlock(RefNC(&nilDB));
   } else {
     PrintInit(&PutsAsArray);
     PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
   }
   while (nArgs--) {
+    if (!stack->ops) YError("print accepts no keywords (do you mean write?)");
     stack->ops->FormOperand(stack, &op);
     if (op.ops==&functionOps) PrintXFN(&op);
     else if (op.ops==&builtinOps) PrintXBI(&op);
@@ -347,21 +353,21 @@ void PrintFunc(const char *s)
       long nhere= printLength-printNow-1;
       char movec= '\0';
       if (nhere>0) {
-	strncpy(&printBuf[printNow], s, nhere);
-	s+= nhere;
-	len-= nhere;
+        strncpy(&printBuf[printNow], s, nhere);
+        s+= nhere;
+        len-= nhere;
       } else if (nhere<0) {  /* only -1 is possible */
-	movec= printBuf[printLength-1];
+        movec= printBuf[printLength-1];
       }
       strcpy(&printBuf[printLength-1], "\\");
       if (printLines++ < maxPrintLines) RawPrinter(printBuf);
       if (nhere >= 0) {
-	printNow= 0;
-	printBuf[0]= '\0';
+        printNow= 0;
+        printBuf[0]= '\0';
       } else {
-	printNow= 1;
-	printBuf[0]= movec;
-	printBuf[1]= '\0';
+        printNow= 1;
+        printBuf[0]= movec;
+        printBuf[1]= '\0';
       }
     }
   }
@@ -669,11 +675,11 @@ static void PrintRawQ(Operand *at)
     if (*(esc= ScanForEscape(s))) {
       char *part;
       do {
-	part= GetTempBuffer(esc-s + 4);
-	strncpy(part, s, esc-s);
-	AddEscapeSeq(part + (esc-s), (int)(*esc));
-	PrintFunc(part);
-	s= esc+1;
+        part= GetTempBuffer(esc-s + 4);
+        strncpy(part, s, esc-s);
+        AddEscapeSeq(part + (esc-s), (int)(*esc));
+        PrintFunc(part);
+        s= esc+1;
       } while (*(esc= ScanForEscape(s)));
       ClearTempBuffer(0);
     }
@@ -726,7 +732,7 @@ static char *rfNames[]= { "avg:","cum:","dif:","max:","min:","mnx:","mxx:",
                 "pcen:","psum:","ptp:","rms:","sum:","uncp:","zcen:","??:" };
 
 static RangeFunc *RFs[]= { &RFavg,&RFcum,&RFdif,&RFmax,&RFmin,&RFmnx,&RFmxx,
-	            &RFpcen,&RFpsum,&RFptp,&RFrms,&RFsum,&RFuncp,&RFzcen,0 };
+                    &RFpcen,&RFpsum,&RFptp,&RFrms,&RFsum,&RFuncp,&RFzcen,0 };
 
 char *GetRFName(RangeFunc *rfTarget)
 {
@@ -796,7 +802,7 @@ static int PDRecurse(Dimension *dims, long *length, int last)
   if (dims->origin!=1L) {
     if (len>45) return -1;
     sprintf(nBuffer+len, "%ld:%ld",
-	    dims->origin, dims->origin+dims->number-1);
+            dims->origin, dims->origin+dims->number-1);
   } else {
     if (len>60) return -1;
     sprintf(nBuffer+len, "%ld", dims->number);
@@ -904,7 +910,7 @@ void PrintIO(Operand *op)
       long i= history->recNumber;
 
       sprintf(text, "  Current record is number %ld of %ld",
-	      i+1, history->nRecords);
+              i+1, history->nRecords);
       PrintFunc(text);
       ForceNewline();
 
@@ -916,20 +922,20 @@ void PrintIO(Operand *op)
       ForceNewline();
 
       if (history->time) {
-	if (history->ncyc) {
-	  sprintf(text, "  Time, cycle of current record: %.6e, %ld",
-		  history->time[i], history->ncyc[i]);
-	} else {
-	  sprintf(text, "  Time of current record: %.6e",
-		  history->time[i]);
-	}
+        if (history->ncyc) {
+          sprintf(text, "  Time, cycle of current record: %.6e, %ld",
+                  history->time[i], history->ncyc[i]);
+        } else {
+          sprintf(text, "  Time of current record: %.6e",
+                  history->time[i]);
+        }
       } else {
-	if (history->ncyc) {
-	  sprintf(text, "  Cycle of current record: %ld",
-		  history->ncyc[i]);
-	} else {
-	  strcpy(text, "  <No time or cycle available>");
-	}
+        if (history->ncyc) {
+          sprintf(text, "  Cycle of current record: %ld",
+                  history->ncyc[i]);
+        } else {
+          strcpy(text, "  <No time or cycle available>");
+        }
       }
       PrintFunc(text);
       ForceNewline();
@@ -947,7 +953,7 @@ void PrintIO(Operand *op)
    at a time.  Since IOFileLink contains several pointers, the alignment
    of an IOFileLink must be at least as strict as a void*.  */
 static MemryBlock ioflBlock= {0, 0, sizeof(IOFileLink),
-				  16*sizeof(IOFileLink)};
+                                  16*sizeof(IOFileLink)};
 
 void AddIOLink(IOFileLink** list, void *ios)
 {
@@ -981,23 +987,23 @@ char *YpReparse(void *function)
   if (position>=0) {
     YpParse(func);  /* This had better not generate any new tasks... */
     msg= MakeErrorLine(ypBeginLine,
-		       nYpIncludes? ypIncludes[nYpIncludes-1].filename : 0);
+                       nYpIncludes? ypIncludes[nYpIncludes-1].filename : 0);
     if (nYpIncludes && ypIncludes[nYpIncludes-1].file) {
       p_fclose(ypIncludes[nYpIncludes-1].file);
       ypIncludes[nYpIncludes-1].file= 0;
     }
   } else if (position<-3) {
     msg= MakeErrorLine(-4,
-		 "*****fseek failed scanning source file for function");
+                 "*****fseek failed scanning source file for function");
   } else if (position<-2) {
     msg= MakeErrorLine(-3,
-		 "*****source file changed, no longer contains function");
+                 "*****source file changed, no longer contains function");
   } else if (position<-1) {
     msg= MakeErrorLine(-2,
-		 "*****source file for function no longer exists");
+                 "*****source file for function no longer exists");
   } else {
     msg= MakeErrorLine(-1,
-		 "*****source file for function unknown");
+                 "*****source file for function unknown");
   }
   return msg;
 }
